/*
 *  Copyright 2004 by Texas Instruments Incorporated.
 *  All rights reserved. Property of Texas Instruments Incorporated.
 *  Restricted rights to use, duplicate or disclose this code are
 *  granted through contract.
 *
 *  @(#) DSP/BIOS Bridge 
 */
/*  ============================================================================
*   File    AsyncPutGetData.c
*
*   Path    $(TESTSUITEROOT)\Analysis\AsyncPutGetData
*
*   Desc:
*      Host initializes and sends a data buffer on the output stream. 
*	   DSP sends the same buffer on the DSP's output stream.Demonstrates
*      data streaming using n-way buffering to/from a DSP program, and 
*      end-of-stream handling.
*
*   Rev     1.0.0
*
*   This computer program is copyright to Texas Instruments Incorporated.
*   The program may not be used without the written permission
*   of Texas Instruments Incorporated. or against the terms and conditions
*   stipulated in the agreement under which this program has been
*   supplied.
*
*   (c) Texas Instruments Incorporated 2003
*
*   ============================================================================
*   Revision History
*
*   SEP 15, 2002  REF=ORG            Raghunandan NagarajaRao.
*      Original Version.
*   ============================================================================
*/

#include <stdio.h> 
#include <getopt_test.h>
#include <dbapi.h>
#include <tst.h>
#include <tstlog.h>
#include <DSPProcessor_OEM.h>
#include <testosal.h>
#include <string.h>
#include <semaphore.h>
#include "AsyncPutGetData.h"

static NODEARGS		NodeArgs;
              
/* 
 *  ======== putGetData ========
 */
int asyncPutGetData(int argc, char ** argv)
{
    STRMCOPY_TASK   strmcopyTask;
    DSP_STATUS      status = DSP_SOK;
    fLoadDsp 	    = FALSE;

    /* Process command line arguments, open data files: */
    status = ProcessArgs(argc, argv);
   
    strmcopyTask.hProcessor = NULL;
    strmcopyTask.hNode = NULL;
    strmcopyTask.hOutStream = NULL;
    strmcopyTask.hInStream = NULL;

    strmcopyTask.ppInBufs = NULL;
    strmcopyTask.ppOutBufs = NULL;

    /* Attach to processor */
    struct DSP_PROCESSORINFO       dspInfo;
    UINT                    numProcs;
    UINT                    index=0;
    INT                     procId=0;
    
    while (DSP_SUCCEEDED(DSPManager_EnumProcessorInfo(index,&dspInfo, (UINT) sizeof (struct DSP_PROCESSORINFO),&numProcs))) {
        if (dspInfo.uProcessorType == DSPTYPE_55 || dspInfo.uProcessorType == DSPTYPE_64) {
            TSTLOG_appendLog(pHandle,"DSP device detected !! \n");
            procId = index;
            status = DSP_SOK;
            break;
        }
        index++;
    }

    /* Attach to an available DSP (in this case, the 1st DSP). */
    if (DSP_SUCCEEDED(status)) {
        
        status = DSPProcessor_Attach(procId, NULL,&strmcopyTask.hProcessor);
        if (DSP_SUCCEEDED(status)) {
            TSTLOG_appendLog(pHandle, "DSPProcessor_Attach succeeded.\n");
        }
        else {
            TSTLOG_appendLog(pHandle, "DSPProcessor_Attach failed. Status = 0x%x\n",status);
        }
    }
    else {
        TSTLOG_appendLog(pHandle, "Failed to get the desired processor \n");
    }
    

    if (DSP_SUCCEEDED(status)) {
        /* Perform processor level initialization. */
        if(fLoadDsp)
			status = InitializeProcessor(&strmcopyTask);
    }

    if (DSP_SUCCEEDED(status)) {
        
        /* Perform node level initialization. */
        status = InitializeNode(&strmcopyTask);
        if (DSP_SUCCEEDED(status)) {
            
            /* Perform stream level initialization. */
            status = InitializeStreams(&strmcopyTask);
            if (DSP_SUCCEEDED(status)) {
                
                /* Run task. */
                status = RunTask(&strmcopyTask);
                if (DSP_FAILED(status)) 
				{
						


					if(fOpenLog)
						TSTLOG_appendLog(pHandle, "RunTask failed.\n");
                }
            }
        }
    }
    
    /* Close opened files. */
    CleanupStreams(&strmcopyTask);
    CleanupNode(&strmcopyTask);
    CleanupProcessor(&strmcopyTask);
    return (DSP_SUCCEEDED(status) ? 0 : -1);
}


/*
 *  ======== InitializeProcessor ========
 *  Perform processor related initialization.
 */
static DSP_STATUS InitializeProcessor(STRMCOPY_TASK *copyTask)
{
    DSP_STATUS          status = DSP_SOK;
	INT					iArgc = 1;

	if (DSP_SUCCEEDED(status)) 
	{
			status = DSPProcessor_Load(copyTask->hProcessor, iArgc, (CONST CHAR **)aArgv, (CONST CHAR **)aEnvp);  

		if (DSP_SUCCEEDED(status)) 
		{
				status = DSPProcessor_Start(copyTask->hProcessor);
				if (DSP_FAILED(status)) 
				{
					if(fOpenLog)
					{
						TSTLOG_appendLog(pHandle, "Error in DSPProcessor Start\n status = 0x%x\n", status);    
					}
				}
		}
		else
		{
				if(fOpenLog)
				{
					TSTLOG_appendLog(pHandle,"Error in DSPProcessor_Load. STATUS = %x\n", status );
				}
		}
	}
	return (status);
}


/*
 *  ======== InitializeNode ========
 *  Perform node related initialization.
 */
static DSP_STATUS InitializeNode(STRMCOPY_TASK *copyTask)
{
    BYTE            argsBuf[ARGSIZE + sizeof(ULONG)];
    struct DSP_CBDATA     *pArgs;
    struct DSP_NODEATTRIN  nodeAttrIn;
    struct DSP_STRMATTR    attrs;
    struct DSP_UUID        uuid;
    DSP_STATUS      status = DSP_SOK;
    uuid = inoutuuid;

    attrs.uBufsize = NodeArgs.iBufferSize;
	attrs.uNumBufs = NUMBUFS;
    attrs.uAlignment = 0;
    attrs.uTimeout = DSP_FOREVER;
	switch(NodeArgs.iMode)
	{
		case STRMMODE_PROCCOPY : 
			attrs.lMode = STRMMODE_PROCCOPY; 
			break;
		case STRMMODE_ZEROCOPY : 
			attrs.lMode = STRMMODE_ZEROCOPY; 
			break;
		case STRMMODE_RDMA : 
			attrs.lMode = STRMMODE_RDMA; 
			break;
		default:
			attrs.lMode = STRMMODE_PROCCOPY; 
			break;
	}

  	copyTask->nStrmMode = attrs.lMode ;

    if (copyTask->nStrmMode == STRMMODE_ZEROCOPY)
    {
            attrs.uSegid = DSP_SHMSEG0;  /* Use SM for zero-copy */
    }
    else 
    {
            attrs.uSegid = DEFAULTSEGMENT;
    }
    
    pArgs = (struct DSP_CBDATA *)argsBuf;
    pArgs->cbData = ARGSIZE;
    nodeAttrIn.uTimeout = DSP_FOREVER;
    nodeAttrIn.iPriority = 5;

#if defined(OMAP_2430)|| defined(OMAP_3430)

    nodeAttrIn.uProfileID = 0xff; 
#endif
    /* Allocate the strmcopy node. */
    if (DSP_SUCCEEDED(status)) {
        status = DSPNode_Allocate(copyTask->hProcessor, &uuid, pArgs,
                &nodeAttrIn, &copyTask->hNode);
        if (DSP_FAILED(status)) 
        {
			if(fOpenLog)
				    TSTLOG_appendLog(pHandle, "DSPNode_Allocate failed. Status = 0x%x\n", status);
        }
    }
    
    /* Connect the strmcopy node to the GPP. */
    if (DSP_SUCCEEDED(status)) 
    {
        status = DSPNode_Connect((DSP_HNODE)DSP_HGPPNODE, 0, copyTask->hNode,
                0, &attrs);
        if (DSP_FAILED(status)) {
			if(fOpenLog)
				TSTLOG_appendLog(pHandle, "DSPNode_Connect failed. Status = 0x%x\n",
                    status);
        }
    }

    if (DSP_SUCCEEDED(status)) {
        status = DSPNode_Connect(copyTask->hNode, 0, (DSP_HNODE)DSP_HGPPNODE,
            0, &attrs);
        if (DSP_FAILED(status)) {
			if(fOpenLog)
				TSTLOG_appendLog(pHandle, "DSPNode_Connect failed. Status = 0x%x\n",
                    status);
        }
    }

    /* Create the strmcopy node on the DSP. */
    if (DSP_SUCCEEDED(status)) {
        status = DSPNode_Create(copyTask->hNode);
        if (DSP_FAILED(status)) 
        {
			if(fOpenLog)
				TSTLOG_appendLog(pHandle, "DSPNode_Create failed. Status = 0x%x\n",
                status);
        }
    }
    
    /* Start the strmcopy node on the DSP. */
    if (DSP_SUCCEEDED(status)) {
        status = DSPNode_Run(copyTask->hNode);
        if (DSP_FAILED(status)) 
        {
			if(fOpenLog)
				TSTLOG_appendLog(pHandle, "DSPNode_Run failed. Status = 0x%x\n",
                status);
        }
    }
    
    return (status);
}


/*
 *  ======== InitializeStreams ========
 *  Perform stream related initialization.
 */
static DSP_STATUS InitializeStreams(STRMCOPY_TASK  *copyTask)
{
    DSP_STATUS      status = DSP_SOK;
 
    
    struct DSP_STREAMATTRIN    attrs;

    attrs.cbStruct = sizeof(struct DSP_STREAMATTRIN);
    attrs.uTimeout = DSP_FOREVER;
    attrs.uAlignment = 0;
    attrs.uNumBufs = NUMBUFS;
    attrs.lMode = copyTask->nStrmMode;

    if (copyTask->nStrmMode == STRMMODE_PROCCOPY)
    {
    	attrs.uSegment = DEFAULTSEGMENT;
    }
    else
    {
	    attrs.uSegment = DSP_SHMSEG0;    /* Use shared memory segment 0 */
    }
    
    /* Open an input data stream from the host to the strmcopy node. */
    if (DSP_SUCCEEDED(status)) 
    {
        status = DSPStream_Open(copyTask->hNode, DSP_TONODE, 0, 
        &attrs, &copyTask->hOutStream);
        if (DSP_FAILED(status)) 
        {
			if(fOpenLog)
				TSTLOG_appendLog(pHandle, 
                "DSPStream_Open of input stream failed. Status = 0x%x\n",
                status);
        }
    }
    

    /* Open an output stream from the copy node to the host. */
    if (DSP_SUCCEEDED(status)) 
    {
        status = DSPStream_Open(copyTask->hNode, DSP_FROMNODE, 0, 
            &attrs, &copyTask->hInStream);
        if (DSP_FAILED(status)) 
        {
			if(fOpenLog)
				TSTLOG_appendLog(pHandle, 
                "DSPStream_Open of output stream failed. Status = 0x%x\n",
                status);
        }
    }
    
    return (status);
}


/*
 *  ======== RunTask ========
 *  Run strmcopy task.
 */
static DSP_STATUS RunTask(STRMCOPY_TASK *copyTask)
{
			DSP_STATUS      status = DSP_SOK;
			BYTE            *pInBuf = NULL;
			BYTE            *pOutBuf = NULL;
            
			DWORD           dwArg = 0;
			DWORD           dwBufSize = NodeArgs.iBufferSize;
			ULONG           ulBufsize = NodeArgs.iBufferSize;
            ULONG           ulBytes;
			INT             i = 0;

			DWORD			dwStartTime;  
			DWORD			dwElapsedTime;
			DWORD			dwEndTime;
			ULONG			ulNumBytesTransferred = 0;
			ULONG			ulBytesPerSec = 0;


  copyTask->ppInBufs = (BYTE**)OsSpecificMemAlloc(sizeof(BYTE*) * NUMBUFS);
   /* Allocate stream buffers */
    if (DSP_SUCCEEDED(status)) 
    {
        status = DSPStream_AllocateBuffers(copyTask->hInStream, 
            dwBufSize, copyTask->ppInBufs, NUMBUFS);
        if (DSP_FAILED(status)) 
        {
            TSTLOG_appendLog(pHandle, 
                "DSPStream_AllocateBuffers for input stream failed. "
                "Status = 0x%x\n", status);
        }
    }
    
    copyTask->ppOutBufs = (BYTE**)OsSpecificMemAlloc(sizeof(BYTE*) * NUMBUFS);
    /* Allocate and issue buffers to output data stream. */
    if (DSP_SUCCEEDED(status)) 
    {
        status = DSPStream_AllocateBuffers(copyTask->hOutStream, 
            dwBufSize, copyTask->ppOutBufs, NUMBUFS);
        if (DSP_FAILED(status)) 
        {
            TSTLOG_appendLog(pHandle, "DSPStream_AllocateBuffers for output stream "
                "failed Status = 0x%x\n", status);
        }
    }
    

    /* Issue some buffers */
    for (i = 0; i < NUMBUFS  ; i++) {
        pInBuf = copyTask->ppInBufs[i];
        pOutBuf = copyTask->ppOutBufs[i];

		((SHORT *)pOutBuf)[0] = ulBufsize / 2;
		((SHORT *)pOutBuf)[1] = ulBufsize / 2;
		((SHORT *)pOutBuf)[2] = NodeArgs.iNumTxfrs & 0xffff ;
		((SHORT *)pOutBuf)[3] = NodeArgs.iNumTxfrs >> 16 ;


        if (DSP_SUCCEEDED(status)) 
        {
            status = DSPStream_Issue(copyTask->hOutStream, pOutBuf, ulBufsize,ulBufsize,  0);
            if (DSP_FAILED(status)) 
            {
                TSTLOG_appendLog(pHandle, "DSPStream_Issue failed: 0x%x\n", status);
                break;
            }
        }
        
        if (DSP_SUCCEEDED(status)) {
            status = DSPStream_Issue(copyTask->hInStream, pInBuf, ulBufsize, ulBufsize, 0);
            if (DSP_FAILED(status)) {
                TSTLOG_appendLog(pHandle, "DSPStream_Issue failed: 0x%x\n", status);
                break;
            }
        }

       
    }


    TST_verbose("Please wait....\n");
	OsSpecificTimerStart(&dwStartTime);
   
	/* Process for a fixed number of loops */
    for (i = 0; i < NodeArgs.iNumTxfrs; i++) 
    {

        
        /* Reclaim output buffer */
        status = DSPStream_Reclaim(copyTask->hOutStream, &pOutBuf, &ulBytes, NULL,&dwArg);
        if (DSP_FAILED(status)) 
        {
            TSTLOG_appendLog(pHandle, "DSPStream_Reclaim O failed: 0x%x\n", status);
            break;
        }
        
        /* Reclaim input buffer */
        status = DSPStream_Reclaim(copyTask->hInStream, &pInBuf, &ulBytes,NULL, &dwArg);
        if (DSP_FAILED(status)) 
        {
            TSTLOG_appendLog(pHandle, "DSPStream_Reclaim I failed: 0x%x\n", status);
            break;
        }

        status = DSPStream_Issue(copyTask->hOutStream, pOutBuf, ulBufsize,ulBufsize,  0); 
        if (DSP_FAILED(status)) 
        {
            TSTLOG_appendLog(pHandle, "DSPStream_Issue failed.\n");
            break;
        }


        /* Issue buffer to the input stream */
        status = DSPStream_Issue(copyTask->hInStream, pInBuf, ulBufsize,ulBufsize, 0); 
        if (DSP_FAILED(status)) 
        {
            TSTLOG_appendLog(pHandle, "DSPStream_Issue failed.\n");
            break;
        }
        

#if 0
		if ((i % 500) == 0)
        {
			TST_verbose("Issue Reclaim %d\n",i) ;
        }
#endif

    }

    OsSpecificTimerStop(&dwEndTime);


    /* Protocol requirement - Issue on output stream, otherwise DSP will block in reclaim */
    status = DSPStream_Reclaim(copyTask->hOutStream, &pOutBuf, &ulBytes, NULL,&dwArg);
    if (DSP_FAILED(status)) 
    {
        TSTLOG_appendLog(pHandle, "DSPStream_Reclaim O failed: 0x%x\n", status);
        
    }
    
    if (DSP_SUCCEEDED(status)) 
    {
        status = DSPStream_Issue(copyTask->hOutStream, pOutBuf, ulBufsize,ulBufsize,  0);
        if (DSP_FAILED(status)) 
        {
            TSTLOG_appendLog(pHandle, "DSPStream_Issue failed: 0x%x\n", status);
        
        }
    }
    
    status = DSPStream_Reclaim(copyTask->hOutStream, &pOutBuf, &ulBytes, NULL,&dwArg);
    if (DSP_FAILED(status)) 
    {
        TSTLOG_appendLog(pHandle, "DSPStream_Reclaim O failed: 0x%x\n", status);
        
    }

    /* Reclaim input buffer */
        status = DSPStream_Reclaim(copyTask->hInStream, &pInBuf, &ulBytes,NULL, &dwArg);
    if (DSP_FAILED(status)) 
    {
            TSTLOG_appendLog(pHandle, "DSPStream_Reclaim I failed: 0x%x\n", status);
        }
		
	OsSpecificTimerDifference(&dwEndTime, &dwStartTime, &dwElapsedTime);
	if(DSP_SUCCEEDED(status))
    {
		ulNumBytesTransferred = dwBufSize * NodeArgs.iNumTxfrs * 2;
		ulBytesPerSec = (ulNumBytesTransferred / dwElapsedTime) * 1000;

        if(fOpenLog)
		{
			TSTLOG_appendLog(pHandle, "LOG: ***********************************\n");
            		TSTLOG_appendLog(pHandle, "LOG: *********ISSUE RECLAIM STATISTICS**********\n");
            		TSTLOG_appendLog(pHandle, "LOG: FINAL STATUS = PASS\n");
            		TSTLOG_appendLog(pHandle, "LOG: BufferSize = %d\n", dwBufSize);
            		TSTLOG_appendLog(pHandle, "LOG: StartTime = %u\n", dwStartTime);
            		TSTLOG_appendLog(pHandle, "LOG: EndTime = %u\n", dwEndTime);
            		TSTLOG_appendLog(pHandle, "LOG: ElapsedTime = %d\n", dwElapsedTime);
            		TSTLOG_appendLog(pHandle, "LOG: Throughput = %d Bytes/sec\n", ulBytesPerSec);
            		TSTLOG_appendLog(pHandle, "LOG: ulNumBytesTransferred = %d\n", ulNumBytesTransferred);
            		TSTLOG_appendLog(pHandle, "LOG: **********END STATISTICS***********\n");
            		TSTLOG_appendLog(pHandle, "LOG: ***********************************\n");
			
		}


    }
    else
    {
        TSTLOG_appendLog(pHandle, "LOG: BufferSize = %d\n", dwBufSize);
        TSTLOG_appendLog(pHandle, "LOG: status = 0x%x\n", status); 
    }
	return (status);
}

/*
 *  ======== CleanupStreams ========
 *  Perform stream related cleanup.
 */ 
static DSP_STATUS CleanupStreams(STRMCOPY_TASK *copyTask)
{
    struct DSP_STREAMINFO  streamInfo;
    DSP_STATUS      status = DSP_SOK;
    DWORD           dwArg = 0;
    DWORD           dwBufsize = NodeArgs.iBufferSize;
    int             i = 0;
    BYTE *pOutBuf, *pInBuf;


	sleep(5); // let the streams settle downs 
    /* Reclaim and free stream buffers. */
    if (copyTask->hOutStream) 
    {
        DSPStream_Idle(copyTask->hOutStream, TRUE);
        status = DSPStream_GetInfo(copyTask->hOutStream, &streamInfo,
            sizeof(streamInfo));

        if (copyTask->ppOutBufs) 
        {
            // Reclaim the buffer if it's still in the stream 
            if (streamInfo.uNumberBufsInStream) 
            {
                for( i =0; i<streamInfo.uNumberBufsInStream; i++)
                    status = DSPStream_Reclaim(copyTask->hOutStream,
                        &pOutBuf, &dwBufsize, NULL, &dwArg);
				if (DSP_FAILED(status))
				{
					TST_verbose("DSPStream_Reclaim Failed 0x%x\n",status) ;
				}
            }
            status = DSPStream_FreeBuffers(copyTask->hOutStream, 
                    copyTask->ppOutBufs, NUMBUFS);
            if (DSP_FAILED(status)) 
            {
				if(fOpenLog)
					TSTLOG_appendLog(pHandle,  "DSPStream_FreeBuffer of input buffer failed. "
                    "Status = 0x%x\n", status);
            }

            OsSpecificMemFree(copyTask->ppOutBufs);
            copyTask->ppOutBufs = NULL;
            // Close open streams. 
            status = DSPStream_Close(copyTask->hOutStream);
            if (DSP_FAILED(status)) 
            {
				if(fOpenLog)
					TSTLOG_appendLog(pHandle, "DSPStream_Close of input stream failed. "
                    "Status = 0x%x\n", status);
            }

            copyTask->hOutStream = NULL;
        }
    }

    if (copyTask->hInStream) 
    {
        DSPStream_Idle(copyTask->hInStream, TRUE);

        /* Need to determine the number of buffers left in the stream */
         if (copyTask->ppInBufs) 
         {
            status = DSPStream_GetInfo(copyTask->hInStream, &streamInfo,
               sizeof(streamInfo));
            if (streamInfo.uNumberBufsInStream) 
            {
            for( i =0; i<streamInfo.uNumberBufsInStream; i++)
                   status = DSPStream_Reclaim(copyTask->hInStream, 
                        &pInBuf, &dwBufsize, NULL, &dwArg);
				if (DSP_FAILED(status))
				{
					TST_verbose("DSPStream_Reclaim Failed 0x%x\n",status) ;
				}
            }

            /* Free buffer. */
            status = DSPStream_FreeBuffers(copyTask->hInStream, 
                copyTask->ppInBufs, NUMBUFS);
            if (DSP_FAILED(status)) 
            {
				if(fOpenLog)
					TSTLOG_appendLog(pHandle, 
                    "DSPStream_FreeBuffer of output buffer failed. "
                    "Status = 0x%x\n", status);
            }
            OsSpecificMemFree(copyTask->ppInBufs);
            copyTask->ppInBufs = NULL;
        }

        status = DSPStream_Close(copyTask->hInStream);
        if (DSP_FAILED(status)) {
			if(fOpenLog)
				TSTLOG_appendLog(pHandle, "DSPStream_Close of output stream failed . "
                "Status = 0x%x\n", status);
        }

        copyTask->hInStream = NULL;
    }
    
    return (status);
}


/*
 *  ======== CleanupNode ========
 *  Perform node related cleanup.
 */ 
static DSP_STATUS CleanupNode(STRMCOPY_TASK *copyTask)
{
    DSP_STATUS      status = DSP_SOK;
    DSP_STATUS      exitStatus;
    
    if (copyTask->hNode) {

        // Terminate DSP node.
        status = DSPNode_Terminate(copyTask->hNode, &exitStatus);
        if (DSP_FAILED(status)) 
        {
			if(fOpenLog)
				TSTLOG_appendLog(pHandle, "DSPNode_Terminate failed: 0x%x\n", status);
        }

        /* Delete DSP node. */
        status = DSPNode_Delete(copyTask->hNode);
        if (DSP_FAILED(status)) 
        {
			if(fOpenLog)
				TSTLOG_appendLog(pHandle, "DSPNode_Delete failed: 0x%x\n", status);
        }

        copyTask->hNode = NULL;
    }

    return (status);
}


/*
 *  ======== CleanupProcessor ========
 *  Perform processor related cleanup.
 */ 
static DSP_STATUS CleanupProcessor(STRMCOPY_TASK *copyTask)
{
    DSP_STATUS      status = DSP_SOK;

    if (copyTask->hProcessor) {
        /* Detach from processor. */
        status = DSPProcessor_Detach(copyTask->hProcessor);
        if (DSP_FAILED(status)) 
        {
			if(fOpenLog)
				TSTLOG_appendLog(pHandle, "DSPProcessor_Detach failed.\n");
        }

        copyTask->hProcessor = NULL;
    }

    return (status);
}


/* 
 *  ======== ProcessArgs ========
 *  Process command line arguments for this sample, returning input and
 *  output file handles.
 */
static DSP_STATUS ProcessArgs(int argc, char ** argv)
{
    DSP_STATUS      status = DSP_SOK;
	INT             opt = 0;
	NodeArgs.iSegmentId = DEFAULTSEGMENT;
	NodeArgs.iMode  = STRMMODE_PROCCOPY;
	optind = 0;

        while ((opt = getopt_test(argc, (char * const)argv, "+vl:x:b:n:ZPDR:")) != EOF) {
        switch (opt) {
         case 'v':
            /* verbose */
            TST_setVerbose(TRUE);
			break;

	      case 'l':
            /* use log and store with name location */
            strncpy(szLogName, optarg, sizeof(szLogName));
            pHandle = TSTLOG_openLog(szLogName);
			fOpenLog = TRUE;
	        break;

          case 'x':
            /* use given executable */
            aArgv[0] = (char *) optarg;
			fLoadDsp = TRUE;
			break;

		case 'b':
            /* set Buffer Size */
            NodeArgs.iBufferSize = CSL_Atoi(optarg);
			break;
        case 'n':
            /* set Number of transfers */
            NodeArgs.iNumTxfrs = CSL_Atoi(optarg);
			break;
		case 'Z':
			/* set Transfer Mode as STRMMODE_ZEROCOPY */
			NodeArgs.iMode = STRMMODE_ZEROCOPY;
			NodeArgs.iSegmentId = DSP_SHMSEG0;
			break;
		case 'D':
			/* set Transfer Mode as STRMMODE_LDMA */
			NodeArgs.iMode = STRMMODE_LDMA;
			break;
		case 'R':
			/* set Transfer Mode as STRMMODE_RDMA */
			NodeArgs.iMode = STRMMODE_RDMA;
			break;
		
        default:
            TSTLOG_appendLog(pHandle, "Invalid option: %c\n", opt);
            break;
        }
    }
		return (status);
}


