/*
*  Copyright 2004 by Texas Instruments Incorporated.
*  All rights reserved. Property of Texas Instruments Incorporated.
*  Restricted rights to use, duplicate or disclose this code are
*  granted through contract.
*
*  @(#) DSP/BIOS Bridge 
*/
/*  ============================================================================
*   File    WaitForEvent.c
*
*   Path    $(TESTSUITEROOT)\Behaviour\Event\
* 
*   Desc:
*           Behaviour test for DSPManager_WaitForEvent API.
*
*
*   Usage:
*
*        WAITEVENT [-v] [-l "log path/filename"] 
*                      [-x "valid executable"] ..
*
*        -v: verbose mode
*        -l: use log and location
*        -x: DSP Image to be used
*
*   ---------------------------------------------------------------------------
*   Rev     1.0.0
* 
*   This computer program is copyright to Texas Instruments Incorporated.
*   The program may not be used without the written permission
*   of Texas Instruments Incorporated. or against the terms and conditions
*   stipulated in the agreement under which this program has been
*   supplied.
*
*   (c) Texas Instruments Incorporated 2003
*
*   ============================================================================
*   Revision History
*
*   NOV 14, 2002    REF=ORG             Swamy MC
*      Original Version.
*   ============================================================================
*/


#include <stdio.h>
#include <string.h>
#include <dbapi.h>
#include <getopt_test.h>
#include <tst.h>
#include <tstlog.h>
#include <uuidutil.h>
#include <testosal.h>
#include "waitforevent.h"
#include <DSPProcessor_OEM.h>

/*
* Function waitForEvent_BEHAVIOUR
*/
INT waitForEvent_BEHAVIOUR(INT argc, CHAR * argv[])
{
    /* Initialise Processor and Node
    * Create the objects needed for Events
    * Register the Processor and Node related Events
    * Call Wait for Events with the list of Notification objects and the mask
    * Based on the type of the event, take appropriate action
    */
    
    BOOL           bUseLog=FALSE;
    FILE           *pHandle = NULL;
    INT            iNumTransfers;
    INT            iBuffSize;
    CHAR           szLogName[DEFNAMESIZE];
    INT            iMandArgCount = NUMOFCOMPARGS;
    DSP_STATUS     status;
    TASK_CONTEXT   contextTask;
    CHAR           *aArgv=DSPEXEC;
    CHAR           *strmArgv=STRMDSPEXEC;
    INT            opt;
    
    
    contextTask.notifications = (struct DSP_NOTIFICATION* *)OsSpecificMemAlloc(NUMNOTIFOBJECTS * sizeof(struct DSP_NOTIFICATION*));
    contextTask.strmnotifications = (struct DSP_NOTIFICATION* *)OsSpecificMemAlloc(3 * sizeof(struct DSP_NOTIFICATION*));
    
    /* Process arguments and Store the values */
    optind = 0;
    opt = getopt_test(argc, (char *const)argv, "+vl:x:y:n:s:");
    while (opt != EOF) {
        switch (opt) {
        case 'v':
            /* verbose */
            TST_setVerbose(TRUE);
            break;
        case 'l':
            /* use log and store with name location */
            strncpy(szLogName, optarg, sizeof(szLogName));
            bUseLog = TRUE;
            pHandle = TSTLOG_openLog(szLogName);
            contextTask.pHandle = pHandle;
            break;
        case 'x':     
            /* use given executable */
            aArgv = (char *) optarg;
            break; 
        case 'y':     
            /* use given executable */
            strmArgv = (char *) optarg;
            break; 
        case 'n':
            /* Number of Transfers */
            iNumTransfers = CSL_Atoi(optarg);
            break;
        case 's':
            /* Buffer Size */
            iBuffSize = CSL_Atoi(optarg);
            break;
        default:
            TST_reportError(("Invalid option: %c\n"), opt);
            if (bUseLog) {
                TSTLOG_appendLog(pHandle,
                    "Invalid option specified.\n");
            }
            break;
        }
        opt = getopt_test(argc, (char *const)argv, "+vl:x:y:n:s:");
    }
    
    /* If all the Mandatory args are not present, abort */
    if (iMandArgCount != 0)
    {
        // Error
        TST_verbose("Number of Mandatory Arguments is incorrect.\n");
        TST_verbose("Some args may be missing\n");
        TSTLOG_appendLog(pHandle, "FAIL : ");
        TSTLOG_appendLog(pHandle,"Number of Mandatory Arguments is incorrect.\n");
        if (bUseLog) 
        {
            TSTLOG_closeLog(pHandle);
        } 
        return 0;
    }
    
    /* Start The actual Test Case */
    status = DspManager_Open(0, NULL);
    contextTask.hProcessor = NULL;
    contextTask.hNode = NULL;
    contextTask.hInStream = NULL;
    contextTask.hOutStream = NULL;
    contextTask.ppInBufs = NULL;
    contextTask.ppOutBufs = NULL;
    contextTask.nStrmMode = STRMMODE_PROCCOPY;
    if (DSP_SUCCEEDED(status)) 
    {
        /* Perform processor level initialization. */
        status = checkProcessorEvents(&contextTask, aArgv);
    }
    
    if (DSP_SUCCEEDED(status)) 
    {
        /* Perform node level initialization. */
        status = checkNodeEvents(&contextTask);
        /* Perform stream level initialization. */
        if (DSP_SUCCEEDED(status)) 
        {
            status = checkStreamEvents(&contextTask,strmArgv);
        }
    }
    CleanupStreams(&contextTask);
    CleanupNode(&contextTask);
    CleanupProcessor(&contextTask);
    
    DspManager_Close(0,NULL);
    
    TSTLOG_appendLog(pHandle, "LOG: ***********************************\n");
    TSTLOG_appendLog(pHandle, "LOG: *********EVENTWAIT  STATISTICS*****\n");
    TSTLOG_appendLog(pHandle, "LOG: FINAL STATUS = 0x%x\n", status);
    TSTLOG_appendLog(pHandle, "LOG: **********END STATISTICS***********\n");
    TSTLOG_appendLog(pHandle, "LOG: ***********************************\n");
    
    return (status);
    
} /* End of waitForEvent_BEHAVIOUR */



    /*
    *  ======== checkProcessorEvents ========
    *  Perform processor related initialization.
    */
static DSP_STATUS checkProcessorEvents(TASK_CONTEXT *copyTask,CHAR *aArgv)
{  
    DSP_STATUS      status = DSP_SOK;
    INT             argc = 1 ;
    UINT            index = 100;
    FILE           *pHandle = copyTask->pHandle;
    struct DSP_PROCESSORINFO       dspInfo;
    UINT                    numProcs;
    UINT                    pindex=0;
    INT                     procId;

    while (DSP_SUCCEEDED(DSPManager_EnumProcessorInfo(pindex,&dspInfo,(UINT) sizeof (struct DSP_PROCESSORINFO),&numProcs)))
    {
      if (dspInfo.uProcessorType == DSPTYPE_55 || dspInfo.uProcessorType == DSPTYPE_64)
      {
          TSTLOG_appendLog(pHandle,"DSP device detected !! \n");
          procId = pindex;
          status = DSP_SOK;
          break;
      }
      pindex++;
    }

    /* Attach to an available DSP (in this case, the 1st DSP). */
    if (DSP_SUCCEEDED(status)) {
      status = DSPProcessor_Attach(procId,NULL,&copyTask->hProcessor);
      if (DSP_SUCCEEDED(status)) {
          TSTLOG_appendLog(pHandle, "DSPProcessor_Attach succeeded.\n");
      }
      else {
          TSTLOG_appendLog(pHandle, "DSPProcessor_Attach failed. Status = 0x%x\n",status);
      }
    }
    else
      TSTLOG_appendLog(pHandle, "Failed to get the desired processor \n");


    if (DSP_SUCCEEDED(status))
    {
        copyTask->procEventMask = DSP_PROCESSORSTATECHANGE;
        TSTLOG_appendLog(pHandle,"Event Mask = 0x%x\n",copyTask->procEventMask);
        copyTask->notifyMask = DSP_SIGNALEVENT;

        copyTask->notifications[0] = OsSpecificMemAlloc(sizeof(struct DSP_NOTIFICATION));

        /* Register for Notifs for Proc State */
        status = DSPProcessor_RegisterNotify(copyTask->hProcessor,copyTask->procEventMask,
          copyTask->notifyMask, copyTask->notifications[0]);

        copyTask->procEventMask = DSP_PROCESSORATTACH;
        TSTLOG_appendLog(pHandle, "Event Mask = 0x%x\n",copyTask->procEventMask);
        copyTask->notifyMask = DSP_SIGNALEVENT;

        copyTask->notifications[1] = OsSpecificMemAlloc(sizeof(struct DSP_NOTIFICATION));

        /* Register for Notifs for Proc State */
        status = DSPProcessor_RegisterNotify(copyTask->hProcessor,copyTask->procEventMask,
          copyTask->notifyMask, copyTask->notifications[1]);
    }

    if (DSP_SUCCEEDED(status))
    {
      status = DSPProcessor_Load(copyTask->hProcessor,argc,(CONST CHAR **)&aArgv,(CONST CHAR **)NULL) ;
  
      if (DSP_FAILED(status))
      {
          TSTLOG_appendLog(pHandle, "DSPProcessor_Load failed.\n") ;
      }
  
      index = 100;
      status = DSPManager_WaitForEvents(copyTask->notifications, TWOEVENTS, &index, 10000);
      if (!DSP_SUCCEEDED(status))
      {
          TSTLOG_appendLog(pHandle, "ERROR: WaitForEvents failed for DSPProcessor_Load status = 0x%x index = 0x%x \n",status,index);
          return(status);
      }
      else
      {
          TSTLOG_appendLog(pHandle, "WaitForEvents PASS for DSPProcessor_Load status = 0x%x index = 0x%x \n",status,index);
      }
      status = DSPProcessor_Start(copyTask->hProcessor);
  
      if (DSP_FAILED(status))
      {
          TSTLOG_appendLog(pHandle, "DSPProcessor_Start failed.\n") ;
      }
      index = 100;
      status = DSPManager_WaitForEvents(copyTask->notifications, TWOEVENTS, &index, 10000);
      if (DSP_FAILED(status))
      {
          TSTLOG_appendLog(pHandle, "ERROR: WaitForEvents failed for DSPProcessor_Load.status = 0x%x index = 0x%x \n",status,index);
          return(status);
      }
      else
      {
          TSTLOG_appendLog(pHandle, "WaitForEvents PASS for DSPProcessor_Start status = 0x%x index = 0x%x \n",status,index);
      }
  
      status = DSPManager_WaitForEvents(copyTask->notifications, TWOEVENTS, &index, 100);
      if (status == DSP_ETIMEOUT)
      {
          /* This was the expected return code, so reset the status value */
          status = DSP_SOK;
      }
  
    }

    return (status);
}


/*
*  ======== checkNodeEvents ========
*  Perform node related initialization.
*/
static DSP_STATUS checkNodeEvents(TASK_CONTEXT *copyTask)
{
    BYTE            argsBuf[ARGSIZE + sizeof(ULONG)];
    struct DSP_CBDATA      *pArgs;
    struct DSP_NODEATTRIN  nodeAttrIn;
    struct DSP_STRMATTR    attrs;
    struct DSP_UUID        uuid;
    DSP_STATUS      status = DSP_SOK;
    UINT            index = 100;
    CHAR            *pszUuid;
    struct DSP_MSG         testMsg;
    UINT            timeout;
    FILE            *pHandle = copyTask->pHandle;
    
    pszUuid = "F95CA077_EE7D_11D5_A8DD_00B0D055F6D1";  // MSG
    UUID_UuidFromString((CHAR *)pszUuid, &uuid);
    
    
    attrs.uBufsize = DEFAULTBUFSIZE;
    attrs.uNumBufs = DEFAULTNBUFS;
    attrs.uAlignment = 0;
    attrs.uTimeout = DSP_FOREVER;
    attrs.lMode = copyTask->nStrmMode;
    if (copyTask->nStrmMode == STRMMODE_ZEROCOPY) {
        attrs.uSegid = DSP_SHMSEG0;  /* Use SM for zero-copy */
    }
    else 
    {
        attrs.uSegid = DEFAULTSEGMENT;
    }
    
    nodeAttrIn.uTimeout = 10000;
    nodeAttrIn.iPriority = 5;
#if defined(OMAP_2430)|| defined(OMAP_3430)
    nodeAttrIn.uProfileID = 0xff;
#endif
    
    pArgs = (struct DSP_CBDATA *)argsBuf;
    pArgs->cbData = ARGSIZE;
    
    /* Allocate the strmcopy node. */
    if (DSP_SUCCEEDED(status)) 
    {
        status = DSPNode_Allocate(copyTask->hProcessor, &uuid, pArgs,
            &nodeAttrIn, &copyTask->hNode);
        if (DSP_FAILED(status)) 
        {
            TSTLOG_appendLog(pHandle, "DSPNode_Allocate failed. Status = 0x%x\n",
                status);
        }
    }
    
    /* Initialiase the Notification object */
    if (DSP_SUCCEEDED(status))
    {
        copyTask->nodeEventMask = DSP_NODESTATECHANGE;
        TSTLOG_appendLog(pHandle,"Event Mask = 0x%x\n",copyTask->nodeEventMask);
        
        copyTask->notifications[2] = OsSpecificMemAlloc(sizeof(struct DSP_NOTIFICATION));
        
        /* Register for Notifs for Proc State */
        status = DSPNode_RegisterNotify(copyTask->hNode, copyTask->nodeEventMask,
            copyTask->notifyMask, copyTask->notifications[2]);
        if (DSP_FAILED(status)) 
        {
            TSTLOG_appendLog(pHandle,"DSPNode_RegisterNotify failed. status = 0x%x \n",status);
        }
    }
    
    if (DSP_SUCCEEDED(status))
    {
        copyTask->nodeEventMask = DSP_NODEMESSAGEREADY;
        TSTLOG_appendLog(pHandle,"Event Mask = 0x%x\n",copyTask->nodeEventMask);
        
        copyTask->notifications[3] = OsSpecificMemAlloc(sizeof(struct DSP_NOTIFICATION));
        
        /* Register for Notifs for Proc State */
        status = DSPNode_RegisterNotify(copyTask->hNode, copyTask->nodeEventMask,
            copyTask->notifyMask, copyTask->notifications[3]);
        if (DSP_FAILED(status)) 
        {
            TSTLOG_appendLog(pHandle,"DSPNode_RegisterNotify failed. status = 0x%x \n",status);
        }
    }
    
    
    if (DSP_SUCCEEDED(status))
    {
        status = DSPManager_WaitForEvents(copyTask->notifications, FOUREVENTS, &index, 100);
    }
    
    if (status == DSP_ETIMEOUT)
    {
        /* This was the expected return code, so reset the status value */
        status = DSP_SOK;
    }
    
    
    /* Create the strmcopy node on the DSP. */
    if (DSP_SUCCEEDED(status)) 
    {
        status = DSPNode_Create(copyTask->hNode);
        if (DSP_FAILED(status)) 
        {
            TSTLOG_appendLog(pHandle, "DSPNode_Create failed. Status = 0x%x\n", status);
        }
    }
    
    if (DSP_SUCCEEDED(status)) 
    {
        status = DSPManager_WaitForEvents(copyTask->notifications, FOUREVENTS, &index, 1000);
        if (DSP_FAILED(status))
        {
            TSTLOG_appendLog(pHandle,"ERROR: WaitForEvents failed for DSPNode_Create. status = 0x%x index = 0x%x \n", status,index);
            return (status);
        }
        else
        {
            TSTLOG_appendLog(pHandle,"WaitForEvents Passed for DSPNode_Create. status = 0x%x index = 0x%x \n", status,index);
        }
    }
    
    
    /* Start the msg node on the DSP. */
    if (DSP_SUCCEEDED(status)) 
    {
        status = DSPNode_Run(copyTask->hNode);
        if (DSP_FAILED(status)) 
        {
            TSTLOG_appendLog(pHandle, "DSPNode_Run failed. Status = 0x%x\n",status);
        }
    }
    
    if (DSP_SUCCEEDED(status)) 
    {
        status = DSPManager_WaitForEvents(copyTask->notifications, FOUREVENTS, &index, 1000);
        if (DSP_FAILED(status))
        {
            TSTLOG_appendLog(pHandle,"ERROR: WaitForEvents failed for DSPNode_Run. status = 0x%x index = 0x%x \n", status,index);
            return (status);
        }
        else
        {
            TSTLOG_appendLog(pHandle,"WaitForEvents Passed for DSPNode_Run. status = 0x%x index = 0x%x \n", status,index);
        }
    }
    
    
    /* Do some message transfers now to check for Message related Events */
    testMsg.dwCmd = RMS_USER + 3;
    testMsg.dwArg1 = 10;
    testMsg.dwArg2 = 10;
    timeout = 1000;
    
    status = DSPNode_PutMessage(copyTask->hNode, &testMsg, timeout);
    
    if (DSP_SUCCEEDED(status)) 
    {
        status = DSPManager_WaitForEvents(copyTask->notifications, FOUREVENTS, &index, 1000);
        if (DSP_FAILED(status))
        {
            TSTLOG_appendLog(pHandle,"ERROR: WaitForEvents failed for DSPNode_PutMessage. status = 0x%x index = 0x%x \n", status,index);
            return (status);
        }
        else
        {
            TSTLOG_appendLog(pHandle,"WaitForEvents Passed for DSPNode_PutMessage. status = 0x%x index = 0x%x \n", status,index);
        }
    }

    while (DSP_SUCCEEDED(status))
    {
        status = DSPNode_GetMessage(copyTask->hNode, &testMsg, timeout);
    }
    
    if (DSP_SUCCEEDED(status)) 
    {
        status = DSPManager_WaitForEvents(copyTask->notifications, FOUREVENTS, &index, 1000);
    }
    
    if (status == DSP_ETIMEOUT)
    {
        /* This was the expected return code, so reset the status value */
        TSTLOG_appendLog(pHandle,"Resetting status to DSP_SOK\n");
        status = DSP_SOK;
    }
    
    CleanupNode(copyTask);
    CleanupProcessor(copyTask);
    OsSpecificMemFree(copyTask->notifications[2]);
    OsSpecificMemFree(copyTask->notifications[3]);
    
    return (status);
}


/*
*  ======== checkStreamEvents ========
*  Perform stream related initialization.
*/
static DSP_STATUS checkStreamEvents(TASK_CONTEXT  *copyTask,CHAR *strmArgv)
{
    BYTE            argsBuf[ARGSIZE + sizeof(ULONG)];
    struct DSP_CBDATA      *pArgs;
    struct DSP_NODEATTRIN  nodeAttrIn;
    struct DSP_STRMATTR    attrs;
    struct DSP_UUID        uuid;
    DSP_STATUS      status = DSP_SOK;
    UINT            index = 100;
    CHAR            *pszUuid;
    INT             i = 0;
    struct DSP_STREAMATTRIN strmAttrs;
    INT             argc = 1 ;
    DWORD           iBuffSize = DEFAULTBUFSIZE;
    DWORD           bufSize ;
    DWORD           dataSize ;
    DWORD           dwArg ;
    FILE            *pHandle = copyTask->pHandle;
    struct DSP_PROCESSORINFO       dspInfo;
    UINT                    numProcs;
    UINT                    pindex=0;
    INT                     procId;
    
    copyTask->hProcessor = NULL;
    copyTask->hNode = NULL;
    copyTask->hInStream = NULL;
    copyTask->hOutStream = NULL;
    
    copyTask->ppInBufs = NULL;
    copyTask->ppOutBufs = NULL;
    copyTask->nStrmMode = STRMMODE_PROCCOPY;
    
    
    pszUuid = "F95CA076_EE7D_11D5_A8DD_00B0D055F6D1";  // INOUT
    UUID_UuidFromString((CHAR *)pszUuid, &uuid);
    
    
    attrs.uBufsize = DEFAULTBUFSIZE;
    attrs.uNumBufs = DEFAULTNBUFS;
    attrs.uAlignment = 0;
    attrs.uTimeout = DSP_FOREVER;
    attrs.lMode = copyTask->nStrmMode;
    if (copyTask->nStrmMode == STRMMODE_ZEROCOPY) {
        attrs.uSegid = DSP_SHMSEG0;  /* Use SM for zero-copy */
    }
    else 
    {
        attrs.uSegid = DEFAULTSEGMENT;
    }
    
    nodeAttrIn.uTimeout = 10000;
    nodeAttrIn.iPriority = 5;
    
#if defined(OMAP_2430)|| defined(OMAP_3430)
    nodeAttrIn.uProfileID = 0xff;
#endif
    pArgs = (struct DSP_CBDATA *)argsBuf;
    pArgs->cbData = ARGSIZE;
    
    while (DSP_SUCCEEDED(DSPManager_EnumProcessorInfo(pindex,&dspInfo,(UINT) sizeof (struct DSP_PROCESSORINFO),&numProcs)))
    {
        if (dspInfo.uProcessorType == DSPTYPE_55 || dspInfo.uProcessorType == DSPTYPE_64)
        {
            TSTLOG_appendLog(pHandle,"DSP device detected !! \n");
            procId = pindex;
            status = DSP_SOK;
            break;
        }
        pindex++;
    }
    /* Attach to an available DSP (in this case, the 1st DSP). */
    if (DSP_SUCCEEDED(status)) {
        status = DSPProcessor_Attach(procId, NULL,&copyTask->hProcessor);
        if (!DSP_SUCCEEDED(status)) {
            TSTLOG_appendLog(pHandle, "DSPProcessor_Attach failed. Status = 0x%x\n",status);
        }
    }
    else
        TSTLOG_appendLog(pHandle, "Failed to get the desired processor \n");
    
    if (DSP_SUCCEEDED(status))
    {
        status = DSPProcessor_Load(copyTask->hProcessor,argc,(CONST CHAR **)&strmArgv,(CONST CHAR **)NULL) ;
        
        if (!DSP_SUCCEEDED(status)) {
            TSTLOG_appendLog(pHandle, "DSPProcessor_Load failed.\n") ;
        }
    }
    
    if (DSP_SUCCEEDED(status))
    {
        status = DSPProcessor_Start(copyTask->hProcessor) ;
        
        if (!DSP_SUCCEEDED(status)) {
            TSTLOG_appendLog(pHandle, "DSPProcessor_Start failed.\n") ;
        }
    }
    
    /* Allocate the strmcopy node. */
    if (DSP_SUCCEEDED(status)) 
    {
        status = DSPNode_Allocate(copyTask->hProcessor, &uuid, pArgs,
            &nodeAttrIn, &copyTask->hNode);
        if (!DSP_SUCCEEDED(status)) {
            TSTLOG_appendLog(pHandle, "DSPNode_Allocate failed. Status = 0x%x\n",
                status);
        }
    }
    
    /* Connect the strmcopy node to the GPP. */
    if (DSP_SUCCEEDED(status)) 
    {
        status = DSPNode_Connect((DSP_HNODE)DSP_HGPPNODE, 0, copyTask->hNode,
            0, &attrs);
        if (!DSP_SUCCEEDED(status)) {
            TSTLOG_appendLog(pHandle, "DSPNode_Connect failed. Status = 0x%x\n", status);
        }
    }
    
    if (DSP_SUCCEEDED(status)) 
    {
        status = DSPNode_Connect(copyTask->hNode, 0, (DSP_HNODE)DSP_HGPPNODE,
            0, &attrs);
        if (!DSP_SUCCEEDED(status)) {
            TSTLOG_appendLog(pHandle, "DSPNode_Connect failed. Status = 0x%x\n",
                status);
        }
    }
    
    /* Create the strmcopy node on the DSP. */
    if (DSP_SUCCEEDED(status)) 
    {
        status = DSPNode_Create(copyTask->hNode);
        if (!DSP_SUCCEEDED(status)) {
            TSTLOG_appendLog(pHandle, "DSPNode_Create failed. Status = 0x%x\n", status);
        }
    }
    
    /* Start the strmcopy node on the DSP. */
    if (DSP_SUCCEEDED(status)) 
    {
        status = DSPNode_Run(copyTask->hNode);
        if (!DSP_SUCCEEDED(status)) {
            TSTLOG_appendLog(pHandle, "DSPNode_Run failed. Status = 0x%x\n",status);
        }
    }
    
    strmAttrs.cbStruct = sizeof(struct DSP_STREAMATTRIN);
    ///strmAttrs.uTimeout = DEFAULTTIMEOUT;
    strmAttrs.uTimeout = 10000;
    strmAttrs.uAlignment = DEFAULTALIGNMENT;
    strmAttrs.uNumBufs = DEFAULTNBUFS;
    strmAttrs.lMode = copyTask->nStrmMode;
    if (copyTask->nStrmMode == STRMMODE_PROCCOPY) 
    {
        strmAttrs.uSegment = DEFAULTSEGMENT;
    }
    else {
        strmAttrs.uSegment = DSP_SHMSEG0;    /* Use shared memory segment 0 */
    }
    
    /* Open an input data stream from the host to the strmcopy node. */
    if (DSP_SUCCEEDED(status)) 
    {
        status = DSPStream_Open(copyTask->hNode, DSP_TONODE, 0,
            &strmAttrs, &copyTask->hInStream);
        
        if (!DSP_SUCCEEDED(status)) {
            TSTLOG_appendLog(pHandle, "DSPStream_Open of input stream failed. Status = 0x%x\n", status);
        }
    }
    
    /* Open an output stream from the strmcopy node to the host. */
    if (DSP_SUCCEEDED(status)) {
        status = DSPStream_Open(copyTask->hNode, DSP_FROMNODE, 0,
            &strmAttrs, &copyTask->hOutStream);
        if (!DSP_SUCCEEDED(status)) {
            TSTLOG_appendLog(pHandle,
                "DSPStream_Open of output stream failed. Status = 0x%x\n",
                status);
        }
    }
    
    
    /* Initialiase the Notification object */
    if (DSP_SUCCEEDED(status))
    {
        copyTask->strmEventMask = DSP_STREAMDONE;
        copyTask->strmnotifications[0] = OsSpecificMemAlloc(sizeof(struct DSP_NOTIFICATION));
        copyTask->notifyMask = DSP_SIGNALEVENT;
        
        /* Register for Notifs for Proc State */
        status = DSPStream_RegisterNotify(copyTask->hInStream, copyTask->strmEventMask,
            copyTask->notifyMask, copyTask->strmnotifications[0]);
    }
    
    /* Initialiase the Notification object */
    if (DSP_SUCCEEDED(status))
    {
        copyTask->strmEventMask = DSP_STREAMIOCOMPLETION;
        copyTask->strmnotifications[1] = OsSpecificMemAlloc(sizeof(struct DSP_NOTIFICATION));
        
        /* Register for Notifs for Proc State */
        status = DSPStream_RegisterNotify(copyTask->hInStream, copyTask->strmEventMask,
            copyTask->notifyMask, copyTask->strmnotifications[1]);
    }
    
    /* Initialiase the Notification object */
    if (DSP_SUCCEEDED(status))
    {
        copyTask->strmEventMask = DSP_STREAMIOCOMPLETION;
        copyTask->strmnotifications[2] = OsSpecificMemAlloc(sizeof(struct DSP_NOTIFICATION));
        
        
        /* Register for Notifs for Proc State */
        status = DSPStream_RegisterNotify(copyTask->hOutStream, copyTask->strmEventMask,
            copyTask->notifyMask, copyTask->strmnotifications[2]);
    }
    
    if (DSP_SUCCEEDED(status))
    {
        status = DSPManager_WaitForEvents(copyTask->strmnotifications, THREEEVENTS, &index, 1000);
    }
    
    if (status == DSP_ETIMEOUT)
    {
        /* This was the expected return code, so reset the status value */
        status = DSP_SOK;
    }
    
    
    /* Allocate and issue buffer to input data stream. */
    if (DSP_SUCCEEDED(status)) {
        copyTask->ppInBufs = (BYTE**)OsSpecificMemAlloc(sizeof(BYTE*) * DEFAULTNBUFS);
        
        status = DSPStream_AllocateBuffers(copyTask->hInStream,
            DEFAULTBUFSIZE, copyTask->ppInBufs, DEFAULTNBUFS);
        if (!DSP_SUCCEEDED(status)) {
            TSTLOG_appendLog(pHandle,
                "DSPStream_AllocateBuffers for input stream failed. Status = 0x%x\n", status);
        }
    }
    
    if (DSP_SUCCEEDED(status))
    {
        index=100;
        status = DSPManager_WaitForEvents(copyTask->strmnotifications, THREEEVENTS, &index, 1000);
    }
    
    if (status == DSP_ETIMEOUT)
    {
        /* This was the expected return code, so reset the status value */
        status = DSP_SOK;
    }
    
    
    /* Allocate and issue buffer to output data stream. */
    if (DSP_SUCCEEDED(status)) {
        copyTask->ppOutBufs = (BYTE**)OsSpecificMemAlloc(sizeof(BYTE*) * DEFAULTNBUFS);
        
        status = DSPStream_AllocateBuffers(copyTask->hOutStream,
            DEFAULTBUFSIZE, copyTask->ppOutBufs, DEFAULTNBUFS);
        if (!DSP_SUCCEEDED(status)) {
            TSTLOG_appendLog(pHandle,
                "DSPStream_AllocateBuffers for output stream failed. Status = 0x%x\n", status);
        }
    }
    
    if (DSP_SUCCEEDED(status))
    {
        index=100;
        status = DSPManager_WaitForEvents(copyTask->strmnotifications, THREEEVENTS, &index, 1000);
    }
    
    if (status == DSP_ETIMEOUT)
    {
        /* This was the expected return code, so reset the status value */
        status = DSP_SOK;
    }
    
    
    for (i=0; i < iBuffSize; i++)
    {
        copyTask->ppInBufs[0][i]  = 10 ;
    }
    
    if (DSP_SUCCEEDED(status))
    {
        /* Issue Full Data Buffer */
        status = DSPStream_Issue(copyTask->hInStream, (BYTE *)copyTask->ppInBufs[0], iBuffSize, iBuffSize, 0);
    }
    
    if (DSP_FAILED(status)) 
    {
        /* Report Error */
        TST_verbose("DSPStream_Issue FAILED\n");
        return status;
    }
    
    if (DSP_SUCCEEDED(status))
    {
        /* Reclaim Empty Buffer */
        status = DSPStream_Reclaim(copyTask->hInStream, &copyTask->ppInBufs[0], &dataSize, &bufSize, &dwArg);
    }
    
    if (DSP_FAILED(status)) 
    {
        /* Report Error */
        TST_verbose("DSPStream_Reclaim FAILED\n");
        return status;
    }
    
    if (DSP_SUCCEEDED(status))
    {
        index=100;
        status = DSPManager_WaitForEvents(copyTask->strmnotifications, THREEEVENTS, &index, 20000);
        if(DSP_SUCCEEDED(status))
        {
            TSTLOG_appendLog(pHandle,"WaitForEvents on DSPStream_Reclaim passed. status = 0x%x index = 0x%x \n",status,index);
        }
        else
        {
            TSTLOG_appendLog(pHandle,"WaitForEvents on DSPStream_Reclaim failed. status = 0x%x index = 0x%x \n",status,index);
        }
    }
    
    if(DSP_SUCCEEDED(status))
    {
        /* Issue Empty Buffer */
        status = DSPStream_Issue(copyTask->hOutStream, copyTask->ppOutBufs[0], iBuffSize, iBuffSize, 0);
    }
    
    if (DSP_FAILED(status)) 
    {
        /* Report Error */
        TST_verbose("DSPStream_Issue FAIL\n");
        return status;
    }
  
    if (DSP_SUCCEEDED(status))
    {
        /* Reclaim Full Buffer */
        status = DSPStream_Reclaim(copyTask->hOutStream, &copyTask->ppOutBufs[0], &dataSize, &bufSize, &dwArg);
    }
    
    if (DSP_FAILED(status)) 
    {
        /* Report Error */
        TST_verbose("DSPStream_Reclaim OutStream FAIL\n");
        return status;
    }

    if (DSP_SUCCEEDED(status))
    {
        index=100;
        status = DSPManager_WaitForEvents(copyTask->strmnotifications, THREEEVENTS, &index, 20000);
        if(DSP_SUCCEEDED(status))
        {
            TSTLOG_appendLog(pHandle,"WaitForEvents on DSPStream_Reclaim passed. status = 0x%x index = 0x%x \n",status,index);
        }
        else
        {
            TSTLOG_appendLog(pHandle,"WaitForEvents on DSPStream_Reclaim failed. status = 0x%x index = 0x%x \n",status,index);
        }
    }
    
    if (DSP_SUCCEEDED(status))
    {
        iBuffSize = 0; /* Terminating the Stream */
        /* Issue Full Data Buffer */
        status = DSPStream_Issue(copyTask->hInStream, copyTask->ppInBufs[0], iBuffSize, DEFAULTBUFSIZE, 0);
    }
    
    if (DSP_SUCCEEDED(status))
    {
        /* Reclaim Empty Buffer */
        status = DSPStream_Reclaim(copyTask->hInStream, &copyTask->ppInBufs[0], &dataSize, &bufSize,&dwArg);
    }
    else
    {
        TSTLOG_appendLog(pHandle,"DSPStream_Issue failed\n");       
    }
    
    if (DSP_SUCCEEDED(status))
    {
        index=100;
        status = DSPManager_WaitForEvents(copyTask->strmnotifications, THREEEVENTS, &index, 1000);
        /* The event wud have been set for the prev issue */
        if (!DSP_SUCCEEDED(status))
        {
            TSTLOG_appendLog(pHandle,"WaitForEvents on DSPStream_Reclaim failed. status = 0x%x index = 0x%x \n",status,index);
        }
        else
        {
            TSTLOG_appendLog(pHandle,"WaitForEvents on DSPStream_Reclaim passed. status = 0x%x index = 0x%x \n",status,index);
        }
    }
    else
    {
        TSTLOG_appendLog(pHandle,"DSPStream_Reclaim failed. status = 0x%x\n", status);
        return (status);
    }
    
    if (DSP_SUCCEEDED(status))
    {
        /* Issue Empty Buffer */
        iBuffSize = DEFAULTBUFSIZE;
        status = DSPStream_Issue(copyTask->hOutStream, copyTask->ppOutBufs[0], iBuffSize, iBuffSize, 0);
    }
    
    if (DSP_SUCCEEDED(status))
    {
        /* Reclaim Full Buffer */
        status = DSPStream_Reclaim(copyTask->hOutStream, &copyTask->ppOutBufs[0], &dataSize, &bufSize,&dwArg);
    }
    if (DSP_SUCCEEDED(status))
    {
        // yet another reclaim
        index=100;
        status = DSPManager_WaitForEvents(copyTask->strmnotifications, THREEEVENTS, &index, 10000);
        if (!DSP_SUCCEEDED(status))
        {
            TSTLOG_appendLog(pHandle,"WaitForEvents on DSPStream_Reclaim failed. status = 0x%x index = 0x%x \n",status,index);
        }
        else
        {
            TSTLOG_appendLog(pHandle,"WaitForEvents on DSPStream_Reclaim passed. status = 0x%x index = 0x%x \n",status,index);
        }
    }
    
    return (status);
}



/*
*  ======== CleanupProcessor ========
*  Perform processor related cleanup.
*/
static DSP_STATUS CleanupProcessor(TASK_CONTEXT *copyTask)
{
    DSP_STATUS      status = DSP_SOK;
    FILE           *pHandle = copyTask->pHandle;
    
    if (copyTask->hProcessor) {
        /* Detach from processor. */
        status = DSPProcessor_Detach(copyTask->hProcessor);
        if (DSP_FAILED(status)) 
        {
            TSTLOG_appendLog(pHandle, "DSPProcessor_Detach failed.\n");
        }
        
        copyTask->hProcessor = NULL;
    }
    
    return (status);
}



static DSP_STATUS CleanupStreams(TASK_CONTEXT *copyTask)
{
    struct DSP_STREAMINFO  streamInfo;
    DSP_STATUS      status = DSP_SOK;
    DWORD           dwArg = 0;
    DWORD           dwBufsize = DEFAULTBUFSIZE;
    BYTE            *pBuf;
    FILE           *pHandle = copyTask->pHandle;
    
    
    /* Reclaim and free stream buffers. */
    if (copyTask->hInStream) 
    {
        DSPStream_Idle(copyTask->hInStream, TRUE);
        status = DSPStream_GetInfo(copyTask->hInStream, &streamInfo,
            sizeof(streamInfo));
        
        if (copyTask->ppInBufs) 
        {
            /* Reclaim the buffer if it's still in the stream */
            if (streamInfo.uNumberBufsInStream) {
                status = DSPStream_Reclaim(copyTask->hInStream, &pBuf, \
                    &dwBufsize, NULL, &dwArg);
            }
            
            OsSpecificMemFree(copyTask->ppInBufs);
            copyTask->ppInBufs = NULL;
            
            /* Close open streams. */
            status = DSPStream_Close(copyTask->hInStream);
            if (DSP_FAILED(status)) {
                TSTLOG_appendLog(pHandle, "DSPStream_Close of input stream failed. "
                    "Status = 0x%x\n", status);
            }
            
            copyTask->hInStream = NULL;
        }
    }
    
    if (copyTask->hOutStream) {
        DSPStream_Idle(copyTask->hOutStream, TRUE);
        
        /* Need to determine the number of buffers left in the stream */
        status = DSPStream_GetInfo(copyTask->hOutStream, &streamInfo,
            sizeof(streamInfo));
        
        if (copyTask->ppOutBufs) {
            if (streamInfo.uNumberBufsInStream) {
                status = DSPStream_Reclaim(copyTask->hOutStream, &pBuf, \
                    &dwBufsize, NULL, &dwArg);
            }
            
            copyTask->ppOutBufs = NULL;
        }
        
        status = DSPStream_Close(copyTask->hOutStream);
        if (DSP_FAILED(status)) 
        {
            TSTLOG_appendLog(pHandle, "DSPStream_Close of output stream failed. "
                "Status = 0x%x\n", status);
        }
        copyTask->hOutStream = NULL;
    }
    
    return (status);
}


/*
*  ======== CleanupNode ========
*  Perform node related cleanup.
*/
static DSP_STATUS CleanupNode(TASK_CONTEXT *copyTask)
{
    DSP_STATUS      status = DSP_SOK;
    DSP_STATUS      exitStatus = DSP_SOK;
    FILE           *pHandle = copyTask->pHandle;
    
    if (copyTask->hNode) {
        
        // Terminate DSP node.
        status = DSPNode_Terminate(copyTask->hNode, &exitStatus);
        if (!DSP_SUCCEEDED(status)) {
            TSTLOG_appendLog(pHandle, "DSPNode_Terminate failed: 0x%x\n", status);
        }
        
        /* Delete DSP node. */
        status = DSPNode_Delete(copyTask->hNode);
        if (!DSP_SUCCEEDED(status)) {
            TSTLOG_appendLog(pHandle, "DSPNode_Delete failed: 0x%x\n", status);
        }
        
        copyTask->hNode = NULL;
    }
    
    return (status);
}


