/*  ============================================================================
 *   File    ivanodest.c
 *
 *   Path    $(TESTSUITEROOT)\Behaviour\IVANodeState
 * 
 *   Desc:
 *        This module tests the state of a node following various
 *        DSPNode API function calls.  This test allows various
 *        APIs which alter the state of the Node.  A Node
 *        is allocated and various function calls that alter the state are
 *        issued by the user in a given sequence to verify the correct
 *        board transitions.  If the user supplied sequence is
 *        incorrect, then one function call may result in a failure.
 *
 *  Usage:
 *        ivanodest [-v] [-l "log path/filename"]
 *                       [-x "valid executable" ]
 *                       [-t "state transitions"]
 *
 *        -v: verbose mode
 *        -l: use output log and location
 *        -x: name of valid IVA executable
 *        -t: input of state transition sequence
 *
 *  API Codes to use to specify sequence as input:
 *
 *         1: DSPNode_Allocate
 *         2: DSPNode_Create
 *         3: DSPNode_Run
 *         4: DSPNode_GetMessage
 *         5: DSPNode_PutMessage
 *         6: DSPNode_AllocMsgBuf
 *         7: DSPNode_FreeMsgBuf
 *         8: DSPNode_Pause
 *         9: DSPNode_Terminate
 *         a: DSPNode_Delete
 *         b: DSPNode_GetAttr
 *         c: DSPNode_RegisterNotify
 *
 *
 *   ---------------------------------------------------------------------------
 *   Rev     1.0.0
 * 
 *   This computer program is copyright to Texas Instruments Incorporated.
 *   The program may not be used without the written permission
 *   of Texas Instruments Incorporated. or against the terms and conditions
 *   stipulated in the agreement under which this program has been
 *   supplied.
 *
 *   (c) Texas Instruments Incorporated 2003
 *
 *   ============================================================================
 *   Revision History
 *
 *   JUL 03, 2002    REF=ORG                   Swamy MC
 *   JULY 22, 2004   Modified for IVA Node States   RAMESH GUPTA G
 *   Original Version.
 *   ============================================================================
 */


#include <stdio.h>
#include <string.h>
#include <dbapi.h>
#include <getopt_test.h>
#include <tst.h>
#include <tstlog.h>
#include <uuidutil.h>
#include <testosal.h>
#include <DSPProcessor_OEM.h>
#include <msgtxfr.h>
#define DEFNAMESIZE         80

#define DSPEXEC             "iva.dof"

#define DEFSTATES           "1 2"
#define SEPARATORS          " , \t\n"

INT ivacheckTransNode(const INT iPrevState, const INT iTransition);

/*
 *  ======== ivanodest_main ========
 *
 */
INT ivanodest_main (INT argc, CHAR * argv[])
{

    INT                         opt;
    INT                         cRetVal = 1;
    INT                         iPrevState;
    INT                         iTransition = 0;
    INT                         iArgc = 1;
    BOOL                        bUseLog=FALSE;
    FILE                        *pHandle = NULL;
    CHAR                        szLogName[DEFNAMESIZE];
    CHAR                        *szErrorMsg = "";
    CHAR                        *aArgv[DEFNAMESIZE] = { DSPEXEC, "arg1" };
    CHAR                        *szProcStates;
    CHAR                        szDefaultStates[DEFNAMESIZE];
    CHAR                        *aEnvp[DEFNAMESIZE] = {NULL};
    struct DSP_NODEATTR                *pAttr;
    DSP_STATUS                  status = DSP_SOK;
    DSP_STATUS                  exitStatus = DSP_SOK;
    struct DSP_UUID                    pNodeID;
    struct DSP_NODEATTRIN              *nodeAttrin;
    struct DSP_PROCESSORATTRIN         *pAttrIn;
    struct DSP_NOTIFICATION*           hNotify;
    struct DSP_CBDATA                  *pArgs;
    struct DSP_MSG                     *pMessage;
    BYTE                        **pBuffer;
    UINT                        uTimeout = 15000;

    CHAR                        *pstrList[50];
    CHAR                        *pszUuid;
    INT                         cListsize = 0;
    INT                         count = 0;
    struct DSP_STRMATTR                attrs;
    BOOL                        fLoadDsp = FALSE;
    
    struct DSP_PROCESSORINFO       dspInfo;
    UINT                    numProcs;
    UINT                    index=0;
    INT                     procId;


    typedef struct {
    DSP_HPROCESSOR    hProcessorIva;
    DSP_HNODE         hNodeIva;
    INT               numMsg;     
 
}IVAMSG_TASK;
    
    IVAMSG_TASK                 ivaMsgTask;
    
    attrs.uSegid                = 0;
    attrs.uBufsize              = 512;
    attrs.uNumBufs              = 2;  //DEFAULTNBUFS;
    attrs.uAlignment            = 0;
    attrs.uTimeout              = 10000; //DSP_FOREVER;
    attrs.lMode                 = STRMMODE_PROCCOPY;

    ivaMsgTask.hProcessorIva = NULL;
    ivaMsgTask.hNodeIva      = NULL;
    ivaMsgTask.numMsg        = NULL;

    pArgs      = OsSpecificMemAlloc(sizeof(struct DSP_CBDATA));	
    pArgs->cbData = 1;

    pAttrIn    = OsSpecificMemAlloc(sizeof(struct DSP_PROCESSORATTRIN));
    pAttrIn->cbStruct = sizeof(struct DSP_PROCESSORATTRIN);
    pAttrIn->uTimeout = 10000;

    pAttr      = OsSpecificMemAlloc(sizeof(struct DSP_NODEATTR));

    nodeAttrin = OsSpecificMemAlloc(sizeof(struct DSP_NODEATTRIN));
    nodeAttrin->cbStruct = sizeof(struct DSP_NODEATTRIN);
    nodeAttrin->uTimeout = 2000;
    nodeAttrin->iPriority = 5;

    pszUuid = "F95CA072_EE7D_11D5_A8DD_00B0D055F6D1";  // dataxfr
    UUID_UuidFromString((CHAR *)pszUuid, &pNodeID);

    pNodeID=Iva_Msguuid;
    pMessage = OsSpecificMemAlloc(sizeof(struct DSP_MSG));
    pMessage->dwCmd = RMS_USER + 1;
    pMessage->dwArg1 = 1;
    pMessage->dwArg2 = 1;

    pBuffer = OsSpecificMemAlloc(sizeof(BYTE *));


    /* Set default set of state transitions */
    strncpy(szDefaultStates, (char *)DEFSTATES, DEFNAMESIZE);
    szProcStates = szDefaultStates;

    optind = 0;
    while ((opt = getopt_test(argc, (char *const)argv, "+vl:u:x:t:")) != EOF) 
    {
        switch (opt) 
        {
        case 'v':
            /* verbose */
            TST_setVerbose(TRUE);
            break;
        case 'l':
            /* use log and store with name location */
            strncpy(szLogName, optarg, sizeof(szLogName));
            bUseLog = TRUE;
            pHandle = TSTLOG_openLog(szLogName);
            break;
        case 'x':
            /* use given executable */
            aArgv[0] = (char *) optarg;
            fLoadDsp = TRUE;
            break;
        case 't':
            /* input state transitions */
            szProcStates = (char *) optarg;
            if (bUseLog) 
            {
                TSTLOG_appendLog(pHandle, "Using board states: \"%s\"\n",
                                 szProcStates);
            }
            TST_verbose("Using board states: \"%s\"\n", szProcStates);
            break;
        default:
            TST_verbose(("Invalid option: %c\n"), opt);
            cRetVal = 0;
            szErrorMsg = "Invalid option specified.";
            break;
        }
    }

    if (cRetVal > 0) 
    {

    while (DSP_SUCCEEDED(DSPManager_EnumProcessorInfo(index,&dspInfo,(UINT) sizeof (struct DSP_PROCESSORINFO),&numProcs)))
        {
                if (dspInfo.uProcessorType == IVA_ARM7)
                {
                        printf("IVA device detected !! \n");
                        procId = index;
                        status = DSP_SOK;
                        break;
                }
                index++;
        }

    /* Attach to IVA */
    if (DSP_SUCCEEDED(status)) 
    {
        status = DSPProcessor_Attach(procId, NULL,&ivaMsgTask.hProcessorIva);
        if (DSP_SUCCEEDED(status)) 
       {
            TSTLOG_appendLog(pHandle, "IVA:DSPProcessor_Attach succeeded.\n");
           }
        else 
       {
            TSTLOG_appendLog(pHandle, "IVA:DSPProcessor_Attach failed. Status = 0x%x\n",status);
           }
        }
        else
           TSTLOG_appendLog(pHandle, "Failed to get the desired processor \n");



        if(fLoadDsp)
        {
            DSPProcessor_Load(ivaMsgTask.hProcessorIva, iArgc, (CONST CHAR **)aArgv, (CONST CHAR **)aEnvp);
            DSPProcessor_Start(ivaMsgTask.hProcessorIva);
        }

        TST_verbose("Initiating IVANode state testing\n");

        /* Process API list */
/*        pstrList[cListsize++] = strtok(szProcStates, SEPARATORS);
        while ((pstrList[cListsize++] = strtok(NULL, SEPARATORS)) != NULL);*/

        /* Adjust for NULL element */
        cListsize--;

        while (cRetVal > 0 && count < cListsize) 
        {
            switch(pstrList[count++][0]) 
            {
            case '1':
                /* DSPNode_Allocate */
                iTransition = 1;
                status = DSPNode_Allocate(ivaMsgTask.hProcessorIva, &pNodeID, pArgs, nodeAttrin, &ivaMsgTask.hNodeIva);

                /* Get Initial State */
                DSPNode_GetAttr(ivaMsgTask.hNodeIva, pAttr, sizeof(struct DSP_NODEATTR));
                szErrorMsg = "Error in DSPNode_Allocate():IVA";
                break;
            case '2':
                /* DSPNode_Create */
                iTransition = 2;
                status = DSPNode_Create(ivaMsgTask.hNodeIva);

                szErrorMsg = "Error in DSPNode_Create():IVA";
                break;
            case '3':
                /* DSPNode_Run */
                iTransition = 3;
                status = DSPNode_Run(ivaMsgTask.hNodeIva);

                szErrorMsg = "Error in DSPNode_Run():IVA";
                break;
            case '4':
                /* DSPNode_GetMessage */
                iTransition = 4;
                status = DSPNode_GetMessage(ivaMsgTask.hNodeIva, pMessage, uTimeout);

                szErrorMsg = "Error in DSPNode_GetMessage():IVA";

                break;
            case '5':
                /* DSPNode_PutMessage */
                iTransition = 5;
                status = DSPNode_PutMessage(ivaMsgTask.hNodeIva, pMessage, uTimeout);
                szErrorMsg = "Error in DSPNode_PutMessage():IVA";
                break;
            case '6':
                /* DSPNode_AllocMsgBuf */
                iTransition = 6;
                status = DSPNode_AllocMsgBuf(ivaMsgTask.hNodeIva, 1024, NULL, pBuffer);

                szErrorMsg = "Error in DSPNode_AllocMsgBuf():IVA";
                break;
            case '7':
                /* DSPNode_FreeMsgBuf */
                iTransition = 7;
                status = DSPNode_FreeMsgBuf(ivaMsgTask.hNodeIva, *pBuffer, NULL);

                szErrorMsg = "Error in DSPNode_FreeMsgBuf():IVA";
                break;
            case '8':
                /* DSPNode_Pause */
                iTransition = 8;
                status = DSPNode_Pause(ivaMsgTask.hNodeIva);

                szErrorMsg = "Error in DSPNode_Pause():IVA";
                break;
            case '9':
                /* DSPNode_Terminate */
                iTransition = 9;
                status = DSPNode_Terminate(ivaMsgTask.hNodeIva, &exitStatus);

                szErrorMsg = "Error in DSPNode_Terminate():IVA";
                break;
            case 'a':
                /* DSPNode_GetAttr */
                iTransition = 10;
                status = DSPNode_GetAttr(ivaMsgTask.hNodeIva, pAttr, sizeof(struct DSP_NODEATTR));
                
                szErrorMsg = "Error in DSPNode_GetAttr():IVA";
                break;
            case 'b':
                /* DSPNode_ChangePriority */
                iTransition = 11;
                status = DSPNode_ChangePriority(ivaMsgTask.hNodeIva, DSP_NODE_MIN_PRIORITY);
                
                szErrorMsg = "Error in DSPNode_ChangePriority():IVA";
                break;
            case 'c':
                /* DSPNode_RegisterNotify */
                iTransition = 12;
                
                hNotify = OsSpecificMemAlloc(sizeof(struct DSP_NOTIFICATION*));
                hNotify->psName = (PSTRING) "Event";
                status = DSPNode_RegisterNotify(ivaMsgTask.hNodeIva, DSP_NODEMESSAGEREADY,
                    DSP_SIGNALEVENT, hNotify);
                
                szErrorMsg = "Error in DSPNode_RegisterNotify():IVA";
                OsSpecificMemFree(hNotify);
                
                break;
            default: TST_verbose("Invalid API call: %c\n", pstrList[cListsize-1][0]);
                break;
            } /* End of Switch stmt */
            
        if (DSP_FAILED(status)) 
        {
            cRetVal = 0;
            break;
        }

        /* Get state and check for correctness, unless detached */
        if (iTransition != 10) 
        {
            iPrevState = pAttr->iNodeInfo.nsExecutionState;
            status = DSPNode_GetAttr(ivaMsgTask.hNodeIva, pAttr, sizeof(struct DSP_NODEATTR));
        }
        else 
        {
            continue;
        }
        
        if (DSP_FAILED(status)) 
        {
            cRetVal = 0;
            szErrorMsg = "Error in DSPNode_GetAttr()\n";
            break;
        }
        else 
        {
            /* Check to see if we are in the correct state */
            switch(pAttr->iNodeInfo.nsExecutionState) 
            {
            case NODE_ALLOCATED:
                if (ivacheckTransNode(iPrevState, iTransition) != NODE_ALLOCATED) 
                {
                    cRetVal = 0;
                    TSTLOG_appendLog(pHandle,
                        "\nTransition %i from state %i to %i invalid.\n",
                        iTransition, iPrevState,
                        pAttr->iNodeInfo.nsExecutionState);
                    
                    TSTLOG_appendLog(pHandle, "\nI should be in %i, NODE_ALLOCATED\n", NODE_ALLOCATED);
                    break;
                }
                break;
            case NODE_CREATED:
                if (ivacheckTransNode(iPrevState, iTransition) != NODE_CREATED) 
                {
                    cRetVal = 0;
                    TSTLOG_appendLog(pHandle,
                        "\nTransition %i from state %i to %i invalid.\n",
                        iTransition, iPrevState,
                        pAttr->iNodeInfo.nsExecutionState);
                    
                    TSTLOG_appendLog(pHandle, "\nI should be in %i, NODE_CREATED\n", NODE_CREATED);
                    break;
                }
                break;
            case NODE_RUNNING:
                if (ivacheckTransNode(iPrevState, iTransition) != NODE_RUNNING) 
                {
                    cRetVal = 0;
                    TSTLOG_appendLog(pHandle,
                        "\nTransition %i from state %i to %i invalid.\n",
                        iTransition, iPrevState,
                        pAttr->iNodeInfo.nsExecutionState);
                    TSTLOG_appendLog(pHandle, "\nI should be in %i, NODE_RUNNING\n", NODE_RUNNING);
                    
                    break;
                }
                break;
            case NODE_PAUSED:
                if (ivacheckTransNode(iPrevState, iTransition) != NODE_PAUSED) 
                {
                    cRetVal = 0;
                    TSTLOG_appendLog(pHandle,
                        "\nTransition %i from state %i to %i invalid.\n",
                        iTransition, iPrevState,
                        pAttr->iNodeInfo.nsExecutionState);
                    TSTLOG_appendLog(pHandle, "\nI should be in %i, NODE_PAUSED\n", NODE_PAUSED);
                    
                    break;
                }
                break;
            case NODE_DONE:
                if (ivacheckTransNode(iPrevState, iTransition) != NODE_DONE) 
                {
                    cRetVal = 0;
                    TSTLOG_appendLog(pHandle,
                        "\nTransition %i from state %i to %i invalid.\n",
                        iTransition, iPrevState,
                        pAttr->iNodeInfo.nsExecutionState);
                    TSTLOG_appendLog(pHandle, "\nI should be in %i, NODE_DONE\n", NODE_DONE);
                    
                    break;
                }
                break;
            default:
                cRetVal = 0;
                TSTLOG_appendLog(pHandle, "State %i doesn't exist!\n",
                    pAttr->iNodeInfo.nsExecutionState);
                szErrorMsg = "Landed in unknown state!\n";
                break;
            }
        }
        
       } /* End of while loop */
       
    } /* End of if - cRetVal */
    
    /* Cleaning up */
    DSPProcessor_Detach(ivaMsgTask.hProcessorIva);
    
    OsSpecificMemFree(pArgs);
    OsSpecificMemFree(pAttrIn); 
    OsSpecificMemFree(pAttr);
    OsSpecificMemFree(pMessage);
    OsSpecificMemFree(pBuffer);
    

    if (cRetVal == 0) 
    {
        /* Report in LOG module */
        if (bUseLog) 
        {
            TSTLOG_appendLog(pHandle, "FAIL\n");
            TSTLOG_appendLog(pHandle, "Error: %s\n", szErrorMsg);
        }
        TST_verbose("FAIL\n");
    }
    else 
    {
        /* Report in LOG module */
        if (bUseLog) 
        {
            TSTLOG_appendLog(pHandle, "PASS\n");
        }
        TST_verbose("PASS\n");
    }
    
    /* Close Log */
    if (bUseLog) 
    {
        TSTLOG_closeLog(pHandle);
    }
    
    return 0; 
}

/*
 *  ======== ivacheckTransNode ========
 *  This function is specific to the DSPNode Module.
 *  It is configured to calculate, given a present state and a transition,
 *  which state it should end up in.  This is used to check whether
 *  or not a given transition performed correctly.
 */
 INT ivacheckTransNode(const INT iPrevState, const INT iTransition)
 {
     INT iGoalState;
     
     switch(iTransition) 
     {
     case 1:
         iGoalState = iPrevState;
         break;
     case 2:
         iGoalState = iPrevState;
         break;
     case 3:
         if (iPrevState == NODE_ALLOCATED) 
         {
             iGoalState = NODE_CREATED;
         }
         else 
         {
             iGoalState = iPrevState;
         }
         break;
     case 4:
         if (iPrevState == NODE_CREATED) 
         {
             iGoalState = NODE_RUNNING;
         }
         else if (iPrevState == NODE_PAUSED) 
         {
             iGoalState = NODE_RUNNING;
         }
         else 
         {
             iGoalState = iPrevState;
         }
         break;
     case 5:
         iGoalState = iPrevState;
         break;
     case 6:
         iGoalState = iPrevState;
         break;
     case 7:
         iGoalState = iPrevState;
         break;
     case 8:
         iGoalState = iPrevState;
         break;
     case 9:
         if (iPrevState == NODE_RUNNING) 
         {
             iGoalState = NODE_PAUSED;
         }
         else 
         {
             iGoalState = iPrevState;
         }
         break;
     case 10:
         if (iPrevState == NODE_RUNNING) 
         {
             iGoalState = NODE_DONE;
         }
         else 
         {
             iGoalState = iPrevState;
         }
         break;
     case 11:
         iGoalState = iPrevState;
         break;
     case 12:
         iGoalState = iPrevState;
         break;
     case 13:
         iGoalState = iPrevState;
         break;
     case 14:
         iGoalState = iPrevState;
         break;
     default:
         iGoalState = 0;
         break;
     }
     return iGoalState;
 }
 
