/*  ============================================================================
 *   File    ivaprocst.c
 *
 *   Path    $(TESTSUITEROOT)\Behaviour\IVAProcState
 * 
 *   Desc:
 *        This module tests the state of a IVA processor following various
 *        DSPProcessor API function calls.  This test allows various
 *        APIs which alter the state of the Processor.  A Processor
 *        is attached and various function calls that alter the state are
 *        issued by the user in a given sequence to verify the correct
 *        board transitions.  If the user supplied sequence is
 *        incorrect, then one function call may result in a failure.
 *        If DSPProcessor_Attach is not called as the first function,
 *        the sequence will fail.
 *
 *  Additional Note : State should change when appropriate API is called
 *			  and it should not change when other allowed APIs are called.
 *
 *  Usage:
 *        dspprocst [-v] [-l "log path/filename"]
 *                       [-x "valid executable" ]
 *                       [-p "processor number" ]
 *                       [-t "state transitions"]
 *
 *        -v: verbose mode
 *        -l: use output log and location
 *        -x: name of valid DSP executable
 *        -p: valid processor number (default 0)
 *        -t: input of state transition sequence
 *
 *  API Codes to use to specify sequence as input:
 *
 *         1: DSPProcessor_Attach
 *         2: DSPProcessor_Detach
 *         3: DSPProcessor_EnumNodes
 *         4: DSPProcessor_GetResourceInfo
 *         5: DSPProcessor_GetState
 *         6: DSPProcessor_RegisterNotify
 *         7: DSPProcessor_Load
 *         8: DSPProcessor_Start
 *         9: DSPProcessor_Ctrl
 *
 *  Example:
 *  framework BEHAVIOUR IVAPROCST -v -l output.txt -x apitest0_nortdxidl.dof -t "1 2"
 *
 *   ---------------------------------------------------------------------------
 *   Rev     1.0.0
 * 
 *   This computer program is copyright to Texas Instruments Incorporated.
 *   The program may not be used without the written permission
 *   of Texas Instruments Incorporated. or against the terms and conditions
 *   stipulated in the agreement under which this program has been
 *   supplied.
 *
 *   (c) Texas Instruments Incorporated 2003
 *
 *   ============================================================================
 *   Revision History
 *
 *   JUL 03, 2002    REF=ORG                                   -Swamy MC
 *   JULY 22,2004    Modified to Support IVA Processor Changes -RAMESH GUPTA G
 *      Original Version.
 *   ============================================================================
 */


#include <stdio.h>
#include <dbapi.h>
#include <string.h>
#include <getopt_test.h>
#include <tst.h>
#include <tstlog.h>
#include <uuidutil.h>
#include <testosal.h>   
#include <DSPProcessor_OEM.h>
#include <msgtxfr.h>
#define DEFNAMESIZE         80
#define DSPEXEC             "iva.dof"

#define DEFSTATES           "1 2"
#define SEPARATORS          " , \t\n"

INT checkIvaTransProc(const INT iPrevState, const INT iTransition);

/*
 *  ======== dspprocst_main ========
 *
 */
INT ivaprocst_main (INT argc, CHAR * argv[])
{

    INT                     opt;
    INT                     cRetVal = 1;
    INT                     iPrevState;
    INT                     iTransition = 0;
    INT                     iArgc = 1;
    INT                     iProc = 1;   /*Processor ID for IVA*/
    UINT		            *puNumNodes, *puAllocated;
    BOOL                    bUseLogFile=FALSE;
    FILE                    *pHandle = NULL;
    CHAR                    szLogName[DEFNAMESIZE];
    CHAR                    *szErrorMsg = "";
    CONST CHAR              *aArgv[DEFNAMESIZE] = { DSPEXEC, "arg1" };
    CONST CHAR              *aEnvp[DEFNAMESIZE] = {NULL};
    CHAR		            *szProcStates;
    DSP_HPROCESSOR          hProcessor;
    DSP_STATUS              status;
    DSP_STATUS              exitStatus = DSP_SOK;
    struct DSP_PROCESSORATTRIN     *pAttrIn;
    struct DSP_CBDATA              *pArgs;
    DSP_HNODE               *aNodeTab;
    DSP_HNODE               hNode;
    struct DSP_UUID                gidNodeID;
    CHAR                    *pszUuid;
    struct DSP_NOTIFICATION        hNotification;
    CHAR                    *pstrList[50];
    INT                     cListsize = 0;
    INT                     count = 0;
    struct DSP_PROCESSORSTATE      *procState;

    struct DSP_PROCESSORINFO       dspInfo;
    UINT                    numProcs;
    UINT                    index=0;
    INT                     procId;
    struct DSP_RESOURCEINFO        rsInfo;


    typedef struct {
    DSP_HPROCESSOR    hProcessorIva;
    DSP_HNODE         hNodeIva;
    INT               numMsg;

    }IVAMSG_TASK;

    IVAMSG_TASK                 ivaMsgTask;


    pArgs       = OsSpecificMemAlloc(sizeof(struct DSP_CBDATA));
    pAttrIn     = OsSpecificMemAlloc(sizeof(struct DSP_PROCESSORATTRIN));
    procState   = OsSpecificMemAlloc(sizeof(struct DSP_PROCESSORSTATE));
    puNumNodes  = OsSpecificMemAlloc(sizeof(struct DSP_PROCESSORATTRIN));
    puAllocated = OsSpecificMemAlloc(sizeof(UINT));
    aNodeTab    = OsSpecificMemAlloc(sizeof(DSP_HNODE));
	   
    pAttrIn->cbStruct = sizeof(struct DSP_PROCESSORATTRIN);
    pAttrIn->uTimeout = 50;

    /* Set default set of state transitions */
    szProcStates = (char *)DEFSTATES;

	optind = 0;
    while ((opt = getopt_test(argc, (char *const)argv, "+vl:u:p:x:t:")) != EOF) 
	{
        switch (opt) 
		{
        case 'v':
            /* verbose */
            TST_setVerbose(TRUE);
            break;
        case 'l':
            /* use log and store with name location */
            strncpy(szLogName, optarg, sizeof(szLogName));
        	bUseLogFile = TRUE;
            pHandle = TSTLOG_openLog(szLogName);
            break;
        case 'p':
            /* Set new processor number to test */
            iProc = CSL_Atoi(optarg);
            break;
        case 'x':
            /* use given executable */
            aArgv[0] = (char *) optarg;
            break;
        case 't':
            /* input state transitions */
            szProcStates = (char *) optarg;
            if (bUseLogFile) 
			{
                TSTLOG_appendLog(pHandle, "Using board states: \"%s\"\n",
                                 szProcStates);
            }
            TST_verbose("Using board states: \"%s\"\n", szProcStates);
            break;
        default:
            TST_verbose(("Invalid option: %c\n"), opt);
            cRetVal = 0;
            szErrorMsg = "Invalid option specified.";
            break;
        }
    }

if(cRetVal >0)
	
	{

while (DSP_SUCCEEDED(DSPManager_EnumProcessorInfo(index,&dspInfo,(UINT) sizeof (struct DSP_PROCESSORINFO),&numProcs)))
        {
                if (dspInfo.uProcessorType == IVA_ARM7)
                {
                        TSTLOG_appendLog(pHandle,"IVA device detected !! \n");
                        procId = index;
                        status = DSP_SOK;
                        break;
                }
                        TSTLOG_appendLog(pHandle,"Vlue of ndex is %d !! \n",index);
                index++;
        }

    /* Attach to IVA */
    if (DSP_SUCCEEDED(status)) 
      {
        status = DSPProcessor_Attach(procId,NULL,&ivaMsgTask.hProcessorIva);
        if (DSP_SUCCEEDED(status)) 
	{
            TSTLOG_appendLog(pHandle, "IVA:DSPProcessor_Attach succeeded.\n");
        }
        else 
	{
            TSTLOG_appendLog(pHandle, "IVA:DSPProcessor_Attach failed. Status = 0x%x\n",status);
        }
      }
        else
           TSTLOG_appendLog(pHandle, "Failed to get the desired processor \n");


    if (!DSP_SUCCEEDED(status)) 
	{
        cRetVal = 0;
        szErrorMsg = "Initial DSPProcessor_Attach():IVA failed";
        }

    /* Set initial state */
   /* status = DSPProcessor_GetState(hProcessor, procState, sizeof(struct DSP_PROCESSORSTATE));

    if (DSP_FAILED(status)) 
	{
        cRetVal = 0;
        szErrorMsg = "Initial DSPProcessor_GetState():IVA failed";
    }*/

    status = DSPProcessor_Detach(ivaMsgTask.hProcessorIva);

    if (DSP_FAILED(status)) 
	{
        cRetVal = 0;
        szErrorMsg = "Initial DSPProcessor_Detach():IVA failed";
    }

    ////if (cRetVal > 0) 
////	{
 ///   TST_verbose("Initiating IVAProcessor state testing\n");

    /* Process API list */
/*        pstrList[cListsize++] = strtok(szProcStates, SEPARATORS);
    while ((pstrList[cListsize++] = strtok(NULL, SEPARATORS)) != NULL);*/

    /* Adjust for NULL element */
    cListsize--;

    while (cRetVal > 0 && count < cListsize) 
	{
        switch(pstrList[count++][0]) 
		{
            case '1':
                /* DSPProcessor_Attach */
                iTransition = 1;
                status = DSPProcessor_Attach(iProc, pAttrIn, &ivaMsgTask.hProcessorIva);

                szErrorMsg = "Error in DSPProcessor_Attach():IVA";
                break;
            case '2':
				/* DSPProcessor_Detach */
                iTransition = 2;
                status = DSPProcessor_Detach(ivaMsgTask.hProcessorIva);
                szErrorMsg = "Error in DSPProcessor_Detach():IVA";
                break;
            case '3':
				/* DSPProcessor_EnumNodes */
                iTransition = 3;

				/* Allocate, Create, and Run a node */
				pszUuid = "7EEB2C7E_785A_11D4_A650_00C04F0C04F3";
                UUID_UuidFromString((CHAR *)pszUuid, &gidNodeID);
		gidNodeID=Iva_Msguuid;

                /* Allocate Node */
                status = DSPNode_Allocate(ivaMsgTask.hProcessorIva, &gidNodeID, NULL, NULL,
                                     &hNode);
                /* Create Node on DSP */
                status = DSPNode_Create(hNode);
                /* Start Node running on DSP */
                status = DSPNode_Run(hNode);
                status = DSPProcessor_EnumNodes(ivaMsgTask.hProcessorIva, aNodeTab, 50,
                         puNumNodes, puAllocated);
                szErrorMsg = "Error in DSPProcessor_EnumNodes():IVA";

				DSPNode_Terminate(hNode, &exitStatus);
				DSPNode_Delete(hNode);
                break;
            case '4':
				/* DSPProcessor_GetResourceInfo */
                iTransition = 4;
                status = DSPProcessor_GetResourceInfo(hProcessor, 0,
                                   &rsInfo, sizeof(struct DSP_RESOURCEINFO));
                szErrorMsg = "Error in DSPProcessor_GetResourceInfo():IVA";
                break;
            case '5':
				/* DSPProcessor_GetState */
                iTransition = 5;
                status = DSPProcessor_GetState(ivaMsgTask.hProcessorIva, procState,
                                               sizeof(struct DSP_PROCESSORSTATE));
                szErrorMsg = "Error in DSPProcessor_GetState():IVA";
                break;
            case '6':
				/* DSPProcessor_RegisterNotify */
                iTransition = 6;
                status = DSPProcessor_RegisterNotify(ivaMsgTask.hProcessorIva,
                struct DSP_PROCESSORSTATECHANGE, DSP_SIGNALEVENT, &hNotification);

                szErrorMsg = "Error in DSPProcessor_RegisterNotify():IVA";

                break;
            case '7':
				/* DSPProcessor_Load */
                iTransition = 7;
                status = DSPProcessor_Load(ivaMsgTask.hProcessorIva, iArgc, (CONST CHAR **)aArgv, 
								(CONST CHAR **)aEnvp);
                szErrorMsg = "Error in DSPProcessor_Load():IVA";
                break;
            case '8':
				/* DSPProcessor_Start */
                iTransition = 8;
                status = DSPProcessor_Start(ivaMsgTask.hProcessorIva);
                szErrorMsg = "Error in DSPProcessor_Start():IVA";
                break;
            case '9':
				/* DSPProcessor_Ctrl */
                iTransition = 9;
                status = DSPProcessor_Ctrl(ivaMsgTask.hProcessorIva, 0x8010, pArgs);
                szErrorMsg = "Error in DSPProcessor_Ctrl():IVA";
                break;
            default: TST_verbose("Invalid API call: %c\n", pstrList[cListsize-1][0]);
                break;
        } /* End of Switch Stmt */

        if (DSP_FAILED(status)) 
		{
            cRetVal = 0;
            break;
        }

       /* Get state and check for correctness, unless detached */
       if (iTransition != 2) 
	   {
          iPrevState = procState->iState;
          status = DSPProcessor_GetState(ivaMsgTask.hProcessorIva, procState,
                                         sizeof(struct DSP_PROCESSORSTATE));
       }
       else 
	   {
          continue;
       }

       if (DSP_FAILED(status)) 
	   {
           cRetVal = 0;
           szErrorMsg = "Error in DSPProcessor_GetState():IVA\n";
           TST_verbose("%x", status);
           break;
       }
       else 
	   {
           /* Check to see if we are in the correct state */
           switch(procState->iState) 
		   {
           case PROC_STOPPED:
               if (checkIvaTransProc(iPrevState, iTransition) != PROC_STOPPED) 
			   {
                   cRetVal = 0;
                   TSTLOG_appendLog(pHandle,
                            "\nTransition %i from state %i to %i invalid.\n",
                            iTransition, iPrevState, procState->iState);
               }
               break;
           case PROC_LOADED:
               if (checkIvaTransProc(iPrevState, iTransition) != PROC_LOADED) 
			   {
                   cRetVal = 0;
                   TSTLOG_appendLog(pHandle,
                           "\nTransition %i from state %i to %i invalid.\n",
                           iTransition, iPrevState, procState->iState);
               }
               break;
           case PROC_RUNNING:
               if (checkIvaTransProc(iPrevState, iTransition) != PROC_RUNNING) 
			   {
                   cRetVal = 0;
                   TSTLOG_appendLog(pHandle,
                           "\nTransition %i from state %i to %i invalid.\n",
                           iTransition, iPrevState, procState->iState);
               }
               break;
           default:
               cRetVal = 0;
               TSTLOG_appendLog(pHandle, "State %i doesn't exist!\n",
                                procState->iState);
               szErrorMsg = "Landed in unknown state!\n";
               break;
           }
       }

	} // end of while loop
    } // end of if stmt for cRetVal

    /* If Error, detach Processor */
    if (cRetVal == 0) 
	{
        DSPProcessor_Detach(ivaMsgTask.hProcessorIva);
    }

    OsSpecificMemFree(pArgs);
    OsSpecificMemFree(pAttrIn);
    OsSpecificMemFree(procState);
    OsSpecificMemFree(aNodeTab);
    OsSpecificMemFree(puNumNodes);
    OsSpecificMemFree(puAllocated);

    if (cRetVal == 0) 
	{
        /* Report in LOG module */
        if (bUseLogFile) 
		{
            TSTLOG_appendLog(pHandle, "FAIL\n");
            TSTLOG_appendLog(pHandle, "Error: %s\n", szErrorMsg);
        }
        TST_verbose("FAIL\n");
    }
    else 
	{
        /* Report in LOG module */
        if (bUseLogFile) 
		{
            TSTLOG_appendLog(pHandle, "PASS\n");
        }
        TST_verbose("PASS\n");
    }

    /* Close Log */
    if (bUseLogFile) 
	{
        TSTLOG_closeLog(pHandle);
    }

    return 0; /* dummy return - swamy */
}

/*
 *  ======== checkIvaTransProc ========
 *  This function is specific to the DSPProcessor Module.
 *  It is configured to calculate, given a present state and a transition,
 *  which state it should end up in.  This is used to check whether
 *  or not a given transition performed correctly.
 */
INT checkIvaTransProc(const INT iPrevState, const INT iTransition)
{
    INT iGoalState;

    switch(iTransition) 
	{
        case 1:
        case 2:
        case 3:
        case 4:
        case 5:
        case 6:
        case 9:
            iGoalState = iPrevState;
            break;
        case 7:
            iGoalState = PROC_LOADED;
            break;
        case 8:
            if (iPrevState == PROC_LOADED) 
			{
                iGoalState = PROC_RUNNING;
            }
            else 
			{
                iGoalState = iPrevState;
            }
            break;
        default:
            iGoalState = 0;
            break;
   }
   return iGoalState;
}

