/*  ============================================================================
 *   File    DmmCopy.c
 *
 *   Path    $(TESTSUITEROOT)\Analysis\
 *
 *   Desc  dmmcopy demonstrates usage of the Dynamic Memory Mapping APIs
 *         by creating two MPU buffers, mapping them to the DSP virtual
 *         address space, and copying data on the DSP from one buffer to
 *         another.
 *
 *   Rev     1.0.0
 *
 *   This computer program is copyright to Texas Instruments Incorporated.
 *   The program may not be used without the written permission
 *   of Texas Instruments Incorporated. or against the terms and conditions
 *   stipulated in the agreement under which this program has been
 *   supplied.
 *
 *   (c) Texas Instruments Incorporated 2004
 *
 *   ============================================================================
 *   Revision History
 *
 *   Nov 24, 2004  REF= Based on the DMMCOPY sample             Ramesh Gupta G
 *      Original Version.
 *   ============================================================================
 */

#include <stdio.h>
#include <dbapi.h>
#include <rms_sh.h>
#include <DSPProcessor_OEM.h>
#include <getopt_test.h>
#include <string.h>
#include <tst.h>
#include <tstlog.h>
#include <testosal.h>
#include <msgtxfr.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/mman.h>
#include <unistd.h>
#include <ctype.h>

#include "pwrsttest.h"

/* dmmcopy task context data structure. */
typedef struct
{
    DSP_HPROCESSOR  hProcessor; /* Handle to processor. */
    DSP_HNODE       hNode;      /* Handle to node. */
    INT             numIterations;
    INT             AccessFlag;
} DMMCOPY_TASK;

/* Forward declarations: */
/* Initialization and cleanup routines. */
static UINT 	    CheckDspPowerState();
static UINT       CheckDspTransition(UINT powerState);
static DSP_STATUS   InitializeDMMProcessor (DMMCOPY_TASK *copyTask, char *dspImage);
static DSP_STATUS   InitializeDMMNode (DMMCOPY_TASK *copyTask);
static DSP_STATUS   CleanupDMMProcessor (DMMCOPY_TASK *copyTask);
static DSP_STATUS   CleanupDMMNode (DMMCOPY_TASK *copyTask);
static DSP_STATUS   RunDMMTask (DMMCOPY_TASK *copyTask, ULONG size);


/*
 *  ======== main ========
 */
INT PowerStateTest_main (INT argc, char **argv)
{
    /*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/
    DMMCOPY_TASK    dmmcopyTask;
    DSP_STATUS      status = DSP_SOK;
    char            *dspImage;
    char            *buff_size;
    ULONG           BufferSize = 0;
    INT             opt = 0;
    INT             inumIterations;
    INT             iAccessFlag;
    INT		    testResult;
    INT 	    retVal;
    INT 	    iCount;
    INT       timeLimit = 0;
    INT       testNumber;
    INT       timeRetHib;
    /*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/

    DspManager_Open (argc, NULL);

    /* Process command line arguments, open data files: */
    optind = 0;
    while ((opt = getopt_test (argc, (char *const)argv, "+vl:x:n:t:ZPR:")) != EOF) {
        switch (opt) {
            case 'v':
                /* verbose */
                TST_setVerbose (TRUE);
                break;

            case 'l':
                /* use log and store with name location */
                strncpy (szLogName, (CHAR *)optarg, sizeof (szLogName));
                pHandle = TSTLOG_openLog (szLogName);
                fOpenLog = TRUE;
                TSTLOG_appendLog (pHandle, "Log File Set to: %s\n", szLogName);
                break;

            case 'x':
                /* use given executable */
                dspImage = (CHAR *)optarg;
                fLoadDsp = TRUE;
                break;
            
            case 'n':
                /* Number of Iterations */
                inumIterations = CSL_Atoi ((CHAR *)optarg);
                TSTLOG_appendLog (pHandle, "Number of iterations is set to :%s\n", optarg);
                break;

            case 't':
                /* Test Number  */
                testNumber = CSL_Atoi ((CHAR *)optarg);
                break;

            default:
                TSTLOG_appendLog (pHandle, "Invalid option: %c\n", opt);
                break;
        }
    }

    /* Time between Retention and Hibernation */
    timeRetHib = TIMEOUT_HIBERNATION - TIMEOUT_RETENTION;
    
    dmmcopyTask.hProcessor = NULL;
    dmmcopyTask.hNode = NULL;
    dmmcopyTask.numIterations = inumIterations;
    dmmcopyTask.AccessFlag = iAccessFlag;
    BufferSize =DEFAULTBUFFERSIZE;
    
    if (DSP_SUCCEEDED (status)) {

        /* Perform processor level initialization. */
        status = InitializeDMMProcessor (&dmmcopyTask, dspImage);
    }

    if (DSP_SUCCEEDED (status)) {

        /* Perform node level initialization. */
        status = InitializeDMMNode (&dmmcopyTask);
        if (DSP_SUCCEEDED (status)) {

            if (DSP_SUCCEEDED (status)) {
                TSTLOG_appendLog (pHandle, "RunDMMTask succeeded.\n");
            } else {
                TSTLOG_appendLog (pHandle, "RunDMMTask failed.\n");
            }
        }
    }        

    switch(testNumber){
        case 1:
            /* On-Retention Test */
            TSTLOG_appendLog (pHandle, "LOG: TESTARGS: On-Retention Test\n");
            
            /* Run task. */
            for (iCount = 0; iCount < dmmcopyTask.numIterations; iCount++) {
                status = RunDMMTask (&dmmcopyTask, BufferSize);
                retVal = CheckDspPowerState();
                
    	           if(retVal != ON_STATE && retVal != RET_STATE){
                      TSTLOG_appendLog (pHandle,"DSP is not able to enter into active state"
                                         "after a message and state is %d\n",retVal);
                       return DSP_EFAIL;
            	  } else {
                    TSTLOG_appendLog (pHandle, "Iteration %i\nActive "
                                      "state: PASS\n", iCount+1);                
                  }
    
                sleep(TIMEOUT_RETENTION);
                testResult = CheckDspTransition(RET_STATE);
                if (testResult) {
                    TSTLOG_appendLog (pHandle, "Retention state: FAIL\n", iCount+1);
                    break;
                } else {
                    TSTLOG_appendLog (pHandle, "Retention state: PASS\n", iCount+1);
                }
    		    }
    		    break;
        case 2:
            /* Off-On-Retention Test */
            /* Run task. */
            TSTLOG_appendLog (pHandle, "LOG: TESTARGS: Off-On-Retention Test\n");
            for (iCount = 0; iCount < dmmcopyTask.numIterations; iCount++) {
                status = RunDMMTask (&dmmcopyTask, BufferSize);
                retVal = CheckDspPowerState();
    
    	        if(retVal != ON_STATE && retVal != RET_STATE){
                    TSTLOG_appendLog (pHandle,"DSP is not able to enter into active state"
                                    "after a message and state is %d\n",retVal);
                    return DSP_EFAIL;
            	} else {
                    TSTLOG_appendLog (pHandle, "Iteration %i\nActive "
                                      "state: PASS\n", iCount+1);                
                }

                sleep(TIMEOUT_RETENTION);
                testResult = CheckDspTransition(RET_STATE);
                if (!testResult){
                    TSTLOG_appendLog (pHandle, "Retention state: PASS\n", iCount+1);
                    sleep(timeRetHib);
                    testResult = CheckDspTransition(HIB_STATE);
                    if (testResult) {
                        TSTLOG_appendLog (pHandle, "Retention state: FAIL\n", iCount+1);
                        break;
                    } else {
                        TSTLOG_appendLog (pHandle, "Hibernation state: PASS\n", iCount+1);
                    }
                } else {
                    TSTLOG_appendLog (pHandle, "Hibernation state: FAIL\n", iCount+1);
                    break;
                }
    	    }
    	    break;
        default:
            testResult = 1;
            break;
    } 
    
    if (!testResult)
        TSTLOG_appendLog (pHandle,"TESTRESULT: PASS\n");                
    else
        TSTLOG_appendLog (pHandle,"TESTRESULT: FAIL\n");
    
    status = CleanupDMMNode (&dmmcopyTask);
    if (DSP_SUCCEEDED (status)) {
        TSTLOG_appendLog (pHandle, "CleanupDMMNode succeeded.\n");
    } else {
        TSTLOG_appendLog (pHandle, "CleanupDMMNode Failed.\n");
    }

    status = CleanupDMMProcessor (&dmmcopyTask);
    if (DSP_SUCCEEDED (status)) {
        TSTLOG_appendLog (pHandle, "CleanupDMMProcessor succeeded.\n");
    } else {
        TSTLOG_appendLog (pHandle, "CleanupDMMProcessor Failed.\n");
    }

    return (DSP_SUCCEEDED (status) ? 0 : -1);
}

/*
 *  ======== InitializeDMMProcessor ========
 *  Perform processor related initialization.
 */
static DSP_STATUS InitializeDMMProcessor (DMMCOPY_TASK *copyTask, char *dspImage)
{
    DSP_STATUS          status = DSP_EFAIL;
    INT                 argc = 1;
    CHAR                *argv = DEFAULTDSPEXEC;
    struct DSP_PROCESSORINFO   dspInfo;
    INT                 numProcs;
    UINT                index = 0;
    INT                 procId;

    /* Attach to DSP */
    while (DSP_SUCCEEDED (DSPManager_EnumProcessorInfo (index, &dspInfo, (UINT) sizeof (struct DSP_PROCESSORINFO), &numProcs))) {
        if (dspInfo.uProcessorType == DSPTYPE_55 || dspInfo.uProcessorType == DSPTYPE_64) {
            procId = index;
            status = DSP_SOK;
            break;
        }

        index++;
    }

    /* Attach to an available DSP (in this case, the 1st DSP). */
    if (DSP_SUCCEEDED (status)) {
        status = DSPProcessor_Attach (procId, NULL, &copyTask->hProcessor);
        if (!DSP_SUCCEEDED (status)) {
            TSTLOG_appendLog (pHandle, "DSPProcessor_Attach failed. Status = 0x%x\n", status);
        }
    } else
        TSTLOG_appendLog (pHandle, "Failed to get the desired processor \n");
    
    if (DSP_SUCCEEDED (status)) {
        status = DSPProcessor_Load (copyTask->hProcessor, argc, &dspImage, NULL);
        if (!DSP_SUCCEEDED (status)) {
            TSTLOG_appendLog (pHandle, "DSPProcessor_Load failed. and status is %x\n", status);
        }

        status = DSPProcessor_Start (copyTask->hProcessor);
        if (!DSP_SUCCEEDED (status)) {
            TSTLOG_appendLog (pHandle, "DSPProcessor_Start failed.\n");
        }
    }

    return (status);
}

/*
 *  ======== InitializeDMMNode ========
 *  Perform node related initialization.
 */
static DSP_STATUS InitializeDMMNode (DMMCOPY_TASK *copyTask)
{
    BYTE            argsBuf[ARGSIZE + sizeof (ULONG)];
    struct DSP_CBDATA      *pArgs;
    struct DSP_NODEATTRIN  nodeAttrIn;
    struct DSP_UUID        uuid;
    DSP_STATUS      status = DSP_SOK;
    DWORD       dwStartTime;
    DWORD       dwElapsedTime;
    DWORD       dwEndTime;
    ULONG       ulNumBytesTransferred = 0;
    long double ulBytesPerSec = 0;
    long double ldElapsedTime;
    uuid = Msguuid;
    nodeAttrIn.uTimeout = DSP_FOREVER;
    nodeAttrIn.iPriority = 5;
    nodeAttrIn.uProfileID = 0xff;
    pArgs = (struct DSP_CBDATA *)argsBuf;
    pArgs->cbData = ARGSIZE;
    pArgs->cData[0] = 0;

    /* Allocate the dmmcopy node. */
    if (DSP_SUCCEEDED (status)) {
        status = DSPNode_Allocate (copyTask->hProcessor, &uuid, pArgs, &nodeAttrIn, &copyTask->hNode);
        if (!DSP_SUCCEEDED (status)) {
            TSTLOG_appendLog (pHandle, "DSPNode_Allocate failed. Status = 0x%x\n", status);
        }
    }

    /* Create the dmmcopy node on the DSP. */
    if (DSP_SUCCEEDED (status)) {
        status = DSPNode_Create (copyTask->hNode);
        if (!DSP_SUCCEEDED (status)) {
            TSTLOG_appendLog (pHandle, "DSPNode_Create failed. Status = 0x%x\n", status);
        }
    }

    /* Start the dmmcopy node on the DSP. */
    if (DSP_SUCCEEDED (status)) {
        status = DSPNode_Run (copyTask->hNode);
        if (!DSP_SUCCEEDED (status)) {
            TSTLOG_appendLog (pHandle, "DSPNode_Run failed. Status = 0x%x\n", status);
        }
    }

    return (status);
}

/*
 *  ======== RunDMMTask ========
 *  Run dmmcopy task.
 */
static DSP_STATUS RunDMMTask (DMMCOPY_TASK *copyTask, ULONG BufferSize)
{
    INT         cBytesRead = 0;
    DSP_STATUS  status = DSP_SOK;

    /* Actual MPU Buffer addresses */
    ULONG       aBufferSend = 0;

    /* Reserved buffer DSP virtual addresses (bytes) */
    ULONG       aDspSendBuffer = 0;

    /* Base addresses of reserved DSP virtual address ranges (bytes) */
    ULONG       dspAddrSend = 0;

    /* MPU buffer size */
    ULONG       ulSendBufferSize;

    /* DSP reserve size = buffersize + 4K page size */
    ULONG       ulSendResv;

    /* Messaging used for GPP/DSP synchronization */
    struct DSP_MSG     msgToDsp;
    struct DSP_MSG     msgFromDsp;
    INT         iCount = 0;
    DWORD       dwStartTime;
    DWORD       dwElapsedTime;
    DWORD       dwEndTime;
    ULONG       ulNumBytesTransferred = 0;
    long double ulBytesPerSec = 0;
    long double ldElapsedTime;
    INT		j =0;
    /* Variables for DMM Throughput*/
    /* Actual MPU Buffer addresses */
    ULONG         aBufferRecv = 0;
    /* Reserved buffer DSP virtual addresses (bytes) */
    ULONG         aDspRecvBuffer = 0;
    /* Base addresses of reserved DSP virtual address ranges (bytes) */
    ULONG         dspAddrRecv = 0;
    /* MPU buffer size */
    ULONG         ulRecvBufferSize;
    /* DSP reserve size = buffersize + 4K page size */
    ULONG         ulRecvResv;
    /* Messaging used for GPP/DSP synchronization */
    struct DSP_MSG     msgToDsp2;

    /* Allocate MPU Buffers */
    ulSendBufferSize = BufferSize;
    ulSendResv = ulSendBufferSize + 0x1000;
    aBufferSend = (ULONG) malloc (ulSendBufferSize);

    if (!aBufferSend) {
        TSTLOG_appendLog (pHandle, "Failed to allocate MPU buffers.\n");
        status = DSP_EMEMORY;
    } else {

        /* Initialize buffers */
        for (iCount = 0; (UINT) iCount < ulSendBufferSize; iCount++) {
            ((PSTR) aBufferSend)[iCount] = 0;
        }

        /* Reserve DSP virtual memory for send buffer */
        status = DSPProcessor_ReserveMemory (copyTask->hProcessor, ulSendResv, (PVOID *) &dspAddrSend);
        if (!DSP_SUCCEEDED (status)) {
            TSTLOG_appendLog (pHandle, "DSPProcessor_ReserveMemory failed. Status = 0x%x\n", status);
        }
    }
            
    /* Setting command to READ/WRITE */
		msgToDsp.dwCmd = 0xDCBD;
		msgToDsp2.dwCmd = 0xADDD;
    /* DMM Throughput case */
    /* Allocate MPU Buffers */
    ulRecvBufferSize = ulSendBufferSize;
    aBufferRecv = (ULONG) malloc(ulRecvBufferSize);
    ulRecvResv = ulSendResv;

    if (!aBufferRecv) {
        TSTLOG_appendLog (pHandle, "Failed to allocate MPU buffers.\n");
        status = DSP_EMEMORY;
    } else {
        
        /* Initialize buffers */
        for (iCount = 0; (UINT) iCount < ulRecvBufferSize; iCount++) {
            ((PSTR) aBufferRecv)[iCount] = 0;
        }

        /* Reserve DSP virtual memory for send buffer */
        status = DSPProcessor_ReserveMemory (copyTask->hProcessor, ulRecvResv, (PVOID *) &dspAddrRecv);
        if (!DSP_SUCCEEDED (status)) {
            TSTLOG_appendLog (pHandle, "DSPProcessor_ReserveMemory2 failed. Status = 0x%x\n", status);
        }
    }

    /* Run DMM Copy for iCount times */
    if (DSP_SUCCEEDED (status)) {
    
    /* Map MPU "send buffer" to DSP "receive buffer" virtual address */
    status = DSPProcessor_Map (copyTask->hProcessor, (PVOID) aBufferSend, ulSendBufferSize, (PVOID) dspAddrSend, (PVOID *) &aDspSendBuffer, 0);
        if (!DSP_SUCCEEDED (status)) {
            TSTLOG_appendLog (pHandle, "DSPProcessor_Map failed.Status = 0x%x\n", status);
        }
    }
            
    if (DSP_SUCCEEDED (status)) {

        /* Map MPU "receive buffer" to DSP "send buffer" virtual address */
        status = DSPProcessor_Map (copyTask->hProcessor, (PVOID) aBufferRecv, ulRecvBufferSize, (PVOID) dspAddrRecv, (PVOID *) &aDspRecvBuffer, 0);
            if (!DSP_SUCCEEDED (status)) {
                TSTLOG_appendLog (pHandle, "DSPProcessor_Map failed.Status = 0x%x\n", status);
            }
    }        

    msgToDsp.dwArg1 = (DWORD) aDspSendBuffer / DSPWORDSIZE;
    msgToDsp.dwArg2 = (DWORD) aDspRecvBuffer / DSPWORDSIZE;
    msgToDsp2.dwArg1 = (DWORD) BufferSize / DSPWORDSIZE;
    msgToDsp2.dwArg2 = (DWORD) BufferSize / DSPWORDSIZE;

    if (DSP_SUCCEEDED (status)) {
        status = DSPProcessor_FlushMemory (copyTask->hProcessor, (PVOID) aBufferSend, ulSendBufferSize, 0);
        if (DSP_FAILED (status)) {
            TSTLOG_appendLog (pHandle, "DSPProcessor_FlushMemory failed.Status = 0x%x\n", status);
            return DSP_EFAIL;
        }
    }
          
    if (DSP_SUCCEEDED (status)) {

        /* Notify DSP of word-adjusted buffer addresses */
        status = DSPNode_PutMessage (copyTask->hNode, &msgToDsp, DSP_FOREVER);
        if (DSP_FAILED (status)) {
            TSTLOG_appendLog (pHandle, "DSPProcessor_PutMessage failed.Status = 0x%x\n", status);
            return DSP_EFAIL;
        }
    
        status = DSPNode_PutMessage (copyTask->hNode, &msgToDsp2, DSP_FOREVER);
        if (DSP_FAILED (status)) {
            TSTLOG_appendLog (pHandle, "DSPProcessor_PutMessage2 failed.Status = 0x%x\n", status);
            return DSP_EFAIL;
        }
                    
        if (msgToDsp.dwCmd != 0xDCBA) {
            status = DSPNode_GetMessage (copyTask->hNode, &msgFromDsp, DSP_FOREVER);
            if (DSP_FAILED (status)) {
                TSTLOG_appendLog (pHandle, "DSPProcessor_GetMessage failed.Status = 0x%x\n", status);
                return DSP_EFAIL;
            }
        }
    }

            if (DSP_SUCCEEDED (status)) {
                status = DSPProcessor_InvalidateMemory (copyTask->hProcessor, (PVOID) aBufferRecv, ulRecvBufferSize);
                if (DSP_FAILED (status)) {
                    TSTLOG_appendLog (pHandle, "DSPProcessor_InvalidateMemory failed.Status = 0x%x\n", status);
                    return DSP_EFAIL;
                }
            }


          for (j = 0; j < ulSendBufferSize; j++) {
                 if (((unsigned char *)aBufferSend)[j] != ((unsigned char *)aBufferRecv)[j]) {
                 fprintf(stdout, "Sent and received buffers don't match \n");
                 fprintf(stdout, "Press any key to continue \n");
                 (void)getchar();
                  return DSP_EFAIL;
                }
	
               }

            
            status = DSPProcessor_UnMap (copyTask->hProcessor, (PVOID) aDspSendBuffer);
            if (DSP_FAILED (status)) {
                TSTLOG_appendLog (pHandle, "DSPProcessor_UnMap failed.Status = 0x%x\n", status);
                return DSP_EFAIL;
            }

            status = DSPProcessor_UnMap (copyTask->hProcessor, (PVOID) aDspRecvBuffer);
            if (DSP_FAILED (status)) {
                TSTLOG_appendLog (pHandle, "DSPProcessor_UnMap2 failed.Status = 0x%x\n", status);
                return DSP_EFAIL;
            }


    status = DSPProcessor_UnReserveMemory (copyTask->hProcessor, (PVOID) dspAddrSend);
    if (!DSP_SUCCEEDED (status)) {
        TSTLOG_appendLog (pHandle, "DSPProcessor_UnReserveMemory failed. Status = 0x%x\n", status);
    }

    /* Free MPU buffers (at originally allocated addresses) */
    if (aBufferSend) {
        free ((VOID *)aBufferSend);
    }

    if(dspAddrRecv) {
        status = DSPProcessor_UnReserveMemory (copyTask->hProcessor, (PVOID) dspAddrRecv);
        if (!DSP_SUCCEEDED (status)) {
            TSTLOG_appendLog (pHandle, "DSPProcessor_UnReserveMemory2 failed. Status = 0x%x\n", status);
        }
    }

    /* Free MPU buffers (at originally allocated addresses) */
    if (aBufferRecv) {
        free ((VOID *)aBufferRecv);
    }

    return (status);
}

/*
 *  ======== CleanupDMMNode ========
 *  Perform node related cleanup.
 */
static DSP_STATUS CleanupDMMNode (DMMCOPY_TASK *copyTask)
{
    DSP_STATUS  status;
    DSP_STATUS  exitStatus;
    if (copyTask->hNode) {

        /* Terminate DSP node */
        status = DSPNode_Terminate (copyTask->hNode, &exitStatus);
        if (!DSP_SUCCEEDED (status)) {
            TSTLOG_appendLog (pHandle, "DSPNode_Terminate failed: 0x%x\n", status);
        }

        /* Delete DSP node */
        status = DSPNode_Delete (copyTask->hNode);
        if (!DSP_SUCCEEDED (status)) {
            TSTLOG_appendLog (pHandle, "DSPNode_Delete failed: 0x%x\n", status);
        } else {
            TSTLOG_appendLog (pHandle, "DSPNode_Delete PASS: 0x%x\n", status);
        }

        copyTask->hNode = NULL;
    }

    return (status);
}

/*
 *  ======== CleanupDMMProcessor ========
 *  Perform processor related cleanup.
 */
static DSP_STATUS CleanupDMMProcessor (DMMCOPY_TASK *copyTask)
{
    DSP_STATUS  status;
    if (copyTask->hProcessor) {

        /* Detach from processor. */
        status = DSPProcessor_Detach (copyTask->hProcessor);
        if (!DSP_SUCCEEDED (status)) {
            TSTLOG_appendLog (pHandle, "DSPProcessor_Detach failed.\n");
        }

        copyTask->hProcessor = NULL;
    }

    return (status);
}


static UINT CheckDspPowerState()
{
        INT fd;
        UINT prm_base = 0;
        UINT cm_base = 0;
        UINT grp_base = 0;
        UINT regVal;
        UINT pm_state;
        UINT ret_state =-1;
        DSP_STATUS status = DSP_SOK;

        fd = open("/dev/mem", O_RDWR|O_SYNC);
        if (fd) {
                prm_base = (UINT) (CHAR *)mmap(0, MAP_SIZE, PROT_READ | PROT_WRITE,
                                                          MAP_SHARED, fd, (UINT)OMAP_IVA2_PRM_BASE);
                cm_base = (UINT) (CHAR *)mmap(0,MAP_SIZE, PROT_READ | PROT_WRITE,
                                                          MAP_SHARED, fd, (UINT)OMAP_IVA2_CM_BASE);
                close(fd);
        }
        else {
                fprintf(stdout,"Failed to open the mmap device \n");
                return DSP_EFAIL;
        }

        if (prm_base) {
                regVal = *((UINT*)(prm_base + 0xe4));

/*
                fprintf(stdout, "PM_PWSTST_IVA2(0x%x) = 0x%x \n",
                                (OMAP_IVA2_PRM_BASE + 0xe4),regVal);
*/                                
                pm_state=regVal & 0x00f;
                switch(pm_state){
                        case 6:
                            /* Falling through */
                        case 7:
                            ret_state = ON_STATE; //ON
                        break;

                        case 5:
                            ret_state = RET_STATE; //RET
                        break;

                        case 4:
                            /* Falling through */
                        case 0:
                            ret_state = HIB_STATE; //OFF
                        break;

                        default:
                            fprintf(stdout,"Invalid state please check\n");
                        break;
                        }
        }
        if (munmap((void*)prm_base,MAP_SIZE) == -1)
                printf("prcm Memory Unmap failed \n");
        if (munmap((void*)cm_base,MAP_SIZE) == -1)
                printf("prcm Memory Unmap failed \n");

        return ret_state;
}

static UINT CheckDspTransition (UINT powerState)
{
    INT        retVal;
    INT        timeLimit = 0;
    INT        targetState;
    
    targetState = powerState;
    retVal = CheckDspPowerState();
          
    while (retVal != powerState){
    usleep(10000);        /* 10 ms */
		    
        /* Wait 3 secs to break */
        if (timeLimit==300){
            fprintf(stdout,"Power State Transition: FAILED\n");
            break;
	}
        retVal = CheckDspPowerState();
        timeLimit++;
    }
    //fprintf(stdout,"Change after %i ms\n", timeLimit*10);
    timeLimit = 0;

    if(retVal != targetState){
        fprintf(stdout,"Power State  transition is not happened ,"
                " please check PM status of the image retVal = %d and targetState = %d\n",retVal,targetState);
        return DSP_EFAIL;
    }
    
    return 0;
}

