/*
 *  Copyright 2004 by Texas Instruments Incorporated.
 *  All rights reserved. Property of Texas Instruments Incorporated.
 *  Restricted rights to use, duplicate or disclose this code are
 *  granted through contract.
 *
 *  @(#) DSP/BIOS Bridge 
 */
/*  ============================================================================
 *   File    LowResource.c
 *
 *   Path    $(TESTSUITEROOT)\Stress\LowResource\LowResource.c
 * 
 *   Desc:
 *           This test case stress tests the Bridge by 
 *           consuming memory in chunks and testing it for proper
 *           functioning under these low resource conditions.
 *
 *  Usage:
 *        LowResource [-v] [-l "log path/filename"] 
 *                      [-x "valid executable"] ..
 *
 *        -v: verbose mode
 *        -l: use log and location
 *        -x: DSP Image to be used
 *        -c: Chunk size to be consumed each iteration(bytes)
 *        -n: Maximum number of Iterations, after which the test is stopped
 *        -s: Buffer Size
 *
 *   ---------------------------------------------------------------------------
 *   Rev     1.0.0
 * 
 *   This computer program is copyright to Texas Instruments Incorporated.
 *   The program may not be used without the written permission
 *   of Texas Instruments Incorporated. or against the terms and conditions
 *   stipulated in the agreement under which this program has been
 *   supplied.
 *
 *   (c) Texas Instruments Incorporated 2003
 *
 *   ============================================================================
 *   Revision History
 *
 *   AUG 07, 2002    REF=ORG             Swamy MC
 *      Original Version.
 *   ============================================================================
 */


#include <stdio.h>
#include <dbapi.h>
#include <getopt_test.h>
#include <tst.h>
#include <tstlog.h>
#include <uuidutil.h>
#include <testosal.h>
#include <DSPProcessor_OEM.h>

#include "LowResource.h"

#define DEFNAMESIZE        100
#define NUMOFCOMPARGS      3	/* Compulsory args: c,n,s */
#define NUMTRANSFERS       10
#define DEFBUFSIZE         64
#define NONE               0
#define FAILURE            1
#define PASS               0
#define DEFAULTDSPUNIT     0
#define SHORT short
static int     funccount = 1;

#define DSPEXEC        "inout_nortdxidl.dof"


/* Global variables */
ListElem *firstElem; /* address of first elem in linked list */
BOOL           bUseLogLowRes=FALSE;
FILE           *pHandleLowRes;
static BOOL		fLoadDsp = FALSE;

/* Local function declaration */
INT delAllNodes(ListElem *elem);



/* Set of functions to implement a linked list which is used
 * to consume memory on GPP side and cause a Low Resource condition
 * retval -> 0 success, 1 fail.
 */



INT delAllNodes(ListElem *elem)
{
    if(funccount == 1)
    	TST_verbose("Entering DelAll Nodes\n");
    if(elem->next != NULL)
	{
        delAllNodes(elem->next);
        funccount++;
	}
    else
    {
        OsSpecificMemFree(elem);
        TST_verbose("Number of Nodes deleted = %d\n", funccount);
        
    }
        
    return 0;

}


INT addNodeToList(INT sizeOfBuff)
{
	static ListElem *prevElem=NULL;
	ListElem *elem;


	if (prevElem == NULL)
	{
		/* First time */
		firstElem = (ListElem *) OsSpecificMemAlloc(sizeof(ListElem));
		if (firstElem == NULL)
		{
			/* error - add a print statement */
			return FAILURE;
		}
		prevElem = firstElem;
	}

	elem = (ListElem *) OsSpecificMemAlloc(sizeof(ListElem));
	if (elem == NULL)
	{
		/* error */
        TST_verbose("ELEM == NULL. Returning Error\n");
		return FAILURE;
	}
	prevElem->next = elem;

	elem->content = (char *)OsSpecificMemAlloc(sizeOfBuff);
	elem->next = NULL;

	prevElem=elem;

	return PASS;
} /* end of addNodeToList */



static DSP_STATUS SendTestConfiguration(SHORT **pInBuf, 
										SHORT **pOutBuf, 
										DSP_HSTREAM	hOutStream, 
										DWORD dwSendBufSize, 
										DWORD dwRecvBufSize)
{

		DSP_STATUS status = DSP_SOK;
		UINT			k;
		DWORD			dwArg;
		DWORD			dummy;

		(*pOutBuf)[0] = dwSendBufSize / 2;
		(*pOutBuf)[1] = dwRecvBufSize / 2;
		(*pOutBuf)[2] = NUMTRANSFERS & 0xffff ;
		(*pOutBuf)[3] = NUMTRANSFERS >> 16 ;

		for ( k = 4; k < dwSendBufSize / 2; k++)
		{
				(*pInBuf)[k] = k;
		}

		status = DSPStream_Issue(hOutStream, (BYTE*)*pOutBuf, 
        dwSendBufSize, dwRecvBufSize, 0);
        if (!DSP_SUCCEEDED(status)) 
		{
            if(bUseLogLowRes)
				TSTLOG_appendLog(pHandleLowRes, "DSPStream_Issue %d failed.\n", k);
        }

		status = DSPStream_Reclaim(hOutStream, 
						(BYTE **)pOutBuf, &dwRecvBufSize, &dummy, &dwArg);
		return status;
}


/* -------------------------------------------------- */
/* Check Bridge Status part of the program            */
/* -------------------------------------------------- */

INT checkBridgeStatus(CHAR **aArgv, INT iBuffSize)
{
    
    
    DSP_STATUS     status;
    ULONG          tmpDataSize;
    ULONG          tmpBuffSize;
    DWORD          tmpArg;
    INT            iNodeCount;
    struct DSP_PROCESSORATTRIN		*pAttrIn;
    DSP_HPROCESSOR			hProcessor;
    INT						iArgc = 1;
    struct DSP_UUID				gidNodeID;
    struct DSP_NODEATTRIN			nodeAttrin;
    DSP_HNODE				hNode;
    struct DSP_STRMATTR			attrs;
    DSP_HSTREAM				hStreamOut;
    DSP_HSTREAM				hStreamIn;
    BYTE					**arrBufOut;
    BYTE					**arrBufIn;
    INT						i;
    DSP_STATUS          	exitStatus = DSP_SOK;
    INT                 	retVal = FAILURE;
    SHORT					* pData ;
    



    gidNodeID = lowresuuid;

    /* Initialising attributes */
    attrs.uSegid = 0;
    attrs.uBufsize = 512;
    attrs.lMode = STRMMODE_PROCCOPY;
    attrs.uNumBufs = 1;
    attrs.uAlignment = 0;
    attrs.uTimeout = 5000;

	nodeAttrin.cbStruct = sizeof(struct DSP_NODEATTRIN);
    nodeAttrin.uTimeout = 1000;
    nodeAttrin.iPriority = 1;
#if defined(OMAP_2430)|| defined(OMAP_3430)
    nodeAttrin.uProfileID = 0xff;
#endif


	iNodeCount = 0; /* Initialise */


	arrBufOut = OsSpecificMemAlloc(sizeof(BYTE *));
	arrBufIn = OsSpecificMemAlloc(sizeof(BYTE *));

    TST_verbose("DSP Executable: %s\n", aArgv[0]);

    pAttrIn = OsSpecificMemAlloc(sizeof(struct DSP_PROCESSORATTRIN));
	if (pAttrIn == NULL)
	{
		/* Error */
		TST_verbose("Malloc for pAttrIn FAILED !!\n");
		return retVal;
	}
    pAttrIn->cbStruct = sizeof(struct DSP_PROCESSORATTRIN);
    pAttrIn->uTimeout = 50;

    /* Attach to processor */
    struct DSP_PROCESSORINFO       dspInfo;
    UINT                    numProcs;
    UINT                    index=0;
    INT                     procId=0;
    
    while (DSP_SUCCEEDED(DSPManager_EnumProcessorInfo(index,&dspInfo, (UINT) sizeof (struct DSP_PROCESSORINFO),&numProcs))) {
        if (dspInfo.uProcessorType == DSPTYPE_55 || dspInfo.uProcessorType == DSPTYPE_64) {
            TSTLOG_appendLog(pHandleLowRes,"DSP device detected !! \n");
            procId = index;
            status = DSP_SOK;
            break;
        }
        index++;
    }

    /* Attach to an available DSP (in this case, the 1st DSP). */
    if (DSP_SUCCEEDED(status)) {
        
        status = DSPProcessor_Attach(procId, pAttrIn, &hProcessor);
        if (DSP_SUCCEEDED(status)) {
            TSTLOG_appendLog(pHandleLowRes, "DSPProcessor_Attach succeeded.\n");
        }
        else {
            TSTLOG_appendLog(pHandleLowRes, "DSPProcessor_Attach failed. Status = 0x%x\n",status);
        }
    }
    else {
        TSTLOG_appendLog(pHandleLowRes, "Failed to get the desired processor \n");
    }
    
    if (!DSP_SUCCEEDED(status)) 
	{
        /* Report Error */
        TST_verbose("FAIL\n");

        if (bUseLogLowRes) 
		{
            TSTLOG_appendLog(pHandleLowRes, "FAIL\n");
            TSTLOG_appendLog(pHandleLowRes,
            "Error in DSPProcessor_Attach()\n");
        }
        
		return retVal;
    }

	
    if(fLoadDsp)
    {
    /* Load default program */
    status = DSPProcessor_Load(hProcessor, iArgc, (CONST CHAR **)aArgv, NULL);
    if (!DSP_SUCCEEDED(status)) 
	{
		TSTLOG_appendLog(pHandleLowRes, "Status Val: %x\n",status);
        /* Report Error */
        TST_verbose("FAIL\n");
        if (bUseLogLowRes) 
		{
            TSTLOG_appendLog(pHandleLowRes, "FAIL\n");
            TSTLOG_appendLog(pHandleLowRes,
            "Error in DSPProcessor_Load()\n");
        }
		
        delAllNodes(firstElem);
		DSPProcessor_Detach(hProcessor);
		return retVal;
    }

    status = DSPProcessor_Start(hProcessor);
    if (!DSP_SUCCEEDED(status)) 
	{
        /* Report Error */
        TST_verbose("FAIL\n");
        if (bUseLogLowRes) 
		{
            TSTLOG_appendLog(pHandleLowRes, "FAIL\n");
            TSTLOG_appendLog(pHandleLowRes,
            "Error in DSPProcessor_Start()\n");
        }
        
        delAllNodes(firstElem);
		DSPProcessor_Detach(hProcessor);
		return retVal;
    }
    }


    /* Allocate Node */
    status = DSPNode_Allocate(hProcessor, &gidNodeID, NULL, &nodeAttrin,
                   &hNode);
	TST_verbose("DSPNode_Allocate ret val = %x\n",status);

    if (!DSP_SUCCEEDED(status)) 
	{
        /* Report Error */
        TST_verbose("FAIL\n");
        if (bUseLogLowRes) 
		{
            TSTLOG_appendLog(pHandleLowRes, "FAIL status = 0x%x\n", status);
            TSTLOG_appendLog(pHandleLowRes,"Error in DSPNode_Allocate().\n");
        }
        
        
        delAllNodes(firstElem);
		DSPProcessor_Detach(hProcessor);
		return retVal;
    }

    status = DSPNode_Connect((DSP_HNODE)DSP_HGPPNODE, 0, hNode, 0,
                             &attrs);

    if (!DSP_SUCCEEDED(status)) 
	{
        /* Report Error */
        TST_verbose("FAIL\n");

        if (bUseLogLowRes) 
		{
            TSTLOG_appendLog(pHandleLowRes, "FAIL\n");
            TSTLOG_appendLog(pHandleLowRes,"Error in DSPNode_Connect()\n");
        }
        
        
        delAllNodes(firstElem);
   		DSPProcessor_Detach(hProcessor);

		return retVal;
    }

    status = DSPNode_Connect(hNode, 0, (DSP_HNODE)DSP_HGPPNODE, 0,
                             &attrs);

    if (!DSP_SUCCEEDED(status)) 
	{
        /* Report Error */
        TST_verbose("FAIL\n");

        if (bUseLogLowRes) 
		{
            TSTLOG_appendLog(pHandleLowRes, "FAIL\n");
            TSTLOG_appendLog(pHandleLowRes,"Error in DSPNode_Connect()\n");
        }
        
        
        delAllNodes(firstElem);
   		DSPProcessor_Detach(hProcessor);

		return retVal;
    }


    /* Create Node on DSP */
    status = DSPNode_Create(hNode);

    if (!DSP_SUCCEEDED(status)) 
	{
        /* Report Error */
        TST_verbose("FAIL\n");

        if (bUseLogLowRes) 
		{
            TSTLOG_appendLog(pHandleLowRes, "FAIL\n");
            TSTLOG_appendLog(pHandleLowRes,"Error in DSPNode_Create()\n");
        }
        
        
        delAllNodes(firstElem);
   		DSPProcessor_Detach(hProcessor);

		return retVal;
    }

    status = DSPNode_Run(hNode);

    if (!DSP_SUCCEEDED(status)) 
	{
        /* Report Error */
        TST_verbose("FAIL\n");

        if (bUseLogLowRes) 
		{
            TSTLOG_appendLog(pHandleLowRes, "FAIL\n");
            TSTLOG_appendLog(pHandleLowRes,"Error in DSPNode_Run()\n");
        }
        
        
        delAllNodes(firstElem);
   		DSPProcessor_Detach(hProcessor);
		return retVal;
    }

    status = DSPStream_Open(hNode, DSP_TONODE, 0, NULL, &hStreamOut);

    if (!DSP_SUCCEEDED(status)) 
	{
        /* Report Error */
        TST_verbose("FAIL\n");

        if (bUseLogLowRes) 
		{
            TSTLOG_appendLog(pHandleLowRes, "FAIL\n");
            TSTLOG_appendLog(pHandleLowRes,"Error in DSPStraem_Open()\n");
        }
        
        delAllNodes(firstElem);
   		DSPProcessor_Detach(hProcessor);
		return retVal;
    }

    status = DSPStream_Open(hNode, DSP_FROMNODE, 0, NULL, &hStreamIn);

    if (!DSP_SUCCEEDED(status)) 
	{
        /* Report Error */
        TST_verbose("FAIL\n");

        if (bUseLogLowRes) 
		{
            TSTLOG_appendLog(pHandleLowRes, "FAIL\n");
            TSTLOG_appendLog(pHandleLowRes,"Error in DSPStraem_Open()\n");
        }
        
        delAllNodes(firstElem);
   		DSPProcessor_Detach(hProcessor);
		return retVal;
    }



    status = DSPStream_AllocateBuffers(hStreamOut, iBuffSize, arrBufOut, 1);

    if (!DSP_SUCCEEDED(status)) 
	{
        /* Report Error */
        TST_verbose("FAIL\n");

        if (bUseLogLowRes) 
		{
            TSTLOG_appendLog(pHandleLowRes, "FAIL\n");
            TSTLOG_appendLog(pHandleLowRes,"Error in DSPStream_AllocateBuffers()\n");
        }
        
        delAllNodes(firstElem);
   		DSPProcessor_Detach(hProcessor);
		return retVal;
    }

    status = DSPStream_AllocateBuffers(hStreamIn, iBuffSize, arrBufIn, 1);

    if (!DSP_SUCCEEDED(status)) 
	{
        /* Report Error */
        TST_verbose("FAIL\n");

        if (bUseLogLowRes) 
		{
            TSTLOG_appendLog(pHandleLowRes, "FAIL\n");
            TSTLOG_appendLog(pHandleLowRes,"Error in DSPStream_AllocateBuffers()\n");
        }
        
        delAllNodes(firstElem);
   		DSPProcessor_Detach(hProcessor);
		return retVal;
    }


	pData = (SHORT *)arrBufOut[0];

	pData[0] = (SHORT) (iBuffSize/2);
	pData[1] = (SHORT) (iBuffSize/2);


    status = SendTestConfiguration((SHORT **)&arrBufIn[0], (SHORT **)&arrBufOut[0], 
					hStreamOut, iBuffSize, iBuffSize);

    /* fill buffer */
    for (i = 2; i < (iBuffSize/2); i++) 
	{
		pData[i] = (SHORT) i;
    }

	/* Check for Data Transfer */

        /*  Data transfer for number of iterations .. specified by the
         *  user as parameters to the test ... set in the constructor
	     */
        for (i = 0 ; i < NUMTRANSFERS ; i++)
	    {
			/* Issue Data */
			status = DSPStream_Issue(hStreamOut, arrBufOut[0], iBuffSize, iBuffSize, 0);
			TST_verbose("DSPStream_Issue %d ret val = %x\n",i,status);

		    if (!DSP_SUCCEEDED(status)) 
			{
		        /* Report Error */
		        TST_verbose("FAIL\n");
	
				if (bUseLogLowRes) 
				{
					TSTLOG_appendLog(pHandleLowRes, "FAIL\n");
					TSTLOG_appendLog(pHandleLowRes,"Error in DSPStraem_Issue()\n");
				}
                delAllNodes(firstElem);
   		        DSPProcessor_Detach(hProcessor);
				return retVal;
			}

			/* Reclaim Data */
			status = DSPStream_Reclaim(hStreamOut, arrBufOut, &tmpDataSize, &tmpBuffSize, &tmpArg);
			TST_verbose("DSPStream_Reclaim %d ret val = %x\n",i,status);

		    if (!DSP_SUCCEEDED(status)) 
			{
		        /* Report Error */
		        TST_verbose("FAIL\n");
	
				if (bUseLogLowRes) 
				{
					TSTLOG_appendLog(pHandleLowRes, "FAIL\n");
					TSTLOG_appendLog(pHandleLowRes,"Error in DSPStraem_Reclaim()\n");
				}
                delAllNodes(firstElem);
   		        DSPProcessor_Detach(hProcessor);        
				return retVal;
			}


			/* Issue Empty Buffer */
			status = DSPStream_Issue(hStreamIn, arrBufIn[0], iBuffSize, iBuffSize, 0);
			TST_verbose("DSPStream_Issue %d ret val = %x\n",i,status);

		    if (!DSP_SUCCEEDED(status)) 
			{
		        /* Report Error */
		        TST_verbose("FAIL\n");
	
				if (bUseLogLowRes) 
				{
					TSTLOG_appendLog(pHandleLowRes, "FAIL\n");
					TSTLOG_appendLog(pHandleLowRes,"Error in DSPStraem_Issue()\n");
				}
                delAllNodes(firstElem);
   		        DSPProcessor_Detach(hProcessor);
				return retVal;
			}

			/* Reclaim Fill Buffer */
			status = DSPStream_Reclaim(hStreamIn, arrBufIn, &tmpDataSize, &tmpBuffSize, &tmpArg);
			TST_verbose("DSPStream_Reclaim %d ret val = %x\n",i,status);

		    if (!DSP_SUCCEEDED(status)) 
			{
		        /* Report Error */
		        TST_verbose("FAIL\n");
	
				if (bUseLogLowRes) 
				{
					TSTLOG_appendLog(pHandleLowRes, "FAIL\n");
					TSTLOG_appendLog(pHandleLowRes,"Error in DSPStraem_Reclaim()\n");
				}
                delAllNodes(firstElem);
   		        DSPProcessor_Detach(hProcessor);
				return retVal;
			}

			/* Check for validity of Data */

	    } // End of For loop - NUMTRANSFERS

	status = DSPStream_FreeBuffers(hStreamIn, arrBufIn, 1);
	status = DSPStream_FreeBuffers(hStreamOut, arrBufOut, 1);

    status = DSPStream_Close(hStreamOut);
	status = DSPStream_Close(hStreamIn);
    status = DSPNode_Terminate(hNode, &exitStatus);
    status = DSPNode_Delete(hNode);

    DSPProcessor_Detach(hProcessor);

    OsSpecificMemFree(pAttrIn);
    OsSpecificMemFree(arrBufOut);
    OsSpecificMemFree(arrBufIn);

	retVal = PASS;
	return retVal;
}
/* -------------------------------------------------- */


/*
 *  ======== main ========
 *
 */
INT lowResource_STRESS(INT argc, CHAR * argv[])
{
	INT            			iChunkSize = 0;
    INT            			iNumIterations = 0;
	

    INT            			iBuffSize = DEFBUFSIZE;
	CHAR           			szLogName[DEFNAMESIZE];
	INT            			iMandArgCount = NUMOFCOMPARGS;
	DSP_STATUS     			status;
    CHAR					*aArgv[DEFNAMESIZE] = { DSPEXEC, "" };
	INT                     iNodeCount;
	INT					    opt;
	


	/* Check Number of Arguments */
	/* Process arguments and Store the values */
	/* Add elem to linked list to increase mem consumption */
	/* Check Bridge Status by Loading the Image on DSP and
	 * doing a sample data transfer */
	/* Continue the above two steps until the Bridge can not
	 * do any more data transfers or till the specified 
	 * number of iterations has reached */
	/* End !! */
	/* Foll is the function to test the Bridge Status */
		/* Load the Image on DSP */
		/* Allocate Buffers for Transfer of Data */
		/* Set the Properties, Allocate and Create the Node on DSP */
		/* Open the Streams */
		/* Run the task */
		/* Check for Data Transfer */



	iNodeCount = 0; /* Initialise */

    TST_verbose("Starting Low Resource Tests\n");

	/* Process arguments and Store the values */
    optind = 0;
	opt = getopt_test(argc, (char * const)argv, "+vl:x:c:n:s:");
    while (opt != EOF) {
        switch (opt) {
        case 'v':
            /* verbose */
            TST_setVerbose(TRUE);
            TST_verbose("DelayTransfer_STRESS: Verbose Mode On\n");
            break;
        case 'l':
            /* use log and store with name location */
            strncpy(szLogName, optarg, sizeof(szLogName));
            bUseLogLowRes = TRUE;
            TST_verbose("Opening %s\n", szLogName);
            pHandleLowRes = TSTLOG_openLog(szLogName);
            TSTLOG_appendLog(pHandleLowRes, "DSPStream_Issue:");
            TST_verbose("Log opened as: %s\n", szLogName);
            break;
        case 'x':
            /* use given executable */
            aArgv[0] = (char *) optarg;
            fLoadDsp = TRUE;
            break;
		case 'c':
			/* Chunk size to be consumed each iteration(bytes) */
			iChunkSize = CSL_Atoi(optarg);
			iMandArgCount--;
			TST_verbose("Chunk size = %d\n",iChunkSize);
            TSTLOG_appendLog(pHandleLowRes, "Chunk size = %d\n",iChunkSize);
			break;
		case 'n':
			/* Maximum number of Iterations, after which the test is stopped */
			iNumIterations = CSL_Atoi(optarg);
			iMandArgCount--;
			TST_verbose("Number of Iterations = %d\n",iNumIterations);
            TSTLOG_appendLog(pHandleLowRes, "Number of Iterations = %d\n",iNumIterations);
			break;
		case 's':
			/* Buffer Size */
			iBuffSize = CSL_Atoi(optarg);
			iMandArgCount--;

			TST_verbose("Buffer Size = %d\n",iBuffSize);
            TSTLOG_appendLog(pHandleLowRes, "Buffer Size = %d\n",iBuffSize);
			break;
        default:
            TSTLOG_appendLog(pHandleLowRes, "Invalid option: %c\n", opt);

            if (bUseLogLowRes) {
                TSTLOG_appendLog(pHandleLowRes,
                "Invalid option specified.\n");
            }
            break;
        }
		opt = getopt_test(argc, (char* const)argv, "+vl:x:c:n:s:");
    }

	/* If all the Mandatory args are not present, abort */
	if (iMandArgCount != 0)
	{
		// Error
		TST_verbose("Number of Mandatory Arguments is incorrect.\n");
		TST_verbose("Some args may be missing FAIL \n");
		return 0;
	}


	while (1)
	{
		status = addNodeToList(iChunkSize);
		if(status != PASS)
		{
			/* failure */
			TST_verbose("Could not add any more nodes to Link List!\n");
			TST_verbose("Calling Delete All nodes\n");
            delAllNodes(firstElem);
            TST_verbose("Deleted all nodes\n");
		}
		status = checkBridgeStatus(aArgv, iBuffSize);
		if(status == FAILURE)
		{
			TST_verbose("checkBridgeStatus FAILED\n");
			TSTLOG_appendLog(pHandleLowRes, "checkBridgeStatus FAILED\n");
			return 0;
		}

		iNodeCount++;
        TST_verbose("iNodeCount = %d\n", iNodeCount);

		/* After every 50 iterations, print a message */
		if((iNodeCount % 50) == 0)
		{
			TST_verbose("Num of Nodes till now = %d\n",iNodeCount);
			TSTLOG_appendLog(pHandleLowRes, "Num of Nodes till now = %d\n",iNodeCount);
		}

		if (iNodeCount >= iNumIterations)
		{
			TST_verbose("Reached the Maximum number of Iterations.Stop now.\n");
			TSTLOG_appendLog(pHandleLowRes, "Reached the Maximum number of Iterations.Stop now.\n");
            return 0;
		}
	}



return 0;
}

