/*
 *  Copyright 2004 by Texas Instruments Incorporated.
 *  All rights reserved. Property of Texas Instruments Incorporated.
 *  Restricted rights to use, duplicate or disclose this code are
 *  granted through contract.
 *
 *  @(#) DSP/BIOS Bridge 
 */
/*  ============================================================================
 *   File    MultiThreadDataQueue.c
 *
 *   Path    $(TESTSUITEROOT)\Stress\MultiThread\MultiThreadDataQueue.c
 *
 *   Desc:
 *           This test case Queues multiple data buffers
 *           for Issue and Reclaims them.
 *
 *  Usage:
 *        MULTHRDDATAQ [-v] [-l "log path/filename"]
 *                      [-x "valid executable"] ..
 *
 *        -v: verbose mode
 *        -l: use log and location
 *        -x: DSP Image to be used
 *        -t: Number of Threads to be created
 *        -n: Number of Transfers for each Delay
 *        -s: Buffer Size
 *
 *   ---------------------------------------------------------------------------
 *   Rev     1.0.0
 *
 *   This computer program is copyright to Texas Instruments Incorporated.
 *   The program may not be used without the written permission
 *   of Texas Instruments Incorporated. or against the terms and conditions
 *   stipulated in the agreement under which this program has been
 *   supplied.
 *
 *   (c) Texas Instruments Incorporated 2003
 *
 *   ============================================================================
 *   Revision History
 *
 *   AUG 20, 2002    REF=ORG             Swamy MC
 *      Original Version.
 *   ============================================================================
 */


#include <stdio.h>
#include <dbapi.h>
#include <getopt.h>
#include <tst.h>
#include <tstlog.h>
#include <uuidutil.h>
#include <testosal.h>
#include <semaphore.h>
#include <DSPProcessor_OEM.h>

#include "MultiThread.h"

#define DEFNAMESIZE      100
#define DEFBUFSIZE        64
#define NUMOFCOMPARGS      4	/* Compulsory args: t,n,s,q */
#define MAXNUMOFTHREADS    10    /* This number taken from Symbian Test Suite */
#define PASS               0
#define FAIL               1
#define MAXNUMOFBUFFERS    6    /* Depends on the CDB file for the DSP executable */
#define DEFAULTDSPUNIT     0
#define SHORT short
#ifdef OMAP_2430
#define DSPEXEC        "mtque_nortdx_ext_2430.dof64P"
#else
#define DSPEXEC        "mtque_nortdx_int_24xx.dof55L"
#endif

static sem_t semCreate = { {0,0}, 1 };

/* Function to exit gracefully */
static INT graceful_exit()
{
	/*printf("FATAL ERROR: Exiting the Test Case !\n");
	exit(0);*/
}

/* Thread function for Multi thread data transfer test case */
/*
 * Note: Address of args is being passed from the main thread,		*
 * dont alter the contents as this will be used/needed by all the	*
 * threads created. Copy the contents to local variables and use	*
 */
static INT threadFxn(MultiThreadArgs *targs)
{

    	struct DSP_STRMATTR			attrs;
	BYTE					**arrOutBuf;
	BYTE					**arrInBuf;

    	struct DSP_NODEATTRIN			nodeAttrin;
    	DSP_HNODE				hNode = NULL;
	DSP_HSTREAM				hOutStream;

	CHAR					*pszUuid;
	struct DSP_UUID				gidNodeID;
    struct	DSP_CBDATA				*cbData = NULL;
	DSP_STATUS				status;
	INT						i,j;
	DSP_STATUS				exitStatus = DSP_SOK;
	INT                     iQueue;
	ULONG                   dwBufSize1;
	ULONG                   dwBufSize2;
	DWORD                   dwArg = 0;

	/* Processor related information */
	DSP_HPROCESSOR			hProcessor;

	/* Test case specific information */
	INT						iNumTransfers;
	INT						iBuffSize;

	/* Logging related information */
	FILE					*pHandle;
	BOOL					bUseLog=FALSE;
	BOOL					bVerbose=FALSE;
	struct DSP_STREAMATTRIN        strmAttrs;
	INT			threadID ;
	SHORT *			pData ;

	///printf("threadFxn: Entered the Thread Function\n");

	/* Copy the contents of passed args to local vars */
	/* This strategy can be changed if the variables are just
	   used/read and not modified. */
	hProcessor = targs->hProcessor;
	iNumTransfers = targs->iNumTransfers;
	iBuffSize = targs->iBuffSize;
	pHandle = targs->pHandle;
	bUseLog = targs->bUseLog;
	bVerbose = targs->bVerbose;
	iQueue = targs->iQueue;
	threadID = targs->ithrdNum ;
	sem_post(&semCreate);

    /* Initialising attributes */
    attrs.uSegid = 0;
    attrs.uBufsize = iBuffSize;
    attrs.lMode = targs->transferMode;
    attrs.uNumBufs = iQueue;
    attrs.uAlignment = 0;
    attrs.uTimeout = 5000;

    if (attrs.lMode == STRMMODE_ZEROCOPY)
    {
            attrs.uSegid = DSP_SHMSEG0;  /* Use SM for zero-copy */
    }

    nodeAttrin.iPriority = 5 ;
    nodeAttrin.uTimeout = 10000 ;

#if defined(OMAP_2430)|| defined(OMAP_3430)
    nodeAttrin.uProfileID = 0xff;
#endif

	arrOutBuf = OsSpecificMemAlloc(sizeof(BYTE *) * iQueue);
	arrInBuf = OsSpecificMemAlloc(sizeof(BYTE *) * iQueue);


	pszUuid = "F95CA076_EE7D_11D5_A8DD_00B0D055F6D1" ;  // taken from inout tcf file
    UUID_UuidFromString((CHAR *)pszUuid, &gidNodeID);

    /* Allocate Node */
    status = DSPNode_Allocate(hProcessor, &gidNodeID, cbData, &nodeAttrin,
                   &hNode);

    if (!DSP_SUCCEEDED(status))
	{
        /* Report Error */
        TST_verbose("FAIL\n");
        if (bUseLog)
		{
            TSTLOG_appendLog(pHandle, "FAIL\n");
            TSTLOG_appendLog(pHandle,"Error in DSPNode_Allocate(). and the Error is 0x%x\n",status);
        }
        return FAIL;
    }

    status = DSPNode_Connect((DSP_HNODE)DSP_HGPPNODE, 0, hNode, 0,
                             &attrs);

    if (!DSP_SUCCEEDED(status))
	{
        /* Report Error */
        TST_verbose("FAIL\n");

        if (bUseLog)
		{
            TSTLOG_appendLog(pHandle, "FAIL\n");
            TSTLOG_appendLog(pHandle,"Error in DSPNode_Connect() and the Error is 0x%x\n",status);
        }
        return FAIL;
    }

    status = DSPNode_Connect(hNode, 0, (DSP_HNODE)DSP_HGPPNODE, 0,
                             &attrs);

    if (!DSP_SUCCEEDED(status))
	{
        /* Report Error */
        TST_verbose("FAIL\n");

        if (bUseLog)
		{
            TSTLOG_appendLog(pHandle, "FAIL\n");
            TSTLOG_appendLog(pHandle,"Error in DSPNode_Connect() and the Error is 0x%x\n",status);
        }
        return FAIL;
    }


    /* Create Node on DSP */
    status = DSPNode_Create(hNode);

    if (!DSP_SUCCEEDED(status))
	{
        /* Report Error */
        TST_verbose("FAIL\n");

        if (bUseLog)
		{
            TSTLOG_appendLog(pHandle, "FAIL\n");
            TSTLOG_appendLog(pHandle,"Error in DSPNode_Create() and the Error code is 0x%x\n",status);
        }
        return FAIL;
    }

    status = DSPNode_Run(hNode);

    if (!DSP_SUCCEEDED(status))
	{
        /* Report Error */
        TST_verbose("FAIL\n");

        if (bUseLog)
		{
            TSTLOG_appendLog(pHandle, "FAIL\n");
            TSTLOG_appendLog(pHandle,"Error in DSPNode_Run() and the Error Code is 0x%x\n",status);
        }
        return FAIL;
    }

	strmAttrs.cbStruct = sizeof(struct DSP_STREAMATTRIN);
	strmAttrs.uTimeout = 5000;
	strmAttrs.uAlignment = 0;
	strmAttrs.uNumBufs = iQueue;   /* To have 4 buffers in the Queue, for this test case */
	strmAttrs.lMode = targs->transferMode ;
	strmAttrs.uDMAChnlId = 0;
	strmAttrs.uDMAPriority = 0;

	if (strmAttrs.lMode == STRMMODE_PROCCOPY)
    {
      	strmAttrs.uSegment = 0;
    }
    else
    {
	    strmAttrs.uSegment = DSP_SHMSEG0;    /* Use shared memory segment 0 */
    }


    status = DSPStream_Open(hNode, DSP_TONODE, 0, &strmAttrs, &hOutStream);

    if (!DSP_SUCCEEDED(status))
	{
        /* Report Error */
        TST_verbose("FAIL\n");

        if (bUseLog)
		{
            TSTLOG_appendLog(pHandle, "FAIL\n");
            TSTLOG_appendLog(pHandle,"Error in DSPStraem_Open()\n");
        }
        return FAIL;
    }

	status = DSPStream_AllocateBuffers(hOutStream, iBuffSize, arrOutBuf, iQueue);

    if (!DSP_SUCCEEDED(status))
	{
        /* Report Error */
        TST_verbose("FAIL\n");

        if (bUseLog)
		{
            TSTLOG_appendLog(pHandle, "FAIL\n");
            TSTLOG_appendLog(pHandle,"Error in DSPStream_AllocateBuffers()\n");
        }
        return FAIL;
    }

	pData = (SHORT *)arrOutBuf[0] ;
	pData[0] = 0 ;
	pData[1] = iBuffSize ;

    for (i = 2; i < (iBuffSize / 2); i++)
    {
        pData[i] = i ;
    }

	for (j = 0; j < iQueue; j++)
	{
        // Do not use orginal pointers
        arrInBuf[j] = arrOutBuf[j] ;
    }


    /*  Data transfer for number of iterations .. specified by the
     *  user as parameters to the test ... set in the constructor
	 */
    for (i = 0 ; i < iNumTransfers ; i++)
	{
	    for (j = 0; j < iQueue; j++)
		{
			/* Issue Data */
			status = DSPStream_Issue(hOutStream, arrInBuf[j], iBuffSize, iBuffSize, 0);

			if (!DSP_SUCCEEDED(status))
			{
				/* Report Error */
				TST_verbose("FAIL\n");

				if (bUseLog)
				{
					TSTLOG_appendLog(pHandle, "FAIL\n");
					TSTLOG_appendLog(pHandle,"Error in DSPStraem_Issue()\n");
				}
				return FAIL;
			}
		} // end of for "j"

		for (j = 0; j < iQueue; j++)
		{
			/* Reclaim Data */
			status = DSPStream_Reclaim(hOutStream, &arrInBuf[j], &dwBufSize1, &dwBufSize2, &dwArg);

			if (!DSP_SUCCEEDED(status))
			{
				/* Report Error */
				TST_verbose("FAIL\n");

				if (bUseLog)
				{
					TSTLOG_appendLog(pHandle, "FAIL\n");
					TSTLOG_appendLog(pHandle,"Error in DSPStraem_Reclaim()\n");
				}
				return FAIL;
			}
		}
		if ((i % 500) == 0)
		{
				////printf("Thread ID:%d Issue Reclaim: %d\n",threadID,i);
		}

	} // End of For loop - iNumTransfers

	status = DSPStream_FreeBuffers(hOutStream, arrOutBuf, iQueue);

    status = DSPStream_Close(hOutStream);

    status = DSPNode_Terminate(hNode, &exitStatus);
    status = DSPNode_Delete(hNode);

    OsSpecificMemFree(arrOutBuf);
    OsSpecificMemFree(arrInBuf);

	/* Return from this Thread */
	return 0;
}



/*
 *  ======== main ========
 *
 */
INT multiThreadDataQueue_STRESS(INT argc, CHAR * argv[])
{
    	BOOL           			bUseLog=FALSE;
	BOOL           			bVerbose=FALSE;
    	FILE           			*pHandle = NULL;
	INT            			iNumThreads = 0;
    	INT            			iNumTransfers = 0;
    	INT            			iBuffSize = 0;
	INT            			iQueue = 0;
	CHAR           			szLogName[DEFNAMESIZE];
	INT            			iMandArgCount = NUMOFCOMPARGS;
	DSP_STATUS     			status;
	DWORD		   			threadID[MAXNUMOFTHREADS];
	HANDLE		   			thandle[MAXNUMOFTHREADS];
	MultiThreadArgs 		thrdArgs;

    	struct DSP_PROCESSORATTRIN		*pAttrIn;
	DSP_HPROCESSOR			hProcessor;
    	CHAR					*aArgv[DEFNAMESIZE] = { DSPEXEC, "" };
	INT						iArgc = 1;
	INT						opt;
	INT						index=0;
    	INT                 	retVal = PASS;
	INT		                threadStatus = 0;
	INT 					transferMode = STRMMODE_PROCCOPY ;
	BOOL					fLoadDsp = FALSE;


	 sem_init(&semCreate, 0, 1);

	/* Check Number of Arguments */
	/* Process arguments and Store the values */
	/* Load the Image on DSP */
	/* Create requested number of Threads and pass the
	   input params to these Threads */
	/* In each of the Thread do the following: */
		/* Allocate Buffers for Transfer of Data */
		/* Set the Properties, Allocate and Create the Node on DSP */
		/* Open the Streams */
		/* Run the task */
		/* Do the Data Transfer */
	/* End !! */





	/* Process arguments and Store the values */
    optind = 0;
	opt = getopt(argc, argv, "+vl:x:t:n:s:q:RZP");
    while (opt != EOF) {
        switch (opt) {
        case 'v':
            /* verbose */
            TST_setVerbose(TRUE);
            TST_verbose("MultiThreadDataQueueTx_STRESS: Verbose Mode On\n");
            break;
        case 'l':
            /* use log and store with name location */
            strncpy(szLogName, optarg, sizeof(szLogName));
            bUseLog = TRUE;
            TST_verbose("Opening %s\n", szLogName);
            pHandle = TSTLOG_openLog(szLogName);
            TSTLOG_appendLog(pHandle, "MultiThreadDataQueue: Log Opened");
            TST_verbose("Log opened as: %s\n", szLogName);
            break;
        case 'x':
            /* use given executable */
            aArgv[0] = (char *) optarg;
            fLoadDsp = TRUE;
            break;
		case 't':
			/* Number of Threads to be created */
			iNumThreads = CSL_Atoi(optarg);
			iMandArgCount--;
			TST_verbose("Number of Threads to be Created = %d\n",iNumThreads);
            TSTLOG_appendLog(pHandle,"Number of Threads to be Created = %d\n",iNumThreads);
			break;
		case 'n':
			/* Number of Transfers */
			iNumTransfers = CSL_Atoi(optarg);
			iMandArgCount--;
			TST_verbose("Number of transfers set to %d\n",iNumTransfers);
            TSTLOG_appendLog(pHandle, "Number of transfer set to %d",iNumTransfers);
			break;
		case 's':
			/* Buffer Size */
			iBuffSize = CSL_Atoi(optarg);
			iMandArgCount--;
			TST_verbose("Buffer Size set to %d\n",iBuffSize);
            TSTLOG_appendLog(pHandle, "Buffer Size set to %d\n",iBuffSize);
			break;
		case 'q':
			/* Number of Messages to be Queued */
			iQueue = CSL_Atoi(optarg);
			iMandArgCount--;
			if (iQueue > MAXNUMOFBUFFERS)
				iQueue = MAXNUMOFBUFFERS;
			TST_verbose("Num of Buffers to be Qd %d\n",iQueue);
            TSTLOG_appendLog(pHandle, "Num of buffers to be Qd %d\n",iQueue);
			break;
	case 'R':
            transferMode = STRMMODE_RDMA ;
            ///printf("RDMA Transfer Mode\n") ;
            break ;
    case 'Z':
            transferMode = STRMMODE_ZEROCOPY ;
           //// printf("Zero Copy transfer mode\n") ;
            break ;

	case 'P':
		    transferMode = STRMMODE_PROCCOPY ;
	////	    printf("Processor Copy transfer mode\n") ;
			break ;
        default:
            TSTLOG_appendLog(pHandle, "Invalid option: %c\n", opt);

            if (bUseLog) {
                TSTLOG_appendLog(pHandle,
                "Invalid option specified.\n");
            }
            break;
        }
		opt = getopt(argc, argv, "+vl:x:t:n:s:q:RZP:");
    }

	/* If all the Mandatory args are not present, abort */
	if (iMandArgCount != 0)
	{
		// Error
		TST_verbose("Number of Mandatory Arguments is incorrect.\n");
		TST_verbose("Some args may be missing\n");
	}

    TST_verbose("DSP Executable: %s\n", aArgv[0]);


    pAttrIn = OsSpecificMemAlloc(sizeof(struct DSP_PROCESSORATTRIN));
	if (pAttrIn == NULL)
	{
		// Error
		TST_verbose("Malloc for pAttrIn FAILED !!\n");
		graceful_exit();
	}
    pAttrIn->cbStruct = sizeof(struct DSP_PROCESSORATTRIN);
    pAttrIn->uTimeout = 50;

    /* Attach to processor */
    struct DSP_PROCESSORINFO       dspInfo;
    UINT                    numProcs;
    INT                     procId=0;
    
    while (DSP_SUCCEEDED(DSPManager_EnumProcessorInfo(index,&dspInfo, (UINT) sizeof (struct DSP_PROCESSORINFO),&numProcs))) {
        if (dspInfo.uProcessorType == DSPTYPE_55 || dspInfo.uProcessorType == DSPTYPE_64) {
            TSTLOG_appendLog(pHandle,"DSP device detected !! \n");
            procId = index;
            status = DSP_SOK;
            break;
        }
        index++;
    }

    /* Attach to an available DSP (in this case, the 1st DSP). */
    if (DSP_SUCCEEDED(status)) {
        
        status = DSPProcessor_Attach(procId, pAttrIn, &hProcessor);
        if (DSP_SUCCEEDED(status)) {
            TSTLOG_appendLog(pHandle, "DSPProcessor_Attach succeeded.\n");
        }
        else {
            TSTLOG_appendLog(pHandle, "DSPProcessor_Attach failed. Status = 0x%x\n",status);
        }
    }
    else {
        TSTLOG_appendLog(pHandle, "Failed to get the desired processor \n");
    }
    
    if (!DSP_SUCCEEDED(status))
	{
        /* Report Error */
        TST_verbose("FAIL\n");

        if (bUseLog)
		{
            TSTLOG_appendLog(pHandle, "FAIL\n");
            TSTLOG_appendLog(pHandle,
            "Error in DSPProcessor_Attach()\n");
        }
        graceful_exit();
    }

    if(fLoadDsp)
    {
	////printf("BOF name: %s\n",aArgv[0]);
    /* Load default program */
    status = DSPProcessor_Load(hProcessor, iArgc, (CONST CHAR **)aArgv, NULL);
    if (!DSP_SUCCEEDED(status))
	{
		TSTLOG_appendLog(pHandle, "Status Val: %x\n",status);
        /* Report Error */
        TST_verbose("FAIL\n");
        if (bUseLog)
		{
            TSTLOG_appendLog(pHandle, "FAIL\n");
            TSTLOG_appendLog(pHandle,
            "Error in DSPProcessor_Load()\n");
        }
		graceful_exit();
    }

    status = DSPProcessor_Start(hProcessor);
    if (!DSP_SUCCEEDED(status))
	{
        /* Report Error */
        TST_verbose("FAIL\n");
        if (bUseLog)
		{
            TSTLOG_appendLog(pHandle, "FAIL\n");
            TSTLOG_appendLog(pHandle,
            "Error in DSPProcessor_Start()\n");
        }
        graceful_exit();
	    }
    }

	/* Fill the thrdArgs which are to be passed on to the Thread */
	thrdArgs.hProcessor = hProcessor;
	thrdArgs.iNumTransfers = iNumTransfers;
	thrdArgs.iBuffSize = iBuffSize;
	thrdArgs.pHandle = pHandle;
	thrdArgs.bUseLog = bUseLog;
	thrdArgs.bVerbose = bVerbose;
	thrdArgs.iQueue = iQueue;
	thrdArgs.transferMode = transferMode ;

	/* Create requested number of Threads and pass the
	   input params to these Threads */
	for (index = 0; index < iNumThreads; index ++)
	{
		sem_wait(&semCreate);
		thrdArgs.ithrdNum = index ;
		thandle[index] = (PVOID)OsSpecificThreadCreate(&threadFxn, &thrdArgs, &threadID[index]);

		if (thandle[index] == NULL)
		{
			/* ERROR: Could not create Thread ! Exit !! */
			TST_verbose("ERROR: Could not create Thread ! Exit !! ERROR\n");
		}

	}


	// Wait for the threads to finish before doing a Detach.
	// Code needs to be added here !!!
	for (index = 0; index < iNumThreads; index ++)
	{
		status = OsSpecificWaitForThread((PVOID)threadID[index],threadStatus);

		if(status)
		{
			TSTLOG_appendLog(pHandle, "ERROR: Thread %d Failed\n",index);
			retVal = FAIL;
			TST_verbose("Thread failed %d ",thandle[index]) ;
		}

		TST_verbose("Thread return status %d \n",threadStatus) ;
	}

    DSPProcessor_Detach(hProcessor);

    OsSpecificMemFree(pAttrIn);

    if (retVal == PASS)
	{
        /* Report in LOG module */
        if (bUseLog) {
            TSTLOG_appendLog(pHandle, "PASS\n");
        }
        TST_verbose("PASS\n");
    }

    /* Close Log */
    if (bUseLog) {
        TSTLOG_closeLog(pHandle);
    }

return 0;
}

