/*
 *  Copyright 2004 by Texas Instruments Incorporated.
 *  All rights reserved. Property of Texas Instruments Incorporated.
 *  Restricted rights to use, duplicate or disclose this code are
 *  granted through contract.
 *
 *  @(#) DSP/BIOS Bridge 
 */
/*  ============================================================================
 *   File    apitst.c
 *
 *   Path    $(TESTSUITEROOT)\TestAPI\apitst\apitst.c
 * 
 *   Desc:
 *        Automated Environment for running GPP Module API Tests 
 *        "apitst -v" will run all tests verbose.
 *        "apitst -l output.txt" will run all tests and store output to 
 *                   a file called "output.txt"
 *
 *  Usage:
 *        apitst [-v] [-l "log path/filename"] 
 *               [-x DSP executable ]
 *               [-m module to test (or ALL for all) ]
 *               [-n list of API functions to test for 
 *                   particular module or "ALL" for all functions ]
 *               [-# number of times ]
 *
 *        -v: verbose mode
 *        -l: use log and location to store results
 *        -x: DSP executable to be used in testing
 *        -m: individual module to test or ALL for all modules
 *            (Default = ALL)
 *        -n: list of API functions (see below) or ALL for all functions
 *            (Default = ALL)
 *        -#: number of times to run each API function test
 *
 *  API functions for DSPManager:
 *         1: DSPManager_EnumNodeInfo
 *         2: DSPManager_EnumProcessorInfo
 *         3: DSPManager_Open
 *         4: DSPManager_Close
 *		   5: DSPManager_RegisterObject
 *
 *  API functions for DSPProcessor:
 *         1: DSPProcessor_Attach
 *         2: DSPProcessor_Detach
 *         3: DSPProcessor_EnumNodes
 *         4: DSPProcessor_EnumResources
 *         5: DSPProcessor_GetState
 *         6: DSPProcessor_RegisterNotify
 *         7: DSPProcessor_Load
 *         8: DSPProcessor_Start
 *         9: DSPProcessor_Ctrl
 *         a: DSPProcessor_FlushMemory
 *         b: DSPProcessor_InvalidateMemory

 *  API functions for DSPNode:
 *         1: DSPNode_Allocate
 *         2: DSPNode_Connect
 *         3: DSPNode_Create
 *         4: DSPNode_Run
 *         5: DSPNode_GetMessage
 *         6: DSPNode_PutMessage
 *         7: DSPNode_AllocMsgBuf
 *         8: DSPNode_FreeMsgBuf
 *         9: DSPNode_Pause
 *         a: DSPNode_Terminate
 *         b: DSPNode_Delete
 *         c: DSPNode_GetAttr
 *         d: DSPNode_ChangePriority
 *         e: DSPNode_RegisterNotify
 *
 *  API functions for DSPStream:
 *         1: DSPStream_Open
 *         2: DSPStream_GetInfo
 *         3: DSPStream_AllocateBuffers
 *         4: DSPStream_FreeBuffers
 *         5: DSPStream_Issue
 *         6: DSPStream_Reclaim
 *         7: DSPStream_Select
 *         8: DSPStream_RegisterNotify
 *         9: DSPStream_Idle
 *         a: DSPStream_Close 
 *         b: DSPStream_PrepareBuffer 
 *         c: DSPStream_UnprepareBuffer 
 *
 *   ---------------------------------------------------------------------------
 *   Rev     1.0.0
 * 
 *   This computer program is copyright to Texas Instruments Incorporated.
 *   The program may not be used without the written permission
 *   of Texas Instruments Incorporated. or against the terms and conditions
 *   stipulated in the agreement under which this program has been
 *   supplied.
 *
 *   (c) Texas Instruments Incorporated 2003
 *
 *   ============================================================================
 *   Revision History
 *
 *   JUL 03, 2002    REF=ORG             Swamy MC
 *      Original Version(Taken From WinCE API Test Suite)
 *   ============================================================================
 */

#include <stdio.h>
#include <string.h>
#include <dbapi.h>
#include <getopt_test.h>
#include <tst.h>
#include <tstlog.h>
#include <testcsl.h>

#include "DSPNodeTST.h"
#include "DSPStreamTST.h"
#include "DSPManagerTST.h"
#include "DSPProcessorTST.h"

#define DEFNAMESIZE     80
#define SEPARATORS      " ,\t\n"
#define NUMDSPPROCESSOR 11
#define NUMDSPMANAGER   6
#define NUMDSPNODE      15
#define NUMDSPSTREAM    13
#define NUMNODE         5
#define NUMSTRM         8       

/* GPP function declarations */
BOOL tstDSPProcessor(BOOL bVerbose, BOOL bUseLog, BOOL bUseExec, 
                     CHAR *szLogName, CHAR *szDSPExec, CHAR *szFunctions,
                     INT iRunTimes); 
BOOL tstDSPManager(BOOL bVerbose, BOOL bUseLog,BOOL  bUseExec, CHAR *szLogName,CHAR* szDSPExec, CHAR *szFunctions,INT iRunTimes);
BOOL tstDSPNode(BOOL bVerbose, BOOL bUseLog, BOOL bUseExec, CHAR *szLogName, 
                CHAR *szFunctions, CHAR *szDSPExec, INT iRunTimes); 
BOOL tstDSPStream(BOOL bVerbose, BOOL bUseLog, BOOL bUseExec, 
                  CHAR *szLogName, CHAR *szDSPExec, CHAR *szFunctions,
                  INT iRunTimes); 

/* 
 *  ======== apitst_main ======== 
 */
INT apitst_main (INT argcount, CHAR ** argvars)
{
    INT         opt;
    INT         cRetVal = 1;
    INT         iRunTimes = 1;
    BOOL        bUseLog=FALSE;
    BOOL        bUseExec=FALSE;
    BOOL        bVerbose=FALSE;
    CHAR        szLogName[DEFNAMESIZE];
    CHAR        * szModName = "ALL";
    CHAR        * szFunctions = "ALL";
    CHAR        * szDSPExec = "./dataxfr_nortdxidl.dof";

    optind = 0;
    while ((opt = getopt_test(argcount, (char *const)argvars, "+vl:u:x:n:m:#:")) != EOF) {
		TST_verbose("apitst_main: In While Loop!\n");
        switch (opt) {
        case 'v': 
            /* verbose */
            TST_setVerbose(TRUE);
            bVerbose = TRUE;
            TST_verbose("AutoAPItst: \n");
            break;

        case '#':
            /* set run times */
            iRunTimes = CSL_Atoi(optarg);
            TST_verbose("Running each test %i times\n", iRunTimes);
            break;

        case 'l': 
            /* use log and store with name location */
            strncpy(szLogName, optarg, sizeof(szLogName));
            bUseLog = TRUE;
            break;

        case 'x':
            /* use a given executable */
            bUseExec = TRUE;
            szDSPExec = (char *)optarg;
            break;

        case 'm':
            /* Number of modules to test */
            szModName = (char *)optarg;
            
            if (CSL_Strcmp(szModName, "DSPManager") != 0 && 
                CSL_Strcmp(szModName, "DSPProcessor") != 0 &&
                CSL_Strcmp(szModName, "DSPNode") != 0 &&
                CSL_Strcmp(szModName, "DSPStream") != 0 &&
                CSL_Strcmp(szModName, "NODE") != 0 &&
                CSL_Strcmp(szModName, "STRM") != 0) {
                
                TST_verbose("Invalid Module name specified: %s\n", szModName);
            }
            
            break;

        case 'n':
            /* Functions to test */
            if (szModName != "") {
                szFunctions = (char *) optarg;
                TST_verbose("Testing functions: \"%s\"\n",szFunctions);
            }
            
            /* 
             *  If testing ALL modules, then test ALL functions, otherwise
             *  we can have trouble if one module doesn't have same number
             *  of functions as another.  This is forced here.
             */ 
            if (CSL_Strcmp(szModName,"ALL") == 0) {
                szFunctions = "ALL";
            }
            break;

        default:
            TST_verbose(("Invalid option: %c\n"), opt);
            cRetVal = 0;
            break;
        }
    }
    
    if (CSL_Strcmp(szModName,"DSPProcessor") == 0) {
		tstDSPProcessor(bVerbose, bUseLog, bUseExec, szLogName, szDSPExec,
            szFunctions, iRunTimes);
    } 
    
    if (CSL_Strcmp(szModName,"DSPManager") == 0) {
        tstDSPManager(bVerbose, bUseLog, bUseExec,szLogName, szDSPExec,szFunctions, iRunTimes);
    }
    
    if (CSL_Strcmp(szModName,"DSPNode") == 0) {
        tstDSPNode(bVerbose,bUseLog, bUseExec, szLogName, szFunctions,
            szDSPExec, iRunTimes);
    }
    
    if (CSL_Strcmp(szModName,"DSPStream") == 0) {
        tstDSPStream(bVerbose, bUseLog, bUseExec, szLogName,
            szDSPExec, szFunctions, iRunTimes);
    }

	return 0;	
}


/* 
 *  Test DSPProcessor module 
 */
BOOL tstDSPProcessor(BOOL bVerbose, BOOL bUseLog, BOOL bUseExec, 
                     CHAR *szLogName, CHAR *szDSPExec, CHAR *szFunctions,
                     INT iRunTimes) 
{
    CHAR    * argv[DEFNAMESIZE];
    CHAR    token[1];
    UINT    ptr = 0;
    INT     argc = 0;
    INT     i = 0, j = 0, iChoice;
    INT     (*fxnDSPProcessor[NUMDSPPROCESSOR])();
   
    /* Initialize array of function pointers */
    fxnDSPProcessor[0] = *DSPProcessor_Attach_API;
    fxnDSPProcessor[1] = *DSPProcessor_Detach_API;
    fxnDSPProcessor[2] = *DSPProcessor_EnumNodes_API;
    fxnDSPProcessor[3] = *DSPProcessor_GetResourceInfo_API;
    fxnDSPProcessor[4] = *DSPProcessor_GetState_API;
    fxnDSPProcessor[5] = *DSPProcessor_RegisterNotify_API;
    fxnDSPProcessor[6] = *DSPProcessor_Load_API;
    fxnDSPProcessor[7] = *DSPProcessor_Start_API;
    fxnDSPProcessor[8] = *DSPProcessor_Ctrl_API;
    fxnDSPProcessor[9] = *DSPProcessor_FlushMemory_API;
    fxnDSPProcessor[10]= *DSPProcessor_InvalidateMemory_API;

    /* Set up argument array */
    argv[argc++] = "DSPProcessor";

    if (bVerbose) {
        argv[argc++] = "-v";
    }

    if (bUseLog) {
        argv[argc++] = "-l";
        argv[argc++] = szLogName;
    }

    /* 
     * Here, we have given the user the option of specifying an 
     * executable of their choice to run on the DSP for those functions
     * which test loading and running.  If this options is not
     * specified, a default is provided in each indivdual test.
     */
    if (bUseExec) {
        argv[argc++] = "-x";
        argv[argc++] = szDSPExec;
    }

    /* 
     * Determine whether user wishes to test all functions 
     * Otherwise, test particular functions selected 
     */
    if (CSL_Strcmp(szFunctions,"ALL") == 0) {
        for (i = 0; i < NUMDSPPROCESSOR; i++) {
            for (j = 0; j < iRunTimes; j++) {
                ((Fxn) fxnDSPProcessor[i]) (argc,argv,j);
            }
        }
    }
    else {
        token[0] = '\0';
        if (ptr < strlen(szFunctions)) {
           sprintf(token,"%c",szFunctions[ptr]);
           ptr += 2;
        }
        while (token[0] != '\0') {
            iChoice = CSL_Atoi(token);
     		switch(token[0]) {
                case 'a': iChoice = 10; break;
                case 'b': iChoice = 11; break;
                default: break;
            }
			fxnDSPProcessor[0] = *DSPProcessor_Attach_API;
            if (iChoice > 0 && iChoice <= NUMDSPPROCESSOR) {
		
		for (i = 0; i < iRunTimes; i++) {
                    ((Fxn) fxnDSPProcessor[iChoice-1]) (argc,argv,i);
                }
            }
            token[0] = '\0';
            if (ptr < strlen(szFunctions)) {
               sprintf(token,"%c",szFunctions[ptr]);
               ptr += 2;
            }
        }
    }

    return TRUE;
}


/*  
 *  Test DSPManager module 
 */
BOOL tstDSPManager(BOOL bVerbose, BOOL bUseLog,BOOL  bUseExec, CHAR *szLogName,CHAR* szDSPExec, CHAR *szFunctions,INT iRunTimes) 
{
    CHAR    * argv[DEFNAMESIZE];
    CHAR    token[1];
    UINT    ptr = 0;
    INT     argc = 0;
    INT     i = 0, j = 0, iChoice;
    INT     (*fxnDSPManager[NUMDSPMANAGER])();

    /* Initialize array of function pointers */
    fxnDSPManager[0] = *DSPManager_EnumNodeInfo_API;
    fxnDSPManager[1] = *DSPManager_EnumProcessorInfo_API;
	fxnDSPManager[2] = *DSPManager_Open_API;
	fxnDSPManager[3] = *DSPManager_Close_API;
	fxnDSPManager[4] = *DSPManager_RegisterObject_API;
	fxnDSPManager[5] = *DSPManager_UnregisterObject_API;

    TST_verbose("Module: DSPManager\n");

    /* Set up argument array */
    argv[argc++] = "DSPManager";

    if (bVerbose) { 
        argv[argc++] = "-v";
    }

    if (bUseLog) {
        argv[argc++] = "-l";
        argv[argc++] = szLogName;
    }

    if (bUseExec) {
        argv[argc++] = "-x";
        argv[argc++] = szDSPExec;
    }


    /*
     * Determine whether user wishes to test all functions
     * Otherwise, test particular functions selected
     */
    if (CSL_Strcmp(szFunctions,"ALL") == 0) {
        for (i = 0; i < NUMDSPMANAGER; i++) {
            for (j = 0; j < iRunTimes; j++) {
                ((Fxn) fxnDSPManager[i]) (argc,argv,j);
            }
        }
    }
    else {
        token[0] = '\0';
        if (ptr < strlen(szFunctions)) {
           sprintf(token,"%c",szFunctions[ptr]);
           ptr += 2;
        }

        while (token[0] != '\0') {
            iChoice = CSL_Atoi(token);

    fxnDSPManager[0] = *DSPManager_EnumNodeInfo_API;
    fxnDSPManager[1] = *DSPManager_EnumProcessorInfo_API;
	fxnDSPManager[2] = *DSPManager_Open_API;
	fxnDSPManager[3] = *DSPManager_Close_API;
	fxnDSPManager[4] = *DSPManager_RegisterObject_API;
	fxnDSPManager[5] = *DSPManager_UnregisterObject_API;


            if (iChoice > 0 && iChoice <= NUMDSPMANAGER) {
                for (i = 0; i < iRunTimes; i++) {
                    ((Fxn) fxnDSPManager[iChoice-1]) (argc,argv,i);
                }
            }
            token[0] = '\0';
            if (ptr < strlen(szFunctions)) {
               sprintf(token,"%c",szFunctions[ptr]);
               ptr += 2;
            }
        }
    }  

    return TRUE;
}


/* 
 *  Test DSPNode module 
 */
BOOL tstDSPNode(BOOL bVerbose, BOOL bUseLog, BOOL bUseExec,CHAR *szLogName, 
                CHAR *szFunctions, CHAR *szDSPExec, INT iRunTimes) 
{

    CHAR    * argv[DEFNAMESIZE];
    CHAR    token[1];
    INT     argc = 0;
    UINT    ptr = 0;
    INT     i = 0, j = 0, iChoice;
    INT     (*fxnDSPNode[NUMDSPNODE])();

    /* Initialize array of function pointers */

    fxnDSPNode[0] = *DSPNode_Allocate_API;
    fxnDSPNode[1] = *DSPNode_Connect_API;
    fxnDSPNode[2] = *DSPNode_Create_API;
    fxnDSPNode[3] = *DSPNode_Run_API;
    fxnDSPNode[4] = *DSPNode_GetMessage_API;
    fxnDSPNode[5] = *DSPNode_PutMessage_API;
    fxnDSPNode[6] = *DSPNode_AllocMsgBuf_API;
    fxnDSPNode[7] = *DSPNode_FreeMsgBuf_API;
    fxnDSPNode[8] = *DSPNode_Pause_API;
    fxnDSPNode[9] = *DSPNode_Terminate_API;
    fxnDSPNode[10] = *DSPNode_Delete_API;
    fxnDSPNode[11] = *DSPNode_GetAttr_API;
    fxnDSPNode[12] = *DSPNode_ChangePriority_API;
    fxnDSPNode[13] = *DSPNode_RegisterNotify_API;
    fxnDSPNode[14] = *DSPNode_AllocMsgBuf1_API;

    TST_verbose("Module: DSPNode\n");

    /* Set up argument array */
    argv[argc++] = "DSPNode";

    if (bVerbose) {
        argv[argc++] = "-v";
    }

    if (bUseLog) {
        argv[argc++] = "-l";
        argv[argc++] = szLogName;
    }

    /* 
     * Here, we have given the user the option of specifying an 
     * executable of their choice to run on the DSP for those functions
     * which test loading and running.  If this options is not
     * specified, a default is provided in each indivdual test.
     */
    if (bUseExec) {
        argv[argc++] = "-x";
        argv[argc++] = szDSPExec;
    }

    /* Determine user selection to test */
    if (CSL_Strcmp(szFunctions,"ALL") == 0) {
        for (i = 0; i < NUMDSPNODE; i++) {
            for (j = 0; j < iRunTimes; j++) {
                ((Fxn)fxnDSPNode[i])(argc, argv, j);
            }
        }
    }
    else {
        token[0] = '\0';
        if (ptr < strlen(szFunctions)) {
           sprintf(token,"%c", szFunctions[ptr]);
           ptr += 2;
        }

        while (token[0] != '\0') {
            iChoice = CSL_Atoi(token);
            switch(token[0]) {
                case 'a': iChoice = 10; break;
                case 'b': iChoice = 11; break;
                case 'c': iChoice = 12; break;
                case 'd': iChoice = 13; break;
                case 'e': iChoice = 14; break;
				case 'f': iChoice = 15; break;
                default: break;
            }
			fxnDSPNode[0] = *DSPNode_Allocate_API;
            if (iChoice > 0 && iChoice <= NUMDSPNODE) {
                for (i = 0; i < iRunTimes; i++) {
                    ((Fxn)fxnDSPNode[iChoice - 1])(argc, argv, i);
                }
            }
            token[0] = '\0';
            if (ptr < strlen(szFunctions)) {
               sprintf(token,"%c", szFunctions[ptr]);
               ptr += 2;
            }
        }
    }

    return TRUE;
}


/* 
 *  Test DSPStream module 
 */
BOOL tstDSPStream(BOOL bVerbose, BOOL bUseLog, BOOL bUseExec,
                  CHAR *szLogName, CHAR *szDSPExec, CHAR *szFunctions,
                  INT iRunTimes) 
{
    CHAR    * argv[DEFNAMESIZE];
    CHAR    token[1];
    UINT    ptr = 0;
    INT     argc = 0;
    INT     i, j, iChoice;
    INT     (*fxnDSPStream[NUMDSPSTREAM])();

    /* Initialize array of function pointers */

    fxnDSPStream[0] = *DSPStream_Open_API;
    fxnDSPStream[1] = *DSPStream_GetInfo_API;
    fxnDSPStream[2] = *DSPStream_AllocateBuffers_API;
    fxnDSPStream[3] = *DSPStream_FreeBuffers_API;
    fxnDSPStream[4] = *DSPStream_Issue_API;
    fxnDSPStream[5] = *DSPStream_Reclaim_API;
    fxnDSPStream[6] = *DSPStream_Select_API;
    fxnDSPStream[7] = *DSPStream_RegisterNotify_API;
    fxnDSPStream[8] = *DSPStream_Idle_API;
    fxnDSPStream[9] = *DSPStream_Close_API;
    fxnDSPStream[10] = *DSPStream_PrepareBuffer_API;
    fxnDSPStream[11] = *DSPStream_UnprepareBuffer_API;


    TST_verbose("Module: DSPStream\n");

    /* Set up argument array */
    argv[argc++] = "DSPStream";

    if (bVerbose) {
        argv[argc++] = "-v";
    }

    if (bUseLog) {
        argv[argc++] = "-l";
        argv[argc++] = szLogName;
    }

    /* 
     * Here, we have given the user the option of specifying an 
     * executable of their choice to run on the DSP for those functions
     * which test loading and running.  If this options is not
     * specified, a default is provided in each indivdual test.
     */
    if (bUseExec) {
        argv[argc++] = "-x";
        argv[argc++] = szDSPExec;
    }

    /* Determine user selection to test */
    if (CSL_Strcmp(szFunctions,"ALL") == 0) {
        for (i = 0; i < NUMDSPSTREAM; i++) {
            for (j = 0; j < iRunTimes; j++) {
                ((Fxn) fxnDSPStream[i]) (argc,argv,j);
            }
        }
    }
    else {
        token[0] = '\0';
        if (ptr < strlen(szFunctions)) {
            sprintf(token,"%c",szFunctions[ptr]);
            ptr += 2;
        }
        while (token[0] != '\0') {
            iChoice = CSL_Atoi(token);
            switch(token[0]) {
                case 'a': iChoice = 10; break;
                case 'b': iChoice = 11; break;
                case 'c': iChoice = 12; break;
                default: break;
            }

			fxnDSPStream[0] = *DSPStream_Open_API;
			fxnDSPStream[1] = *DSPStream_GetInfo_API;
			fxnDSPStream[2] = *DSPStream_AllocateBuffers_API;
			fxnDSPStream[3] = *DSPStream_FreeBuffers_API;
			fxnDSPStream[4] = *DSPStream_Issue_API;
			fxnDSPStream[5] = *DSPStream_Reclaim_API;
			fxnDSPStream[6] = *DSPStream_Select_API;
			fxnDSPStream[7] = *DSPStream_RegisterNotify_API;
			fxnDSPStream[8] = *DSPStream_Idle_API;
			fxnDSPStream[9] = *DSPStream_Close_API;
			fxnDSPStream[10] = *DSPStream_PrepareBuffer_API;
			fxnDSPStream[11] = *DSPStream_UnprepareBuffer_API;

            if (iChoice > 0 && iChoice <= NUMDSPSTREAM) {
                for (i = 0; i < iRunTimes; i++) {
                    ((Fxn) fxnDSPStream[iChoice-1]) (argc,argv,i);
                }
            }

            token[0] = '\0';
            if (ptr < strlen(szFunctions)) {
               sprintf(token,"%c",szFunctions[ptr]);
               ptr += 2;
            }
        }
    }
 
    return TRUE;
}

