/*  ============================================================================
 *   File    ivaapitst.c
 *
 *   Path    $(TESTSUITEROOT)\TestAPIIVA\ivaapitst\ivaapitst.c
 * 
 *   Desc:
 *        Automated Environment for running GPP Module API Tests 
 *        "apitst -v" will run all tests verbose.
 *        "apitst -l output.txt" will run all tests and store output to 
 *                   a file called "output.txt"
 *
 *  Usage:
 *        apitst [-v] [-l "log path/filename"] 
 *               [-x DSP executable ]
 *               [-m module to test (or ALL for all) ]
 *               [-n list of API functions to test for 
 *                   particular module or "ALL" for all functions ]
 *               [-# number of times ]
 *
 *        -v: verbose mode
 *        -l: use log and location to store results
 *        -x: DSP executable to be used in testing
 *        -m: individual module to test or ALL for all modules
 *            (Default = ALL)
 *        -n: list of API functions (see below) or ALL for all functions
 *            (Default = ALL)
 *        -#: number of times to run each API function test
 *
 *  API functions for DSPManager:
 *         1: DSPManager_EnumNodeInfo
 *         2: DSPManager_EnumProcessorInfo
 *         3: DSPManager_Open
 *         4: DSPManager_Close
 *	   5: DSPManager_RegisterObject
 *
 *  API functions for DSPProcessor:
 *         1: DSPProcessor_Attach
 *         2: DSPProcessor_Detach
 *         3: DSPProcessor_EnumNodes
 *         4: DSPProcessor_EnumResources
 *         5: DSPProcessor_GetState
 *         6: DSPProcessor_RegisterNotify
 *         7: DSPProcessor_Load
 *         8: DSPProcessor_Start
 *         9: DSPProcessor_Ctrl
 *
 *  API functions for DSPNode:
 *         1: DSPNode_Allocate
 *         2: DSPNode_Create
 *         3: DSPNode_Run
 *         4: DSPNode_GetMessage
 *         5: DSPNode_PutMessage
 *         6: DSPNode_AllocMsgBuf
 *         7: DSPNode_FreeMsgBuf
 *         8: DSPNode_Pause
 *         9: DSPNode_Terminate
 *         a: DSPNode_Delete
 *         b: DSPNode_GetAttr
 *         c: DSPNode_RegisterNotify
 *
 *
 *   ---------------------------------------------------------------------------
 *   Rev     1.0.0
 * 
 *   This computer program is copyright to Texas Instruments Incorporated.
 *   The program may not be used without the written permission
 *   of Texas Instruments Incorporated. or against the terms and conditions
 *   stipulated in the agreement under which this program has been
 *   supplied.
 *
 *   (c) Texas Instruments Incorporated 2003
 *
 *   ============================================================================
 *   Revision History
 *
 *   JUL 03, 2002    REF=ORG             Swamy MC
 *   AUG 05, 2004    REF=Based on DSP API tests             Ramesh Gupta G
 *      Original Version(Taken From WinCE API Test Suite)
 *   ============================================================================
 */

#include <stdio.h>
#include <string.h>
#include <dbapi.h>
#include <getopt_test.h>
#include <tst.h>
#include <tstlog.h>
#include <testcsl.h>

#include "IVADSPNodeTST.h"
#include "IVADSPManagerTST.h"
#include "IVADSPProcessorTST.h"

#define DEFNAMESIZE     80
#define SEPARATORS      " ,\t\n"
#define NUMDSPPROCESSOR 9
#define NUMDSPMANAGER   6
#define NUMDSPNODE      15
#define NUMDSPSTREAM    12
#define NUMNODE         5
#define NUMSTRM         8       

/* GPP function declarations */
BOOL ivatstDSPProcessor(BOOL bVerbose, BOOL bUseLog, BOOL bUseExec, 
                     CHAR *szLogName, CHAR *szDSPExec, CHAR *szFunctions,
                     INT iRunTimes); 
BOOL ivatstDSPManager(BOOL bVerbose, BOOL bUseLog, CHAR *szLogName, 
                   CHAR *szFunctions, INT iRunTimes); 
BOOL ivatstDSPNode(BOOL bVerbose, BOOL bUseLog, BOOL bUseExec, CHAR *szLogName, 
                CHAR *szFunctions, CHAR *szDSPExec, INT iRunTimes); 

/* 
 *  ======== apitst_main ======== 
 */
INT ivaapitst_main (INT argcount, CHAR ** argvars)
{
    INT         opt;
    INT         cRetVal = 1;
    INT         iRunTimes = 1;
    BOOL        bUseLog=FALSE;
    BOOL        bUseExec=FALSE;
    BOOL        bVerbose=FALSE;
    CHAR        szLogName[DEFNAMESIZE];
    CHAR        * szModName = "ALL";
    CHAR        * szFunctions = "ALL";
    CHAR        * szDSPExec = "./iva.dof55l";

    optind = 0;
    while ((opt = getopt_test(argcount, (char *const)argvars, "+vl:u:x:n:m:#:")) != EOF) {
		TST_verbose("ivaapitst_main: In While Loop!\n");
        switch (opt) {
        case 'v': 
            /* verbose */
            TST_setVerbose(TRUE);
            bVerbose = TRUE;
            TST_verbose("AutoAPItst: \n");
            break;

        case '#':
            /* set run times */
            iRunTimes = CSL_Atoi(optarg);
            TST_verbose("Running each test %i times\n", iRunTimes);
            break;

        case 'l': 
            /* use log and store with name location */
            strncpy(szLogName, optarg, sizeof(szLogName));
            bUseLog = TRUE;
            break;

        case 'x':
            /* use a given executable */
            bUseExec = TRUE;
            szDSPExec = (char *)optarg;
            break;

        case 'm':
            /* Number of modules to test */
            szModName = (char *)optarg;
            
            if (CSL_Strcmp(szModName, "IVADSPManager") != 0 && 
                CSL_Strcmp(szModName, "IVADSPProcessor") != 0 &&
                CSL_Strcmp(szModName, "IVADSPNode") != 0 &&
                CSL_Strcmp(szModName, "IVANODE") != 0)         {
                
                TST_verbose("Invalid IVA API Module name specified: %s\n", szModName);
            }
            
            break;

        case 'n':
            /* Functions to test */
            if (szModName != "") {
                szFunctions = (char *) optarg;
                TST_verbose("Testing functions: \"%s\"\n",szFunctions);
            }
            
            /* 
             *  If testing ALL modules, then test ALL functions, otherwise
             *  we can have trouble if one module doesn't have same number
             *  of functions as another.  This is forced here.
             */ 
            if (CSL_Strcmp(szModName,"ALL") == 0) {
                szFunctions = "ALL";
            }
            break;

        default:
            TST_verbose(("Invalid option: %c\n"), opt);
            cRetVal = 0;
            break;
        }
    }
    
    if (CSL_Strcmp(szModName,"IVADSPProcessor") == 0) {
		ivatstDSPProcessor(bVerbose, bUseLog, bUseExec, szLogName, szDSPExec,
            szFunctions, iRunTimes);
    } 
    
    if (CSL_Strcmp(szModName,"IVADSPManager") == 0) {
        ivatstDSPManager(bVerbose, bUseLog, szLogName, szFunctions, iRunTimes);
    }
    
    if (CSL_Strcmp(szModName,"IVADSPNode") == 0) {
        ivatstDSPNode(bVerbose,bUseLog, bUseExec, szLogName, szFunctions,
            szDSPExec, iRunTimes);
    }
    

	return 0;	
}


/* 
 *  Test DSPProcessor module 
 */
BOOL ivatstDSPProcessor(BOOL bVerbose, BOOL bUseLog, BOOL bUseExec, 
                     CHAR *szLogName, CHAR *szDSPExec, CHAR *szFunctions,
                     INT iRunTimes) 
{
    CHAR    * argv[DEFNAMESIZE];
    CHAR    token[1];
    UINT    ptr = 0;
    INT     argc = 0;
    INT     i = 0, j = 0, iChoice;
    INT     (*fxnDSPProcessor[NUMDSPPROCESSOR])();
   
    /* Initialize array of function pointers */
    fxnDSPProcessor[0] = *IVADSPProcessor_Attach_API;
    fxnDSPProcessor[1] = *IVADSPProcessor_Detach_API;
    fxnDSPProcessor[2] = *IVADSPProcessor_EnumNodes_API;
    fxnDSPProcessor[3] = *IVADSPProcessor_GetResourceInfo_API;
    fxnDSPProcessor[4] = *IVADSPProcessor_GetState_API;
    fxnDSPProcessor[5] = *IVADSPProcessor_RegisterNotify_API;
    fxnDSPProcessor[6] = *IVADSPProcessor_Load_API;
    fxnDSPProcessor[7] = *IVADSPProcessor_Start_API;
    fxnDSPProcessor[8] = *IVADSPProcessor_Ctrl_API;

    /* Set up argument array */
    argv[argc++] = "IVADSPProcessor";

    if (bVerbose) {
        argv[argc++] = "-v";
    }

    if (bUseLog) {
        argv[argc++] = "-l";
        argv[argc++] = szLogName;
    }

    /* 
     * Here, we have given the user the option of specifying an 
     * executable of their choice to run on the DSP for those functions
     * which test loading and running.  If this options is not
     * specified, a default is provided in each indivdual test.
     */
    if (bUseExec) {
        argv[argc++] = "-x";
        argv[argc++] = szDSPExec;
    }

    /* 
     * Determine whether user wishes to test all functions 
     * Otherwise, test particular functions selected 
     */
    if (CSL_Strcmp(szFunctions,"ALL") == 0) {
        for (i = 0; i < NUMDSPPROCESSOR; i++) {
            for (j = 0; j < iRunTimes; j++) {
                ((Fxn) fxnDSPProcessor[i]) (argc,argv,j);
            }
        }
    }
    else {
        token[0] = '\0';
        if (ptr < strlen(szFunctions)) {
           sprintf(token,"%c",szFunctions[ptr]);
           ptr += 2;
        }
        while (token[0] != '\0') {
            iChoice = CSL_Atoi(token);
            if (iChoice > 0 && iChoice <= NUMDSPPROCESSOR) {
				fxnDSPProcessor[0] = *IVADSPProcessor_Attach_API;

                for (i = 0; i < iRunTimes; i++) {
                    ((Fxn) fxnDSPProcessor[iChoice-1]) (argc,argv,i);
                }
            }
            token[0] = '\0';
            if (ptr < strlen(szFunctions)) {
               sprintf(token,"%c",szFunctions[ptr]);
               ptr += 2;
            }
        }
    }

    return TRUE;
}


/*  
 *  Test DSPManager module 
 */
BOOL ivatstDSPManager(BOOL bVerbose, BOOL bUseLog, CHAR *szLogName, 
                   CHAR *szFunctions,INT iRunTimes) 
{
    CHAR    * argv[DEFNAMESIZE];
    CHAR    token[1];
    UINT    ptr = 0;
    INT     argc = 0;
    INT     i = 0, j = 0, iChoice;
    INT     (*fxnDSPManager[NUMDSPMANAGER])();

    /* Initialize array of function pointers */
        fxnDSPManager[0] = *IVADSPManager_EnumNodeInfo_API;
        fxnDSPManager[1] = *IVADSPManager_EnumProcessorInfo_API;
	fxnDSPManager[2] = *IVADSPManager_Open_API;
	fxnDSPManager[3] = *IVADSPManager_Close_API;

    TST_verbose("Module: IVADSPManager\n");

    /* Set up argument array */
    argv[argc++] = "IVADSPManager";

    if (bVerbose) { 
        argv[argc++] = "-v";
    }

    if (bUseLog) {
        argv[argc++] = "-l";
        argv[argc++] = szLogName;
    }

    /*
     * Determine whether user wishes to test all functions
     * Otherwise, test particular functions selected
     */
    if (CSL_Strcmp(szFunctions,"ALL") == 0) {
        for (i = 0; i < NUMDSPMANAGER; i++) {
            for (j = 0; j < iRunTimes; j++) {
                ((Fxn) fxnDSPManager[i]) (argc,argv,j);
            }
        }
    }
    else {
        token[0] = '\0';
        if (ptr < strlen(szFunctions)) {
           sprintf(token,"%c",szFunctions[ptr]);
           ptr += 2;
        }

        while (token[0] != '\0') {
            iChoice = CSL_Atoi(token);

	fxnDSPManager[0] = *IVADSPManager_EnumNodeInfo_API;
    	fxnDSPManager[1] = *IVADSPManager_EnumProcessorInfo_API;
	fxnDSPManager[2] = *IVADSPManager_Open_API;
	fxnDSPManager[3] = *IVADSPManager_Close_API;


            if (iChoice > 0 && iChoice <= NUMDSPMANAGER) {
                for (i = 0; i < iRunTimes; i++) {
                    ((Fxn) fxnDSPManager[iChoice-1]) (argc,argv,i);
                }
            }
            token[0] = '\0';
            if (ptr < strlen(szFunctions)) {
               sprintf(token,"%c",szFunctions[ptr]);
               ptr += 2;
            }
        }
    }  

    return TRUE;
}


/* 
 *  Test DSPNode module 
 */
BOOL ivatstDSPNode(BOOL bVerbose, BOOL bUseLog, BOOL bUseExec,CHAR *szLogName, 
                CHAR *szFunctions, CHAR *szDSPExec, INT iRunTimes) 
{

    CHAR    * argv[DEFNAMESIZE];
    CHAR    token[1];
    INT     argc = 0;
    UINT    ptr = 0;
    INT     i = 0, j = 0, iChoice;
    INT     (*fxnDSPNode[NUMDSPNODE])();

    /* Initialize array of function pointers */

    fxnDSPNode[0] = *IVADSPNode_Allocate_API;
    fxnDSPNode[1] = *IVADSPNode_Create_API;
    fxnDSPNode[2] = *IVADSPNode_Run_API;
    fxnDSPNode[3] = *IVADSPNode_GetMessage_API;
    fxnDSPNode[4] = *IVADSPNode_PutMessage_API;
    fxnDSPNode[5] = *IVADSPNode_AllocMsgBuf_API;
    fxnDSPNode[6] = *IVADSPNode_FreeMsgBuf_API;
    fxnDSPNode[7] = *IVADSPNode_Pause_API;
    fxnDSPNode[8] = *IVADSPNode_Terminate_API;
    fxnDSPNode[9] = *IVADSPNode_GetAttr_API;
    fxnDSPNode[10] = *IVADSPNode_RegisterNotify_API;
    fxnDSPNode[11] = *IVADSPNode_AllocMsgBuf1_API;

    TST_verbose("Module: IVADSPNode\n");

    /* Set up argument array */
    argv[argc++] = "IVADSPNode";

    if (bVerbose) {
        argv[argc++] = "-v";
    }

    if (bUseLog) {
        argv[argc++] = "-l";
        argv[argc++] = szLogName;
    }

    /* 
     * Here, we have given the user the option of specifying an 
     * executable of their choice to run on the DSP for those functions
     * which test loading and running.  If this options is not
     * specified, a default is provided in each indivdual test.
     */
    if (bUseExec) {
        argv[argc++] = "-x";
        argv[argc++] = szDSPExec;
    }

    /* Determine user selection to test */
    if (CSL_Strcmp(szFunctions,"ALL") == 0) {
        for (i = 0; i < NUMDSPNODE; i++) {
            for (j = 0; j < iRunTimes; j++) {
                ((Fxn)fxnDSPNode[i])(argc, argv, j);
            }
        }
    }
    else {
        token[0] = '\0';
        if (ptr < strlen(szFunctions)) {
           sprintf(token,"%c", szFunctions[ptr]);
           ptr += 2;
        }

        while (token[0] != '\0') {
            iChoice = CSL_Atoi(token);
            switch(token[0]) {
                case 'a': iChoice = 10; break;
                case 'b': iChoice = 11; break;
                case 'c': iChoice = 12; break;
                case 'd': iChoice = 13; break;
                case 'e': iChoice = 14; break;
				case 'f': iChoice = 15; break;
                default: break;
            }
			fxnDSPNode[0] = *IVADSPNode_Allocate_API;
            if (iChoice > 0 && iChoice <= NUMDSPNODE) {
                for (i = 0; i < iRunTimes; i++) {
                    ((Fxn)fxnDSPNode[iChoice - 1])(argc, argv, i);
                }
            }
            token[0] = '\0';
            if (ptr < strlen(szFunctions)) {
               sprintf(token,"%c", szFunctions[ptr]);
               ptr += 2;
            }
        }
    }

    return TRUE;
}

