/*
 *  Copyright 2001-2008 Texas Instruments - http://www.ti.com/
 * 
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */


/*
 *  ======== debuginfo.c ========
 *  Description:
 *      This module provides the debug facility to debug 
 *      the issues in DSP bridge. 
 *  Demonstrates:
 *      It provides the functionality
 *      to get the information such as Process's information stored in
 *      the process context list. Some of the information that
 *      it displays is Process's state, Node, Stream, DMM, and
 *      Heap information. Displays PRCM registers that are specific to
 *      Bridge. Reads and Writes to the required register.
 *     
  *  Requires:
 *     Install the Samples
 *  Usage:
 *      Run the command ./debuginfo.out with a specific argument
 *      
 *  Notes:
 *
 */


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <dbapi.h>		/* DSP/BIOS Bridge APIs                  */
#include <fcntl.h>
#include <sys/mman.h>
#include <unistd.h>
#include <ctype.h>

#define MAP_SIZE 4096UL
#define MAP_MASK (MAP_SIZE - 1)
#define OMAP_IVA2_PRM_BASE 0x48306000
#define OMAP_IVA2_CM_BASE 0x48004000
#define OMAP_GRP_SEL_BASE 0x48307000
static DSP_STATUS ProcessArgs(int argc, char **argv);
static DSP_STATUS printProcInfo();
static DSP_STATUS bridge_prcm_registers_dump();
static DSP_STATUS dbg_Reg_read(UINT reg, UINT dataBitSize);
static DSP_STATUS dbg_Reg_write(UINT reg, UINT dataBitSize, UINT value);
static DSP_STATUS dbg_address_dump(UINT target, UINT numOfWords);
DSP_STATUS hexStringtoInteger(const char* hexString, UINT* result);

/*
 *  ======== main ========
 */
int main(int argc, char **argv)
{
	DSP_STATUS status = DSP_SOK;
	
	ProcessArgs(argc, argv);	
	return status;
}


/*
 *  ======== ProcessArgs ========
 *  Process command line arguments for this sample
 */
static DSP_STATUS ProcessArgs(int argc, char **argv)
{
	
	INT commandType;
	UINT regAddress;
	INT regValue;
	DSP_STATUS status = DSP_SOK;

	if (argc <2){
		printf("Usage commands: ./debuginfo.out <arg> \r\n<arg> values are"
			  " d for proc info, r for Register read, w for Register write, "
			  "p for PRCM reg dump\n");
		return DSP_EFAIL;
	}
	commandType = argv[1][0];
	switch(commandType){
	case 'p':
		bridge_prcm_registers_dump();
		break;
	case 'd':
		printProcInfo();
		break;
	case 'a':
		if (argc < 4)
			printf("Usage: <r> <Hex registerAddress without 0x prefix>"
			" <numOfWordsToDump>\n");
		else {
			hexStringtoInteger(argv[2],&regAddress);
			status = dbg_address_dump(regAddress,atoi(argv[3]));
		}
		break;
	case 'r':
		if (argc < 4)
			printf("Usage: <r> <Hex registerAddress without 0x prefix>"
			" <regAccessSize =8, 16, 32>\n");
		else {
			hexStringtoInteger(argv[2],&regAddress);
			status = dbg_Reg_read(regAddress, atoi(argv[3]));
		}
		break;
	case 'w':
		if (argc < 5)
			printf("Usage: <w> <Hex registerAddress without 0x prefix>"
			" <regAccessSize =8, 16, 32> <value>\n ");
		else {
			hexStringtoInteger(argv[2],&regAddress);
			hexStringtoInteger(argv[4],&regValue);
			status = dbg_Reg_write(regAddress, atoi(argv[3]),regValue);
		}
		break;		
	default:
		printf("Usage commands: ./debuginfo.out <arg> \r\n<arg> values are"
			  " d for proc info, r for Register read, w for Register write, "
			  "p for PRCM reg dump\n");
		break;
	}
	return (status);
}

/*
 *  This function reads the and prints the requested register value.
 */
static DSP_STATUS printProcInfo()
{
	DSP_STATUS status = DSP_SOK;
	UINT             *pBuf = NULL;
       UINT             uSize = 0;	  
	   
#ifndef RES_CLEANUP_DISABLE
       status = DspManager_Open (0, NULL); 
	pBuf = (void *)malloc(8092); 
	status = DSPManager_GetProcResourceInfo(pBuf,&uSize);
	printf("\n=====End of Processes information======\n");
	free(pBuf);		
	status = DspManager_Close(0,NULL) ;
#else
	printf("Warning:Resource Cleanup is not enabled in the bridge , Build"
	                        "bridge without RES_CLEANUP_DISABLE flag !! \n");
#endif	 
	return status;
}

/*
 *  This function reads the and prints the requested register value.
 */
static DSP_STATUS dbg_Reg_read(UINT target, UINT dataBitSize)
{
 	UINT  readVal =0;
	DSP_STATUS status = DSP_SOK;
	INT fd;
    	void *map_base; 
	void *virt_addr;

	fd = open ("/dev/mem", O_RDWR|O_SYNC); 
	if (fd) {
		map_base = mmap(0, MAP_SIZE, PROT_READ | PROT_WRITE, MAP_SHARED, fd,
													    target & ~MAP_MASK);
		if(map_base == (void *) -1) {
			printf("Failed to do memory mapping \n");
			return DSP_EFAIL;
		}
	}else{
		printf("Failed opening /dev/mem file\n");
		return DSP_EFOPEN;
	}
	virt_addr = map_base + (target & MAP_MASK);

	switch (dataBitSize){ 
	case 32:
	   	 readVal = (*(volatile UINT *)(virt_addr)); 
		break;
	case 16:
	    	readVal = (*(volatile USHORT *)(virt_addr));  
	    break;
	case 8:
	    	readVal = (*(volatile UCHAR *)(virt_addr));    
	    	break;
	default:
		printf("ILLEGAL dataBitSize: Enter 8, 16, or 32 bits \n");
		return DSP_EFAIL;
	}
	printf(" The value of register 0x%x is 0x%x\n", target, readVal); 
	if (munmap(map_base,MAP_SIZE) == -1)
		printf("Memory Unmap failed \n");
	close(fd);
	return status;
}

/*
 *  This function writes to the requested register value.
 */
static DSP_STATUS dbg_Reg_write(UINT target, UINT dataBitSize, UINT value)
{
	UINT  readVal =0;
	DSP_STATUS status = DSP_SOK;
	INT fd;
    	void *map_base; 
	void *virt_addr;

	fd = open ("/dev/mem", O_RDWR|O_SYNC); 
	if (fd) {
		map_base = mmap(0, MAP_SIZE, PROT_READ | PROT_WRITE, MAP_SHARED, fd,
														target & ~MAP_MASK);
		if(map_base == (void *) -1) {
			printf("Failed to do memory mapping \n");
			return DSP_EFAIL;
		}
	}else{
		printf("Failed opening /dev/mem file\n");
		return DSP_EFOPEN;
	}
	virt_addr = map_base + (target & MAP_MASK);

	switch (dataBitSize){ 
	case 32:
	   	(*(volatile UINT *)(virt_addr)) = value; 
		readVal = (*(volatile UINT *)(virt_addr)); 
		break;
	case 16:
	    	(*(volatile USHORT *)(virt_addr)) = (USHORT)value;  
		readVal = (*(volatile USHORT *)(virt_addr));  
	    break;
	case 8:
	    	(*(volatile UCHAR *)(virt_addr)) = (UCHAR)value;  
		readVal = (*(volatile UCHAR *)(virt_addr));  
	    	break;
	default:
		printf("ILLEGAL dataBitSize: Enter 8, 16, or 32 bits \n");
		return DSP_EFAIL;
	}
	printf(" The value of register 0x%x is 0x%x\n", target, readVal); 
	if (munmap(map_base,MAP_SIZE) == -1)
		printf("Memory Unmap failed \n");
	close(fd);
	return status;
}

/*
 *  This function dumps the prcm registers that are Bridge specific
 */
static DSP_STATUS bridge_prcm_registers_dump(void)
{
	INT fd;
	UINT prm_base = 0;
	UINT cm_base = 0;
	UINT grp_base = 0;
	UINT regVal;
	DSP_STATUS status = DSP_SOK;

    	fd = open("/dev/mem", O_RDWR|O_SYNC);
	if (fd) {
	    	prm_base = (UINT) (CHAR *)mmap(0, MAP_SIZE, PROT_READ | PROT_WRITE,
	            					  MAP_SHARED, fd, (UINT)OMAP_IVA2_PRM_BASE);
	    	cm_base = (UINT) (CHAR *)mmap(0,MAP_SIZE, PROT_READ | PROT_WRITE,
	            					  MAP_SHARED, fd, (UINT)OMAP_IVA2_CM_BASE);
	    	grp_base = (UINT) (CHAR *)mmap(0,MAP_SIZE, PROT_READ | PROT_WRITE,
	            					  MAP_SHARED, fd, (UINT)OMAP_GRP_SEL_BASE);
	    	close(fd);				
	}
	else {
	    	fprintf(stdout,"Failed to open the mmap device \n");
		return DSP_EFAIL;
	}

	if (prm_base && cm_base) {
		fprintf(stdout,"\n******* REG DUMP ********************* \n");
	        regVal = *((UINT*)(cm_base + 0x00));
		fprintf(stdout, "CM_FCLKEN_IVA2(0x%x) = 0x%x \n",
				  (OMAP_IVA2_CM_BASE + 0x00),regVal);
	       regVal = *((UINT*)(cm_base + 0x04));
		fprintf(stdout, "CM_CLKEN_PLL_IVA2(0x%x) = 0x%x \n",
				 (OMAP_IVA2_CM_BASE + 0x04),regVal);
	       regVal = *((UINT*)(cm_base + 0x10));
		fprintf(stdout, "CM_ICLKEN1_IVA2(0x%x) = 0x%x \n",
				(OMAP_IVA2_CM_BASE + 0x10),regVal);
		regVal = *((UINT*)(cm_base + 0x20));
		fprintf(stdout, "CM_IDLEST_IVA2(0x%x) = 0x%x \n",
				(OMAP_IVA2_CM_BASE + 0x20),regVal);
	        regVal = *((UINT*)(cm_base + 0x24));
		fprintf(stdout, "CM_IDLEST_PLL_IVA2(0x%x) = 0x%x \n",
				(OMAP_IVA2_CM_BASE + 0x24),regVal);
	        regVal = *((UINT*)(cm_base + 0x34));
		fprintf(stdout, "CM_AUTOIDLE_PLL_IVA2(0x%x) = 0x%x \n",
				(OMAP_IVA2_CM_BASE + 0x34),regVal);
	        regVal = *((UINT*)(cm_base + 0x40));
		fprintf(stdout, "CM_CLKSEL1_PLL_IVA2(0x%x) = 0x%x \n",
				(OMAP_IVA2_CM_BASE + 0x40),regVal);
	        regVal = *((UINT*)(cm_base + 0x48));
		fprintf(stdout, "CM_CLKSTCTRL_IVA2(0x%x) = 0x%x \n",
				(OMAP_IVA2_CM_BASE + 0x48),regVal);
	        regVal = *((UINT*)(cm_base + 0x4c));
		fprintf(stdout, "CM_CLKSTST_IVA2(0x%x) = 0x%x \n",
				(OMAP_IVA2_CM_BASE + 0x4c),regVal);
	        regVal = *((UINT*)(prm_base + 0x50));
		fprintf(stdout, "RM_RSTCTRL_IVA2(0x%x) = 0x%x \n",
				(OMAP_IVA2_PRM_BASE + 0x50),regVal);
	        regVal = *((UINT*)(prm_base + 0x58));
		fprintf(stdout, "RM_RSTST_IVA2(0x%x) = 0x%x \n",
				(OMAP_IVA2_PRM_BASE + 0x58),regVal);
	        regVal = *((UINT*)(prm_base + 0xe0));
		fprintf(stdout, "PM_PWSTCTRL_IVA2(0x%x) = 0x%x \n",
				(OMAP_IVA2_PRM_BASE + 0xe0),regVal);
	        regVal = *((UINT*)(prm_base + 0xe4));
		fprintf(stdout, "PM_PWSTST_IVA2(0x%x) = 0x%x \n",
				(OMAP_IVA2_PRM_BASE + 0xe4),regVal);
	        regVal = *((UINT*)(cm_base + 0xA10));
		fprintf(stdout, "CM_ICLKEN1_CORE(0x%x) = 0x%x \n",
				(OMAP_IVA2_PRM_BASE + 0xa10),regVal);
	        regVal = *((UINT*)(grp_base + 0xA4));
		fprintf(stdout, "PM_MPUGRPSEL_PER(0x%x) = 0x%x \n",
				(OMAP_GRP_SEL_BASE + 0xa4),regVal);
	        regVal = *((UINT*)(grp_base + 0xA8));
		fprintf(stdout, "PM_IVA2GRPSEL_PER(0x%x) = 0x%x \n",
				(OMAP_GRP_SEL_BASE + 0xa8),regVal);
		regVal = *((UINT*)(prm_base + 0xf8));
		fprintf(stdout, "PRM_IRQSTATUS_IVA2(0x%x) = 0x%x \n",
				(OMAP_IVA2_PRM_BASE + 0xf8),regVal);
    	}
	if (munmap((void*)prm_base,MAP_SIZE) == -1)
		printf("prcm Memory Unmap failed \n");
	if (munmap((void*)cm_base,MAP_SIZE) == -1)
		printf("prcm Memory Unmap failed \n");

	return status;
}

/*
 *  This function dumps the requested number of words
 */
static DSP_STATUS dbg_address_dump(UINT target, UINT numOfWords)
{
	UINT  readVal =0;
	DSP_STATUS status = DSP_SOK;
	INT fd;
    	UINT map_base; 
	UINT virt_addr;

	fd = open ("/dev/mem", O_RDWR|O_SYNC); 
	if (fd) {
		map_base = (UINT)(UINT*)mmap(0, MAP_SIZE, PROT_READ | PROT_WRITE, 
										MAP_SHARED, fd, target & ~MAP_MASK);
		if(map_base == (void *) -1) {
			printf("Failed to do memory mapping \n");
			return DSP_EFAIL;
		}
	}else{
		printf("Failed opening /dev/mem file\n");
		return DSP_EFOPEN;
	}
	virt_addr = map_base + (target & MAP_MASK);
	printf(" Address    Value\n"); 
	printf("________________________\n"); 

	readVal = (*(volatile UINT *)(virt_addr)); 
	while(numOfWords != 0)
	{
		readVal = *(volatile UINT *)(virt_addr); 
		printf(" 0x%x....... 0x%x\n", target, readVal); 
		virt_addr= virt_addr+4;
		target = target+4;
		numOfWords--;		
	}
	if (munmap(map_base,MAP_SIZE) == -1)
		printf("Memory Unmap failed \n");
	close(fd);
	return status;
}
/*
 *  This function converst a hex string to Integer
 */
DSP_STATUS hexStringtoInteger(const char* hexString, UINT* result)
{
 	INT i;
	INT temp;
	INT factor;
	INT length;
	DSP_STATUS status = DSP_SOK;
	
	length = strlen(hexString);
 	if ((length == 0) || (length>8)) 
		return DSP_EFAIL;
	*result = 0;
  	factor = 1;
	/* Convert the characters to hex number */
	for(i=length-1; i>=0 ;i--)
	{
		if (isxdigit(*(hexString+i))) {
			if (*(hexString+i)>=97){
				temp = ( *(hexString+i) - 97) + 10;
			}else if ( *(hexString+i) >= 65){
			 	temp = (*(hexString+i) - 65) + 10;
			}else{
			 	temp = *(hexString+i) - 48;
			}
		*result += (temp * factor);
		factor *= 16;
		}else{
	        	/* Conversion was abnormally terminated by 
	        	     non hexadecimal digit */
			return DSP_EFAIL;
	 	}
	}
 	return status;
}

